@@ -16,7 +16,7 @@ namespace at {
     const auto num_arguments = arguments.size();
     const auto stack_start = stack->size() - num_arguments;
 
-    c10::optional<bool> is_write;
+    std::optional<bool> is_write;
     for (const auto i : c10::irange(num_arguments)) {
       const auto& alias_info = arguments[i].alias_info();
       if (alias_info != nullptr) {
