@@ -3,6 +3,7 @@
 
 using namespace c10;
 
+// NOLINTBEGIN(performance-move-const-arg, bugprone-use-after-move, *analyzer*Move)
 TEST(ListTestIValueBasedList, givenEmptyList_whenCallingEmpty_thenReturnsTrue) {
     List<string> list;
     EXPECT_TRUE(list.empty());
@@ -37,7 +38,6 @@ TEST(ListTestIValueBasedList, whenCallingGetWithExistingPosition_thenReturnsElem
 
 TEST(ListTestIValueBasedList, whenCallingGetWithNonExistingPosition_thenThrowsException) {
   List<string> list({"3", "4"});
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list.get(2), std::out_of_range);
 }
 
@@ -55,7 +55,6 @@ TEST(ListTestIValueBasedList, whenCallingExtractWithExistingPosition_thenListEle
 
 TEST(ListTestIValueBasedList, whenCallingExtractWithNonExistingPosition_thenThrowsException) {
   List<string> list({"3", "4"});
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list.extract(2), std::out_of_range);
 }
 
@@ -78,14 +77,12 @@ TEST(ListTestIValueBasedList, whenCallingMovingSetWithExistingPosition_thenChang
 TEST(ListTestIValueBasedList, whenCallingCopyingSetWithNonExistingPosition_thenThrowsException) {
   List<string> list({"3", "4"});
   string value = "5";
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list.set(2, value), std::out_of_range);
 }
 
 TEST(ListTestIValueBasedList, whenCallingMovingSetWithNonExistingPosition_thenThrowsException) {
   List<string> list({"3", "4"});
   string value = "5";
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list.set(2, std::move(value)), std::out_of_range);
 }
 
@@ -121,7 +118,6 @@ TEST(ListTestIValueBasedList, whenSwappingFromAccessOperator_thenSwapsElements)
 
 TEST(ListTestIValueBasedList, whenCallingAccessOperatorWithNonExistingPosition_thenThrowsException) {
   List<string> list({"3", "4"});
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list[2], std::out_of_range);
 }
 
@@ -585,7 +581,6 @@ TEST(ListTestNonIValueBasedList, whenCallingGetWithExistingPosition_thenReturnsE
 
 TEST(ListTestNonIValueBasedList, whenCallingGetWithNonExistingPosition_thenThrowsException) {
   List<int64_t> list({3, 4});
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list.get(2), std::out_of_range);
 }
 
@@ -597,7 +592,6 @@ TEST(ListTestNonIValueBasedList, whenCallingExtractWithExistingPosition_thenRetu
 
 TEST(ListTestNonIValueBasedList, whenCallingExtractWithNonExistingPosition_thenThrowsException) {
   List<int64_t> list({3, 4});
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list.extract(2), std::out_of_range);
 }
 
@@ -621,14 +615,12 @@ TEST(ListTestNonIValueBasedList, whenCallingMovingSetWithExistingPosition_thenCh
 TEST(ListTestNonIValueBasedList, whenCallingCopyingSetWithNonExistingPosition_thenThrowsException) {
   List<int64_t> list({3, 4});
   int64_t value = 5;
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list.set(2, value), std::out_of_range);
 }
 
 TEST(ListTestNonIValueBasedList, whenCallingMovingSetWithNonExistingPosition_thenThrowsException) {
   List<int64_t> list({3, 4});
   int64_t value = 5;
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,performance-move-const-arg,hicpp-avoid-goto)
   EXPECT_THROW(list.set(2, std::move(value)), std::out_of_range);
 }
 
@@ -664,7 +656,6 @@ TEST(ListTestNonIValueBasedList, whenSwappingFromAccessOperator_thenSwapsElement
 
 TEST(ListTestNonIValueBasedList, whenCallingAccessOperatorWithNonExistingPosition_thenThrowsException) {
   List<int64_t> list({3, 4});
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-goto,hicpp-avoid-goto)
   EXPECT_THROW(list[2], std::out_of_range);
 }
 
@@ -1117,7 +1108,7 @@ TEST(ListTestNonIValueBasedList, sameValueDifferentStorage_thenIsReturnsFalse) {
 TEST(ListTest, canAccessStringByReference) {
   List<std::string> list({"one", "two"});
   const auto& listRef = list;
-  static_assert(std::is_same<decltype(listRef[1]), const std::string&>::value,
+  static_assert(std::is_same_v<decltype(listRef[1]), const std::string&>,
                 "const List<std::string> access should be by const reference");
   std::string str = list[1];
   const std::string& strRef = listRef[1];
@@ -1126,17 +1117,18 @@ TEST(ListTest, canAccessStringByReference) {
 }
 
 TEST(ListTest, canAccessOptionalStringByReference) {
-  List<c10::optional<std::string>> list({"one", "two", c10::nullopt});
+  List<std::optional<std::string>> list({"one", "two", std::nullopt});
   const auto& listRef = list;
   static_assert(
-      std::is_same<decltype(listRef[1]), c10::optional<std::reference_wrapper<const std::string>>>::value,
-      "List<c10::optional<std::string>> access should be by const reference");
-  c10::optional<std::string> str1 = list[1];
-  c10::optional<std::string> str2 = list[2];
-  decltype(auto) strRef1 = listRef[1];
-  decltype(auto) strRef2 = listRef[2];
-  EXPECT_EQ("two", str1.value());
+      std::is_same_v<decltype(listRef[1]), std::optional<std::reference_wrapper<const std::string>>>,
+      "List<std::optional<std::string>> access should be by const reference");
+  std::optional<std::string> str1 = list[1];
+  std::optional<std::string> str2 = list[2];
+  auto const& strRef1 = listRef[1];
+  auto const& strRef2 = listRef[2];
+  EXPECT_EQ("two", str1);
   EXPECT_FALSE(str2.has_value());
+  EXPECT_TRUE(strRef1.has_value());
   EXPECT_EQ("two", strRef1.value().get());
   EXPECT_FALSE(strRef2.has_value());
 }
@@ -1145,7 +1137,7 @@ TEST(ListTest, canAccessTensorByReference) {
   List<at::Tensor> list;
   const auto& listRef = list;
   static_assert(
-      std::is_same<decltype(listRef[0]), const at::Tensor&>::value,
+      std::is_same_v<decltype(listRef[0]), const at::Tensor&>,
       "List<at::Tensor> access should be by const reference");
 }
 
@@ -1159,3 +1151,4 @@ TEST(ListTest, toTypedList) {
   genericList = impl::toList(std::move(stringList));
   EXPECT_THROW(c10::impl::toTypedList<int64_t>(std::move(genericList)), c10::Error);
 }
+// NOLINTEND(performance-move-const-arg, bugprone-use-after-move, *analyzer*Move)
