@@ -7,12 +7,8 @@
 #include <mutex>
 #include <sstream>
 #include <string>
-#include <unordered_map>
-#include <vector>
 #include <c10/util/Exception.h>
 #include <ATen/core/interned_strings_class.h>
-#include <c10/util/Exception.h>
-#include <c10/util/Optional.h>
 
 namespace c10 {
 
