@@ -4,8 +4,7 @@
 
 namespace c10 {
 
-namespace detail {
-namespace infer_schema {
+namespace detail::infer_schema {
 namespace {
 
 std::vector<Argument> createArgumentVector(c10::ArrayRef<ArgumentDef> args) {
@@ -40,7 +39,6 @@ FunctionSchema make_function_schema(
     c10::ArrayRef<ArgumentDef> returns) {
   return make_function_schema("", "", arguments, returns);
 }
-} // namespace infer_schema
 } // namespace detail
 
 std::optional<std::string> findSchemaDifferences(
