@@ -7,14 +7,14 @@
 #include <ATen/functorch/BatchRulesHelper.h>
 #include <ATen/functorch/PlumbingHelper.h>
 
-namespace at { namespace functorch {
+namespace at::functorch {
 
 namespace{
-std::tuple<Tensor,optional<int64_t>>
+std::tuple<Tensor, std::optional<int64_t>>
 clone_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
-    optional<MemoryFormat> memory_format) {
+    std::optional<int64_t> self_bdim,
+    std::optional<MemoryFormat> memory_format) {
   // Memory format support is a little tricky because vmap is allowed to move
   // around batch dimensions and some memory formats are rank-dependent.
   // Another weird case is:
@@ -48,24 +48,17 @@ clone_batch_rule(
   return std::make_tuple(result, self_bdim);
 }
 
-std::tuple<Tensor,optional<int64_t>>
-view_as_complex_batch_rule(const Tensor& self, optional<int64_t> self_bdim) {
+std::tuple<Tensor, std::optional<int64_t>>
+view_as_complex_batch_rule(const Tensor& self, std::optional<int64_t> self_bdim) {
   // guard against the user passing in a batch of scalar tensors with batch
   // size equal to 2.
-  TORCH_CHECK(self.sizes().size() > 1, "Input tensor must have one or more dimensions");
+  TORCH_CHECK(self.sym_sizes().size() > 1, "Input tensor must have one or more dimensions");
 
   auto self_ = moveBatchDimToFront(self, self_bdim);
   auto result = at::view_as_complex(self_);
   return std::make_tuple(result, 0);
 }
 
-std::tuple<Tensor,optional<int64_t>>
-to_other_batch_rule(const Tensor& self, optional<int64_t> self_bdim,
-                    const Tensor& other, optional<int64_t> other_bdim,
-                    bool non_blocking,
-                    bool copy, c10::optional<at::MemoryFormat> memory_format) {
-  return std::make_tuple(self.to(other, non_blocking, copy, memory_format), self_bdim);
-}
 }
 
 TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
@@ -77,15 +70,9 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   POINTWISE_BOXED(op ## _); \
   VMAP_SUPPORT(op, BASIC_UNARY_BATCH_RULE(ATEN_FN(op)));
 
-  UNARY_POINTWISE(imag);
-  UNARY_POINTWISE(real);
   UNARY_POINTWISE(view_as_real);
   VMAP_SUPPORT(view_as_complex, view_as_complex_batch_rule);
   VMAP_SUPPORT(clone, clone_batch_rule);
-  VMAP_SUPPORT2(to, device, BASIC_UNARY_BATCH_RULE(ATEN_FN2(to, device)));
-  VMAP_SUPPORT2(to, dtype, BASIC_UNARY_BATCH_RULE(ATEN_FN2(to, dtype)));
-  VMAP_SUPPORT2(to, dtype_layout, BASIC_UNARY_BATCH_RULE(ATEN_FN2(to, dtype_layout)));
-  VMAP_SUPPORT2(to, other, to_other_batch_rule);
 
   UNARY_POINTWISE(_to_copy);
   UNARY_POINTWISE(alias);
@@ -110,12 +97,10 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   UNARY_POINTWISE_ALL(expm1);
   UNARY_POINTWISE_ALL(floor);
   UNARY_POINTWISE_ALL(frac);
-  UNARY_POINTWISE(isfinite);
   UNARY_POINTWISE(isnan);
   UNARY_POINTWISE(isinf);
   UNARY_POINTWISE(isposinf);
   UNARY_POINTWISE(isneginf);
-  UNARY_POINTWISE(isreal);
   UNARY_POINTWISE_ALL(lgamma);
   UNARY_POINTWISE_ALL(log);
   UNARY_POINTWISE_ALL(log10);
@@ -175,7 +160,6 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   UNARY_POINTWISE_ALL(hardtanh);
   UNARY_POINTWISE_ALL(hardswish);
   UNARY_POINTWISE_ALL(leaky_relu);
-  UNARY_POINTWISE(log_sigmoid);
   UNARY_POINTWISE_ALL(relu);
   UNARY_POINTWISE_ALL(celu);
   UNARY_POINTWISE(gelu);
@@ -194,4 +178,4 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
 }
 
 #undef INVOKE
-}}
+} // namespace at::functorch
