@@ -5,7 +5,6 @@
 // LICENSE file in the root directory of this source tree.
 
 #include <ATen/functorch/BatchRulesHelper.h>
-#include <iostream>
 #include <utility>
 
 #include <ATen/Operators.h>
@@ -17,7 +16,7 @@
 #include <c10/util/SmallBuffer.h>
 #include <ATen/InferSize.h>
 
-namespace at { namespace functorch {
+namespace at::functorch {
 
 // Note [Adding vmap support for an operator]
 // Hey there! So you have an operator and you want to get it to work with vmap.
@@ -35,10 +34,10 @@ namespace at { namespace functorch {
 //
 // For example, let's consider writing a batch rule for
 // `Tensor sum(const Tensor& self, int64_t dim)`. The signature of the
-// batch rule has an additional optional<int64_t> argument after each
+// batch rule has an additional std::optional<int64_t> argument after each
 // Tensor argument and return. So, in this case, the batch rule has signature
-//   tuple<Tensor,optional<int64_t>> sum_batch_rule(
-//       const Tensor& self, optional<int64_t> self_bdim, int64_t dim);
+//   tuple<Tensor, std::optional<int64_t>> sum_batch_rule(
+//       const Tensor& self, std::optional<int64_t> self_bdim, int64_t dim);
 //
 // The vmap call above invokes the batch rule with `self = tensor`,
 // `self_bdim = 0`, and `dim = 0`. Note that there are **no BatchedTensors**
@@ -91,9 +90,9 @@ namespace at { namespace functorch {
 
 namespace{
 
-std::tuple<Tensor,optional<int64_t>> unsqueeze_batch_rule(
+std::tuple<Tensor, std::optional<int64_t>> unsqueeze_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     int64_t dim) {
   auto self_ = moveBatchDimToFront(self, self_bdim);
   auto rank = rankWithoutBatchDim(self, self_bdim);
@@ -102,9 +101,9 @@ std::tuple<Tensor,optional<int64_t>> unsqueeze_batch_rule(
 }
 
 // NB: repeat is not actually a view, but it is in this file
-std::tuple<Tensor,optional<int64_t>> repeat_batch_rule(
+std::tuple<Tensor, std::optional<int64_t>> repeat_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     c10::SymIntArrayRef sizes) {
 
   SymDimVector sizes_with_bdim = { sizes.begin(), sizes.end() };
@@ -117,9 +116,9 @@ std::tuple<Tensor,optional<int64_t>> repeat_batch_rule(
 }
 
 
-std::tuple<Tensor,optional<int64_t>> _unsafe_view_batch_rule(
+std::tuple<Tensor, std::optional<int64_t>> _unsafe_view_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     c10::SymIntArrayRef size) {
   auto self_ = moveBatchDimToFront(self, self_bdim);
   SymDimVector view_size(size);
@@ -138,7 +137,7 @@ std::tuple<Tensor,optional<int64_t>> _unsafe_view_batch_rule(
   return std::make_tuple(at::_unsafe_view_symint(self_, view_size), 0);
 }
 
-std::tuple<Tensor,optional<int64_t>> flip_batch_rule(const Tensor& self, optional<int64_t> self_bdim, IntArrayRef dims) {
+std::tuple<Tensor, std::optional<int64_t>> flip_batch_rule(const Tensor& self, std::optional<int64_t> self_bdim, IntArrayRef dims) {
   auto self_ = moveBatchDimToFront(self, self_bdim);
   VmapDimVector new_dims;
   for (auto i: dims) {
@@ -150,7 +149,7 @@ std::tuple<Tensor,optional<int64_t>> flip_batch_rule(const Tensor& self, optiona
 const Tensor& resize__plumbing(
     const Tensor& self,
     IntArrayRef size,
-    c10::optional<MemoryFormat> optional_memory_format) {
+    std::optional<MemoryFormat> optional_memory_format) {
   TORCH_CHECK(
       !optional_memory_format.has_value() ||
       optional_memory_format == c10::MemoryFormat::Contiguous,
@@ -163,15 +162,13 @@ const Tensor& resize__plumbing(
     return self.resize_(size, optional_memory_format);
   }
 
-  Tensor self_value;
-  optional<int64_t> self_bdim;
-  std::tie(self_value, self_bdim) = unwrapTensorAtLevel(self, cur_level);
+  auto [self_value, self_bdim] = unwrapTensorAtLevel(self, cur_level);
   TORCH_INTERNAL_ASSERT(self_bdim.has_value());
 
   // TODO: The following algorithm only works for batch dim == 0.
   // To get it to work for something else we need the ability to modify
   // the BatchDims attribute of BatchedTensorImpl
-  TORCH_INTERNAL_ASSERT(self_bdim.value() == 0, "NYI: resize_ batch rule for batch dim != 0");
+  TORCH_INTERNAL_ASSERT(self_bdim == 0, "NYI: resize_ batch rule for batch dim != 0");
 
   // Resize the wrapped tensor
   c10::impl::ExcludeDispatchKeyGuard guard(DispatchKey::FuncTorchBatched);
@@ -188,7 +185,7 @@ const Tensor& resize__plumbing(
   return self;
 }
 
-std::tuple<Tensor, optional<int64_t>> squeeze_batch_rule(const Tensor& self, optional<int64_t> bdim) {
+std::tuple<Tensor, std::optional<int64_t>> squeeze_batch_rule(const Tensor& self, std::optional<int64_t> bdim) {
   TORCH_INTERNAL_ASSERT(bdim.has_value());
   // Special case for scalar arrays to replicate PyTorch behavior.
   if (self.dim() == 1) {
@@ -198,13 +195,13 @@ std::tuple<Tensor, optional<int64_t>> squeeze_batch_rule(const Tensor& self, opt
   // Manually calculate the output shape by eliding all dimensions of
   // size 1 keeping track of where the batch index started and where it
   // ended up moving to. We also ensure we do not drop the batch index.
-  auto shape = self.sizes();
-  DimVector squeezed_sizes;
+  auto shape = self.sym_sizes();
+  SymDimVector squeezed_sizes;
   bool before_batch_idx = true;
   int64_t new_batch_idx = 0;
   int64_t original_idx = 0;
 
-  for (auto it : shape) {
+  for (const auto& it : shape) {
     // Keep only dimensions != 1 and the batch dimension (irrespective of size).
     if (it != 1 || original_idx == bdim) {
       squeezed_sizes.push_back(it);
@@ -219,12 +216,12 @@ std::tuple<Tensor, optional<int64_t>> squeeze_batch_rule(const Tensor& self, opt
     ++original_idx;
   }
 
-  auto result = self.view(squeezed_sizes);
-  return std::make_tuple(std::move(result), c10::optional<int64_t>(new_batch_idx));
+  auto result = self.view_symint(squeezed_sizes);
+  return std::make_tuple(std::move(result), std::optional<int64_t>(new_batch_idx));
 }
 
-std::tuple<Tensor, optional<int64_t>> squeeze_dims_batch_rule(
-    const Tensor& self, optional<int64_t> bdim, IntArrayRef dims) {
+std::tuple<Tensor, std::optional<int64_t>> squeeze_dims_batch_rule(
+    const Tensor& self, std::optional<int64_t> bdim, IntArrayRef dims) {
   TORCH_INTERNAL_ASSERT(bdim.has_value());
   // Special case for scalar arrays to replicate PyTorch behavior.
   auto ndim = self.dim();
@@ -252,17 +249,17 @@ std::tuple<Tensor, optional<int64_t>> squeeze_dims_batch_rule(
       d = actual_dim + 1;
     }
   }
-  return std::make_tuple(self.squeeze(adjusted_dims), optional<int64_t>(updated_batch_idx));
+  return std::make_tuple(self.squeeze(adjusted_dims), std::optional<int64_t>(updated_batch_idx));
 }
 
-std::tuple<Tensor, optional<int64_t>> squeeze_dim_batch_rule(
-    const Tensor& self, optional<int64_t> bdim, int64_t dim) {
+std::tuple<Tensor, std::optional<int64_t>> squeeze_dim_batch_rule(
+    const Tensor& self, std::optional<int64_t> bdim, int64_t dim) {
   return squeeze_dims_batch_rule(self, bdim, {dim});
 }
 
-std::tuple<Tensor, optional<int64_t>> select_batching_rule(const Tensor& self, optional<int64_t> bdim, int64_t dim, c10::SymInt index) {
+std::tuple<Tensor, std::optional<int64_t>> select_batching_rule(const Tensor& self, std::optional<int64_t> bdim, int64_t dim, c10::SymInt index) {
   if (!bdim) {
-    return std::make_tuple(self.select_symint(dim, std::move(index)), nullopt);
+    return std::make_tuple(self.select_symint(dim, std::move(index)), std::nullopt);
   }
 
   auto _self = moveBatchDimToFront(self, bdim);
@@ -271,7 +268,7 @@ std::tuple<Tensor, optional<int64_t>> select_batching_rule(const Tensor& self, o
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor, optional<int64_t>> _reshape_alias_batch_rule(const Tensor& self, optional<int64_t> bdim, const c10::SymIntArrayRef shape, const c10::SymIntArrayRef strides) {
+std::tuple<Tensor, std::optional<int64_t>> _reshape_alias_batch_rule(const Tensor& self, std::optional<int64_t> bdim, const c10::SymIntArrayRef shape, const c10::SymIntArrayRef strides) {
   (void) strides;
   TORCH_INTERNAL_ASSERT(bdim.has_value());
 
@@ -282,7 +279,7 @@ std::tuple<Tensor, optional<int64_t>> _reshape_alias_batch_rule(const Tensor& se
   return std::make_tuple(at::reshape_symint(self_, new_shape), 0);
 }
 
-std::tuple<Tensor, optional<int64_t>> roll_batch_rule(const Tensor& self, optional<int64_t> bdim, SymIntArrayRef shifts, IntArrayRef dims) {
+std::tuple<Tensor, std::optional<int64_t>> roll_batch_rule(const Tensor& self, std::optional<int64_t> bdim, SymIntArrayRef shifts, IntArrayRef dims) {
   TORCH_INTERNAL_ASSERT(bdim.has_value());
 
   auto self_ = moveBatchDimToFront(self, bdim);
@@ -294,7 +291,7 @@ std::tuple<Tensor, optional<int64_t>> roll_batch_rule(const Tensor& self, option
     return std::make_tuple(at::roll_symint(self_, shifts, new_dims), 0);
   }
   // We will do something like: t.reshape(a, -1).roll(1, dims=[1, ]).reshape(old_shape)
-  auto old_shape = self_.sizes();
+  auto old_shape = self_.sym_sizes();
   new_dims.push_back(1);
   auto logical_rank = rankWithoutBatchDim(self, bdim);
   if (logical_rank == 0) {
@@ -304,12 +301,12 @@ std::tuple<Tensor, optional<int64_t>> roll_batch_rule(const Tensor& self, option
   auto output = at::roll_symint(self_.flatten(1), shifts, new_dims);
   // NOTE: For scalar tensor, we don't need to unsqueeze as reshape
   // with `old_shape` takes care of it.
-  output = output.reshape(old_shape);
+  output = output.reshape_symint(old_shape);
   return std::make_tuple(output, 0);
 }
 
-std::tuple<Tensor, optional<int64_t>> diagonal_batching_rule(
-    const Tensor &self, optional<int64_t> self_bdim,
+std::tuple<Tensor, std::optional<int64_t>> diagonal_batching_rule(
+    const Tensor &self, std::optional<int64_t> self_bdim,
     int64_t offset, int64_t dim1, int64_t dim2)
 {
   auto logical_rank = rankWithoutBatchDim(self, self_bdim);
@@ -320,8 +317,8 @@ std::tuple<Tensor, optional<int64_t>> diagonal_batching_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor,optional<int64_t>> diagonal_backward_batch_rule(
-    const Tensor& grad_input, optional<int64_t> grad_input_bdim,
+std::tuple<Tensor, std::optional<int64_t>> diagonal_backward_batch_rule(
+    const Tensor& grad_input, std::optional<int64_t> grad_input_bdim,
     c10::SymIntArrayRef input_sizes, int64_t offset, int64_t dim1, int64_t dim2) {
   auto logical_rank = rankWithoutBatchDim(grad_input, grad_input_bdim);
   auto grad_input_ = moveBatchDimToFront(grad_input, grad_input_bdim);
@@ -334,12 +331,12 @@ std::tuple<Tensor,optional<int64_t>> diagonal_backward_batch_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor,optional<int64_t>> slice_batch_rule(
+std::tuple<Tensor, std::optional<int64_t>> slice_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     int64_t dim,
-    c10::optional<c10::SymInt> start,
-    c10::optional<c10::SymInt> end,
+    std::optional<c10::SymInt> start,
+    std::optional<c10::SymInt> end,
     c10::SymInt step) {
   auto self_ = moveBatchDimToFront(self, self_bdim);
   dim = getPhysicalDim(self, self_bdim.has_value(), dim);
@@ -352,10 +349,10 @@ static bool is_allowed_dim_on_scalar_tensor(int64_t dim) {
   return dim == 0 || dim == -1;
 }
 
-std::tuple<Tensor,optional<int64_t>>
+std::tuple<Tensor, std::optional<int64_t>>
 transpose_int_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     int64_t dim0,
     int64_t dim1) {
   // PyTorch has a special case where scalar_tensor.transpose(dim0, dim1) works
@@ -374,8 +371,8 @@ transpose_int_batch_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor, optional<int64_t>> permute_batching_rule(
-    const Tensor &self, optional<int64_t> self_bdim, IntArrayRef dims)
+std::tuple<Tensor, std::optional<int64_t>> permute_batching_rule(
+    const Tensor &self, std::optional<int64_t> self_bdim, IntArrayRef dims)
 {
   if (!self_bdim.has_value()) {
     return std::make_tuple(self.permute(dims), self_bdim);
@@ -392,8 +389,8 @@ std::tuple<Tensor, optional<int64_t>> permute_batching_rule(
   return std::make_tuple(self_.permute(dims_), 0);
 }
 
-std::tuple<Tensor,optional<int64_t>> select_backward_batch_rule(
-    const Tensor& grad_input, optional<int64_t> grad_input_bdim,
+std::tuple<Tensor, std::optional<int64_t>> select_backward_batch_rule(
+    const Tensor& grad_input, std::optional<int64_t> grad_input_bdim,
     c10::SymIntArrayRef input_sizes, int64_t dim, c10::SymInt index) {
   auto logical_rank = rankWithoutBatchDim(grad_input, grad_input_bdim);
   auto grad_input_ = moveBatchDimToFront(grad_input, grad_input_bdim);
@@ -405,8 +402,8 @@ std::tuple<Tensor,optional<int64_t>> select_backward_batch_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor,optional<int64_t>> slice_backward_batch_rule(
-    const Tensor& grad_input, optional<int64_t> grad_input_bdim,
+std::tuple<Tensor, std::optional<int64_t>> slice_backward_batch_rule(
+    const Tensor& grad_input, std::optional<int64_t> grad_input_bdim,
     SymIntArrayRef input_sizes, int64_t dim, c10::SymInt start, c10::SymInt end, c10::SymInt step) {
   auto logical_rank = rankWithoutBatchDim(grad_input, grad_input_bdim);
   auto grad_input_ = moveBatchDimToFront(grad_input, grad_input_bdim);
@@ -418,8 +415,8 @@ std::tuple<Tensor,optional<int64_t>> slice_backward_batch_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor, optional<int64_t>> view_batching_rule(
-    const Tensor &self, optional<int64_t> self_bdim, SymIntArrayRef sym_size)
+std::tuple<Tensor, std::optional<int64_t>> view_batching_rule(
+    const Tensor &self, std::optional<int64_t> self_bdim, SymIntArrayRef sym_size)
 {
   TORCH_INTERNAL_ASSERT(self_bdim.has_value());
   auto self_ = moveBatchDimToFront(self, self_bdim);
@@ -430,9 +427,9 @@ std::tuple<Tensor, optional<int64_t>> view_batching_rule(
   return std::make_tuple(self_.view_symint(size_), 0);
 }
 
-std::tuple<Tensor,optional<int64_t>> view_copy_batch_rule(
+std::tuple<Tensor, std::optional<int64_t>> view_copy_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     c10::SymIntArrayRef size) {
   auto self_ = moveBatchDimToFront(self, self_bdim);
   SymDimVector view_size(size.size() + 1);
@@ -444,8 +441,8 @@ std::tuple<Tensor,optional<int64_t>> view_copy_batch_rule(
 
 
 template <typename F, F Func>
-std::tuple<Tensor, optional<int64_t>> expand_batch_rule(
-    const Tensor &self, optional<int64_t> self_bdim, SymIntArrayRef size, bool implicit)
+std::tuple<Tensor, std::optional<int64_t>> expand_batch_rule(
+    const Tensor &self, std::optional<int64_t> self_bdim, SymIntArrayRef size, bool implicit)
 {
   auto self_dim = self.dim();
   TORCH_CHECK(static_cast<uint64_t>(self_dim - 1) <= size.size(),
@@ -453,8 +450,8 @@ std::tuple<Tensor, optional<int64_t>> expand_batch_rule(
               "must be greater or equal to the number of dimensions in the tensor (", static_cast<uint64_t>(self_dim - 1), ")");
 
   auto self_ = moveBatchDimToFront(self, self_bdim);
-  auto self_sizes = self_.sizes();
-  auto batch_size = self_sizes[0];
+  auto self_sizes = self_.sym_sizes();
+  const auto& batch_size = self_sizes[0];
 
   c10::SmallVector<c10::SymInt> size_(size.size() + 1);
   size_[0] = batch_size;
@@ -478,8 +475,8 @@ std::tuple<Tensor, optional<int64_t>> expand_batch_rule(
   return std::make_tuple(Func(self_.view_symint(view_shape), size_, implicit), 0);
 }
 
-std::tuple<Tensor, optional<int64_t>> unfold_batch_rule(
-    const Tensor &self, optional<int64_t> self_bdim, int64_t dim, int64_t size, int64_t step)
+std::tuple<Tensor, std::optional<int64_t>> unfold_batch_rule(
+    const Tensor &self, std::optional<int64_t> self_bdim, int64_t dim, int64_t size, int64_t step)
 {
   TORCH_INTERNAL_ASSERT(self_bdim.has_value());
   auto self_ = moveBatchDimToFront(self, self_bdim);
@@ -495,8 +492,8 @@ std::tuple<Tensor, optional<int64_t>> unfold_batch_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor, optional<int64_t>> narrow_copy_batch_rule(
-    const Tensor &self, optional<int64_t> self_bdim, int64_t dim, c10::SymInt start, c10::SymInt length)
+std::tuple<Tensor, std::optional<int64_t>> narrow_copy_batch_rule(
+    const Tensor &self, std::optional<int64_t> self_bdim, int64_t dim, c10::SymInt start, c10::SymInt length)
 {
   TORCH_INTERNAL_ASSERT(self_bdim.has_value());
   auto self_ = moveBatchDimToFront(self, self_bdim);
@@ -507,9 +504,9 @@ std::tuple<Tensor, optional<int64_t>> narrow_copy_batch_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<std::vector<Tensor>, optional<int64_t>> unsafe_split_batch_rule(
+std::tuple<std::vector<Tensor>, std::optional<int64_t>> unsafe_split_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     c10::SymInt split_size,
     int64_t dim) {
   TORCH_INTERNAL_ASSERT(self_bdim.has_value());
@@ -520,14 +517,7 @@ std::tuple<std::vector<Tensor>, optional<int64_t>> unsafe_split_batch_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor, optional<int64_t>> movedim_batch_rule(const Tensor& self, optional<int64_t> self_bdim, IntArrayRef source, IntArrayRef destination) {
-  auto self_ = moveBatchDimToFront(self, self_bdim);
-  auto source_ = getPhysicalDims(self_, self_bdim.has_value(), source);
-  auto destination_ = getPhysicalDims(self_, self_bdim.has_value(), destination);
-  return std::make_tuple(self_.movedim(source_, destination_), 0);
-}
-
-std::tuple<Tensor, optional<int64_t>> diag_embed_batch_rule(const Tensor& self, optional<int64_t> self_bdim, int64_t offset, int64_t dim1, int64_t dim2) {
+std::tuple<Tensor, std::optional<int64_t>> diag_embed_batch_rule(const Tensor& self, std::optional<int64_t> self_bdim, int64_t offset, int64_t dim1, int64_t dim2) {
   auto logical_rank = rankWithoutBatchDim(self, self_bdim);
   auto self_ = moveBatchDimToFront(self, self_bdim);
   dim1 = maybe_wrap_dim(dim1, logical_rank + 1) + 1;
@@ -540,9 +530,9 @@ Tensor trace_decomp(const Tensor& tensor) {
   return tensor.diagonal().sum();
 }
 
-std::tuple<Tensor,optional<int64_t>> tril_batch_rule(
+std::tuple<Tensor, std::optional<int64_t>> tril_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     int64_t diagonal = 0) {
   TORCH_CHECK(self.dim() >= 2, "tril: The input tensor must have at least 2 dimensions.");
   auto self_ = moveBatchDimToFront(self, self_bdim);
@@ -550,9 +540,9 @@ std::tuple<Tensor,optional<int64_t>> tril_batch_rule(
   return std::make_tuple(std::move(result), 0);
 }
 
-std::tuple<Tensor,optional<int64_t>> triu_batch_rule(
+std::tuple<Tensor, std::optional<int64_t>> triu_batch_rule(
     const Tensor& self,
-    optional<int64_t> self_bdim,
+    std::optional<int64_t> self_bdim,
     int64_t diagonal = 0) {
   TORCH_CHECK(self.dim() >= 2, "triu: The input tensor must have at least 2 dimensions.");
   auto self_ = moveBatchDimToFront(self, self_bdim);
@@ -563,7 +553,6 @@ std::tuple<Tensor,optional<int64_t>> triu_batch_rule(
 }
 
 TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
-  m.impl("flatten.using_ints", static_cast<decltype(&ATEN_FN2(flatten, using_ints))>(native::flatten));
   VMAP_SUPPORT(flip, flip_batch_rule);
   m.impl("trace", trace_decomp);
   VMAP_SUPPORT(tril, tril_batch_rule);
@@ -588,7 +577,6 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   VMAP_SUPPORT(expand, SINGLE_ARG(expand_batch_rule<decltype(&ATEN_FN(expand)), &ATEN_FN(expand)>));
   VMAP_SUPPORT(expand_copy, SINGLE_ARG(expand_batch_rule<decltype(&ATEN_FN(expand_copy)), &ATEN_FN(expand_copy)>));
   VMAP_SUPPORT(unfold, unfold_batch_rule);
-  VMAP_SUPPORT2(movedim, intlist, movedim_batch_rule);
   VMAP_SUPPORT2(slice, Tensor, slice_batch_rule);
   VMAP_SUPPORT2(transpose, int, transpose_int_batch_rule);
   m.impl("t", native::t);  // CompositeExplicitAutograd, should not go in BatchRulesDecompositions.cpp
@@ -598,4 +586,4 @@ TORCH_LIBRARY_IMPL(aten, FuncTorchBatched, m) {
   VMAP_SUPPORT2(unsafe_split, Tensor, unsafe_split_batch_rule);
 }
 
-}}
+} // namespace at::functorch
