@@ -4,7 +4,7 @@
 
 // MKL Sparse is not currently supported on Windows
 // See https://github.com/pytorch/pytorch/issues/97352
-#if AT_MKL_ENABLED() && (!defined(_WIN32))
+#if AT_MKL_ENABLED()
 #define AT_USE_MKL_SPARSE() 1
 #else
 #define AT_USE_MKL_SPARSE() 0
