@@ -17,8 +17,7 @@
 #include <ATen/ops/linalg_cross_native.h>
 #endif
 
-namespace at {
-namespace meta {
+namespace at::meta {
 
 TORCH_META_FUNC(linalg_cross)
 (const Tensor & input, const Tensor & other, int64_t dim) {
@@ -36,12 +35,12 @@ TORCH_META_FUNC(linalg_cross)
   set_output_raw_strided(0, out_size, {}, input.options());
 }
 
-}
-namespace native {
+} // namespace at::meta
+namespace at::native {
 
 DEFINE_DISPATCH(cross_stub);
 
-static int64_t _default_cross_dim(const c10::optional<int64_t> &dimension, SymIntArrayRef sizes) {
+static int64_t _default_cross_dim(const std::optional<int64_t> &dimension, SymIntArrayRef sizes) {
   // If dimension is not given, it defaults to the first dimension found with the size 3.
   // Note that this behaviour might be unexpected.
   // _default_cross_dim is called internally inside the cross implementation to calculate
@@ -58,12 +57,19 @@ static int64_t _default_cross_dim(const c10::optional<int64_t> &dimension, SymIn
   TORCH_CHECK(false, "no dimension of size 3 in input");
 }
 
-Tensor cross(const Tensor & input, const Tensor & other, const c10::optional<int64_t> dimension) {
+Tensor cross(const Tensor & input, const Tensor & other, const std::optional<int64_t> dimension) {
+  if (!dimension) {
+    TORCH_WARN_ONCE(
+      "Using torch.cross without specifying the dim arg is deprecated.\n",
+      "Please either pass the dim explicitly or simply use torch.linalg.cross.\n",
+      "The default value of dim will change to agree with that of linalg.cross in a future release."
+    );
+  }
   auto dim = _default_cross_dim(dimension, input.sym_sizes());
   return at::linalg_cross(input, other, dim);
 }
 
-Tensor & cross_out(const Tensor & input, const Tensor & other, const c10::optional<int64_t> dimension, Tensor & out) {
+Tensor & cross_out(const Tensor & input, const Tensor & other, const std::optional<int64_t> dimension, Tensor & out) {
   auto dim = _default_cross_dim(dimension, input.sym_sizes());
   return at::linalg_cross_out(out, input, other, dim);
 }
@@ -79,4 +85,4 @@ TORCH_IMPL_FUNC(linalg_cross_out)
   cross_stub(input.device().type(), out, input_broadcasted, other_broadcasted, dim);
 }
 
-}} // namespace at::native
+} // namespace at::native
