@@ -12,14 +12,33 @@
 #include <ATen/Functions.h>
 #include <ATen/NativeFunctions.h>
 #else
+#include <ATen/ops/linspace.h>
+#include <ATen/ops/logspace.h>
 #include <ATen/ops/arange_native.h>
 #include <ATen/ops/linspace_native.h>
 #include <ATen/ops/logspace_native.h>
 #include <ATen/ops/range_native.h>
 #endif
 
-namespace at { namespace native {
+namespace at::native {
 
+Tensor& linspace_out(const Tensor& start, const Tensor& end, int64_t steps, Tensor& result) {
+  TORCH_CHECK(start.dim() == 0 && end.dim() == 0, "linspace only supports 0-dimensional start and end tensors, "
+    "but got start with ", start.dim(), " dimension(s) and end with ", end.dim()," dimension(s).");
+  return at::linspace_out(result, start.item(), end.item(), steps);
+}
+
+Tensor& linspace_out(const Tensor& start, const Scalar& end, int64_t steps, Tensor& result) {
+  TORCH_CHECK(start.dim() == 0, "linspace only supports 0-dimensional start and end tensors, "
+    "but got start with ", start.dim(), " dimension(s).");
+  return at::linspace_out(result, start.item(), end, steps);
+}
+
+Tensor& linspace_out(const Scalar& start, const Tensor& end, int64_t steps, Tensor& result) {
+  TORCH_CHECK(end.dim() == 0, "linspace only supports 0-dimensional start and end tensors, "
+    "but got end with ", end.dim()," dimension(s).");
+  return at::linspace_out(result, start, end.item(), steps);
+}
 
 Tensor& linspace_out(const Scalar& start, const Scalar& end, int64_t steps, Tensor& result) {
   TORCH_CHECK(steps >= 0, "number of steps must be non-negative");
@@ -47,6 +66,24 @@ Tensor& linspace_out(const Scalar& start, const Scalar& end, int64_t steps, Tens
   return result;
 }
 
+Tensor& logspace_out(const Tensor& start, const Tensor& end, int64_t steps, double base, Tensor& result) {
+  TORCH_CHECK(start.dim() == 0 && end.dim() == 0, "logspace only supports 0-dimensional start and end tensors, "
+    "but got start with ", start.dim(), " dimension(s) and end with ", end.dim()," dimension(s).");
+  return at::logspace_out(result, start.item(), end.item(), steps, base);
+}
+
+Tensor& logspace_out(const Tensor& start, const Scalar& end, int64_t steps, double base, Tensor& result) {
+  TORCH_CHECK(start.dim() == 0, "logspace only supports 0-dimensional start and end tensors, "
+    "but got start with ", start.dim(), " dimension(s).");
+  return at::logspace_out(result, start.item(), end, steps, base);
+}
+
+Tensor& logspace_out(const Scalar& start, const Tensor& end, int64_t steps, double base, Tensor& result) {
+  TORCH_CHECK(end.dim() == 0, "logspace only supports 0-dimensional start and end tensors, "
+    "but got end with ", end.dim()," dimension(s).");
+  return at::logspace_out(result, start, end.item(), steps, base);
+}
+
 Tensor& logspace_out(const Scalar& start, const Scalar& end, int64_t steps, double base, Tensor& result) {
   TORCH_CHECK(steps >= 0, "number of steps must be non-negative");
 
@@ -88,7 +125,7 @@ Tensor& logspace_out(const Scalar& start, const Scalar& end, int64_t steps, doub
       });
     });
   } else {
-    AT_DISPATCH_ALL_TYPES_AND(kBFloat16, r.scalar_type(), "logspace_cpu", [&]() {
+    AT_DISPATCH_ALL_TYPES_AND2(kBFloat16, kHalf, r.scalar_type(), "logspace_cpu", [&]() {
       double scalar_base = static_cast<double>(base); // will be autopromoted anyway
       scalar_t scalar_start = start.to<scalar_t>();
       scalar_t scalar_end = end.to<scalar_t>();
@@ -114,7 +151,7 @@ Tensor& logspace_out(const Scalar& start, const Scalar& end, int64_t steps, doub
 }
 
 Tensor& range_out(const Scalar& start, const Scalar& end, const Scalar& step, Tensor& result) {
-  AT_DISPATCH_ALL_TYPES_AND(kBFloat16, result.scalar_type(), "range_cpu", [&]() {
+  AT_DISPATCH_ALL_TYPES_AND2(kBFloat16, kHalf, result.scalar_type(), "range_cpu", [&]() {
     using accscalar_t = at::acc_type<scalar_t, false>;
     auto xstart = start.to<accscalar_t>();
     auto xend = end.to<accscalar_t>();
@@ -220,4 +257,4 @@ Tensor& arange_out(const Scalar& start, const Scalar& end, const Scalar& step, T
 DEFINE_DISPATCH(arange_stub);
 DEFINE_DISPATCH(linspace_stub);
 
-}} // namespace at::native
+} // namespace at::native
