@@ -17,8 +17,7 @@
 #include <ATen/ops/min_native.h>
 #endif
 
-namespace at {
-namespace native {
+namespace at::native {
 
 DEFINE_DISPATCH(min_all_stub);
 DEFINE_DISPATCH(max_all_stub);
@@ -74,4 +73,4 @@ std::tuple<Tensor, Tensor> _aminmax_all(const Tensor &self) {
   return at::aminmax(self);
 }
 
-}} // namespace at::native
+} // namespace at::native
