@@ -385,7 +385,7 @@ Tensor _segment_reduce_cpu_offsets_backward_kernel(
 
 Tensor segment_reduce_kernel(
     const Tensor& data,
-    c10::string_view reduce,
+    std::string_view reduce,
     const std::optional<Tensor>& lengths,
     const std::optional<Tensor>& indices,
     const std::optional<Tensor>& offsets,
@@ -461,21 +461,23 @@ Tensor segment_reduce_kernel(
 REGISTER_ARCH_DISPATCH(
     _segment_reduce_lengths_stub,
     DEFAULT,
-    &_segment_reduce_lengths_cpu_kernel);
-REGISTER_AVX2_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel);
-REGISTER_AVX512_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel);
-REGISTER_VSX_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel);
-REGISTER_ZVECTOR_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel);
+    &_segment_reduce_lengths_cpu_kernel)
+REGISTER_AVX2_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel)
+REGISTER_AVX512_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel)
+REGISTER_VSX_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel)
+REGISTER_ZVECTOR_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel)
+REGISTER_SVE256_DISPATCH(_segment_reduce_lengths_stub, &_segment_reduce_lengths_cpu_kernel)
 
 // offsets dispatches
 REGISTER_ARCH_DISPATCH(
     _segment_reduce_offsets_stub,
     DEFAULT,
-    &_segment_reduce_offsets_cpu_kernel);
-REGISTER_AVX2_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel);
-REGISTER_AVX512_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel);
-REGISTER_VSX_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel);
-REGISTER_ZVECTOR_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel);
+    &_segment_reduce_offsets_cpu_kernel)
+REGISTER_AVX2_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel)
+REGISTER_AVX512_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel)
+REGISTER_VSX_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel)
+REGISTER_ZVECTOR_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel)
+REGISTER_SVE256_DISPATCH(_segment_reduce_offsets_stub, &_segment_reduce_offsets_cpu_kernel)
 
 // Currently some computation is being duplicated across forward and backward.
 // TODO: Cache indices in forward pass to re-use in backward
@@ -483,7 +485,7 @@ Tensor _segment_reduce_backward_kernel(
     const Tensor& grad,
     const Tensor& output,
     const Tensor& data,
-    c10::string_view reduce,
+    std::string_view reduce,
     const std::optional<Tensor>& lengths,
     const std::optional<Tensor>& offsets,
     int64_t axis,
@@ -533,35 +535,41 @@ Tensor _segment_reduce_backward_kernel(
 REGISTER_ARCH_DISPATCH(
     _segment_reduce_lengths_backward_stub,
     DEFAULT,
-    &_segment_reduce_cpu_lengths_backward_kernel);
+    &_segment_reduce_cpu_lengths_backward_kernel)
 REGISTER_AVX512_DISPATCH(
     _segment_reduce_lengths_backward_stub,
-    &_segment_reduce_cpu_lengths_backward_kernel);
+    &_segment_reduce_cpu_lengths_backward_kernel)
 REGISTER_AVX2_DISPATCH(
     _segment_reduce_lengths_backward_stub,
-    &_segment_reduce_cpu_lengths_backward_kernel);
+    &_segment_reduce_cpu_lengths_backward_kernel)
 REGISTER_VSX_DISPATCH(
     _segment_reduce_lengths_backward_stub,
-    &_segment_reduce_cpu_lengths_backward_kernel);
+    &_segment_reduce_cpu_lengths_backward_kernel)
 REGISTER_ZVECTOR_DISPATCH(
     _segment_reduce_lengths_backward_stub,
-    &_segment_reduce_cpu_lengths_backward_kernel);
+    &_segment_reduce_cpu_lengths_backward_kernel)
+REGISTER_SVE256_DISPATCH(
+    _segment_reduce_lengths_backward_stub,
+    &_segment_reduce_cpu_lengths_backward_kernel)
 
 REGISTER_ARCH_DISPATCH(
     _segment_reduce_offsets_backward_stub,
     DEFAULT,
-    &_segment_reduce_cpu_offsets_backward_kernel);
+    &_segment_reduce_cpu_offsets_backward_kernel)
 REGISTER_AVX512_DISPATCH(
     _segment_reduce_offsets_backward_stub,
-    &_segment_reduce_cpu_offsets_backward_kernel);
+    &_segment_reduce_cpu_offsets_backward_kernel)
 REGISTER_AVX2_DISPATCH(
     _segment_reduce_offsets_backward_stub,
-    &_segment_reduce_cpu_offsets_backward_kernel);
+    &_segment_reduce_cpu_offsets_backward_kernel)
 REGISTER_VSX_DISPATCH(
     _segment_reduce_offsets_backward_stub,
-    &_segment_reduce_cpu_offsets_backward_kernel);
+    &_segment_reduce_cpu_offsets_backward_kernel)
 REGISTER_ZVECTOR_DISPATCH(
     _segment_reduce_offsets_backward_stub,
-    &_segment_reduce_cpu_offsets_backward_kernel);
+    &_segment_reduce_cpu_offsets_backward_kernel)
+REGISTER_SVE256_DISPATCH(
+    _segment_reduce_offsets_backward_stub,
+    &_segment_reduce_cpu_offsets_backward_kernel)
 
 } // namespace at::native
