@@ -170,9 +170,7 @@
 
 #include <cmath>
 
-namespace at {
-
-namespace meta {
+namespace at::meta {
 
 // Unary float operations always produce floating point
 // outputs for floating point and integral types
@@ -292,9 +290,9 @@ TORCH_META_FUNC(ceil) (const Tensor& self) {
   build_borrowing_unary_op(maybe_get_output(), self);
 }
 
-} // namespace meta
+} // namespace at::meta
 
-namespace native {
+namespace at::native {
 // NOTE: These are helper functions that reduce redundant code in implementing the most typical kind of unary operators.
 // YOU ARE NOT OBLIGED TO USE THESE HELPERS---if you're writing something more specialized, please don't try to make
 // them work for your case, but just write something new instead. Here we use helper functions instead of a flat fat
@@ -414,7 +412,6 @@ template <typename Stub, typename ...Args>
 static inline Tensor& unary_op_impl_float_out(Tensor& result, const Tensor& self, Stub& stub, Args... args) {
   auto iter = TensorIterator::unary_float_op(result, self);
   stub(iter.device_type(), iter, args...);
-  iter.cast_outputs();
   return result;
 }
 
@@ -775,23 +772,23 @@ Tensor square(const Tensor& self) { return at::pow(self, 2); }
 Tensor& square_(Tensor& self) { return self.pow_(2); }
 
 Tensor& logit_out(const Tensor& self,
-    c10::optional<double> eps,
+    std::optional<double> eps,
     Tensor& result) {
   return unary_op_impl_float_out(
       result, self, logit_stub, Scalar(eps ? eps.value() : -1.0));
 }
-Tensor logit(const Tensor& self, c10::optional<double> eps) {
+Tensor logit(const Tensor& self, std::optional<double> eps) {
   return unary_op_impl_float(
       self, logit_stub, Scalar(eps ? eps.value() : -1.0));
 }
-Tensor& logit_(Tensor& self, c10::optional<double> eps) {
+Tensor& logit_(Tensor& self, std::optional<double> eps) {
   return at::logit_out(self, self, eps);
 }
 
-Tensor& special_logit_out(const Tensor& self, c10::optional<double> eps, Tensor& result) {
+Tensor& special_logit_out(const Tensor& self, std::optional<double> eps, Tensor& result) {
   return at::logit_out(result, self, eps);
 }
-Tensor special_logit(const Tensor& self, c10::optional<double> eps) {
+Tensor special_logit(const Tensor& self, std::optional<double> eps) {
   return self.logit(eps);
 }
 
@@ -804,9 +801,9 @@ Tensor special_expit(const Tensor& self) {
 }
 
 Tensor& nan_to_num_out(const Tensor& self,
-    c10::optional<double> nan,
-    c10::optional<double> pos_inf,
-    c10::optional<double> neg_inf,
+    std::optional<double> nan,
+    std::optional<double> pos_inf,
+    std::optional<double> neg_inf,
     Tensor& result) {
   TORCH_CHECK(
       self.scalar_type() == result.scalar_type(),
@@ -828,18 +825,18 @@ Tensor& nan_to_num_out(const Tensor& self,
 
 Tensor nan_to_num(
     const Tensor& self,
-    c10::optional<double> nan,
-    c10::optional<double> pos_inf,
-    c10::optional<double> neg_inf) {
+    std::optional<double> nan,
+    std::optional<double> pos_inf,
+    std::optional<double> neg_inf) {
   auto result = at::empty_like(self);
   return at::nan_to_num_out(result, self, nan, pos_inf, neg_inf);
 }
 
 Tensor& nan_to_num_(
     Tensor& self,
-    c10::optional<double> nan,
-    c10::optional<double> pos_inf,
-    c10::optional<double> neg_inf) {
+    std::optional<double> nan,
+    std::optional<double> pos_inf,
+    std::optional<double> neg_inf) {
   return at::nan_to_num_out(self, self, nan, pos_inf, neg_inf);
 }
 
@@ -870,7 +867,7 @@ Tensor& logical_not_out(const Tensor& self, Tensor& result) {
   TensorIterator iter = TensorIteratorConfig()
     .check_all_same_dtype(false)
     .add_output(result)
-    .add_input(self)
+    .add_const_input(self)
     .build();
   logical_not_stub(iter.device_type(), iter);
   return result;
@@ -935,11 +932,11 @@ Tensor& mvlgamma_out(const Tensor& self, int64_t p, Tensor& result) {
 
 Tensor special_multigammaln(const Tensor& self, int64_t p) {
   return self.mvlgamma(p);
-};
+}
 
 Tensor& special_multigammaln_out(const Tensor& self, int64_t p, Tensor& result) {
   return at::mvlgamma_out(result, self, p);
-};
+}
 
 std::tuple<Tensor, Tensor> frexp(const Tensor& self) {
   Tensor mantissa = at::empty_like(self);
@@ -966,7 +963,7 @@ std::tuple<Tensor&, Tensor&> frexp_out(const Tensor& self,
   auto iter = TensorIteratorConfig()
     .add_output(mantissa)
     .add_output(exponent)
-    .add_input(self)
+    .add_const_input(self)
     .check_all_same_dtype(false)
     .set_check_mem_overlap(true)
     .build();
@@ -975,7 +972,7 @@ std::tuple<Tensor&, Tensor&> frexp_out(const Tensor& self,
   return std::tuple<Tensor&, Tensor&>(mantissa, exponent);
 }
 
-// alias for lgamma, implements special.gammanln equivalent to
+// alias for lgamma, implements special.gammaln equivalent to
 // scipy.special.gammaln
 Tensor special_gammaln(const Tensor& self) { return self.lgamma(); }
 Tensor& special_gammaln_out(const Tensor& self, Tensor& result) { return at::lgamma_out(result, self); }
@@ -1050,5 +1047,4 @@ DEFINE_DISPATCH(special_scaled_modified_bessel_k0_stub); // NOLINT(cppcoreguidel
 DEFINE_DISPATCH(special_scaled_modified_bessel_k1_stub); // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
 DEFINE_DISPATCH(special_spherical_bessel_j0_stub); // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
 
-} // namespace native
-} // namespace at
+} // namespace at::native
