@@ -5,11 +5,12 @@
 #include <ATen/Functions.h>
 #include <ATen/NativeFunctions.h>
 #else
+#include <ATen/ops/empty.h>
 #include <ATen/ops/unfold_backward_native.h>
 #include <ATen/ops/zeros.h>
 #endif
 
-namespace at { namespace native {
+namespace at::native {
 
 DEFINE_DISPATCH(unfold_backward_stub);
 
@@ -21,6 +22,11 @@ Tensor unfold_backward(
   int64_t step
 ) {
   auto grad_input = at::zeros(input_sizes, grad.options());
+  if (step >= size) {
+    auto gI_unfolded = grad_input.unfold(dim, size, step);
+    gI_unfolded.copy_(grad);
+    return grad_input;
+  }
 
   unfold_backward_stub(
     grad.device().type(),
@@ -32,4 +38,4 @@ Tensor unfold_backward(
   return grad_input;
 }
 
-}} // namespace at::native
+} // namespace at::native
