@@ -18,8 +18,7 @@
 #include <ATen/ops/empty.h>
 #endif
 
-namespace ao {
-namespace sparse {
+namespace ao::sparse {
 
 int register_linear_params();
 
@@ -195,4 +194,4 @@ TORCH_LIBRARY_IMPL(sparse, CPU, m) {
 }
 
 } // namespace
-}} // namespace ao::sparse
+} // namespace ao::sparse
