@@ -520,6 +520,6 @@ Tensor _convolution_depthwise3x3_winograd(
 
 }  // namespace
 
-ALSO_REGISTER_AVX512_DISPATCH(convolution_depthwise3x3_winograd_stub, &_convolution_depthwise3x3_winograd);
+ALSO_REGISTER_AVX512_DISPATCH(convolution_depthwise3x3_winograd_stub, &_convolution_depthwise3x3_winograd)
 
 }  // namespace at::native
