@@ -60,7 +60,7 @@ void bernoulli_scalar_kernel(const TensorBase &self, double p, std::optional<Gen
   AT_DISPATCH_ALL_TYPES_AND3(at::ScalarType::Bool, at::ScalarType::BFloat16, at::ScalarType::Half,
   self.scalar_type(), "bernoulli_scalar_cpu_", [&] {
     at::Tensor tmp_int_tensor;
-    if (std::is_same<scalar_t, int>::value && contig) {
+    if (std::is_same_v<scalar_t, int> && contig) {
       tmp_int_tensor = self;
     } else {
       tmp_int_tensor = at::empty(self.sizes(), self.options().dtype(at::kInt));
@@ -81,7 +81,7 @@ void bernoulli_scalar_kernel(const TensorBase &self, double p, std::optional<Gen
 
         // vectorized copy if using buffer and contiguous, i.e., being non-int
         // type and contiguous
-        if (!std::is_same<scalar_t, int>::value && contig) {
+        if (!std::is_same_v<scalar_t, int> && contig) {
           scalar_t *self_seg = self_ptr + begin;
           int* tmp_seg = sample_int_ptr + begin;
           at::vec::convert<int, scalar_t>(tmp_seg, self_seg, len);
@@ -129,17 +129,17 @@ void exponential_kernel(TensorIteratorBase &iter, double lambda, std::optional<G
 
     AT_DISPATCH_FLOATING_TYPES_AND2(at::ScalarType::Half, at::ScalarType::BFloat16, self.scalar_type(), "exponential_cpu", [&] {
       at::Tensor tmp_tensor;
-      constexpr bool is_df = std::is_same<scalar_t, float>::value || std::is_same<scalar_t, double>::value;
+      constexpr bool is_df = std::is_same_v<scalar_t, float> || std::is_same_v<scalar_t, double>;
       if (is_df && contig) {
         tmp_tensor = self;
-      } else if (std::is_same<scalar_t, double>::value) {
+      } else if (std::is_same_v<scalar_t, double>) {
         tmp_tensor = at::empty(self.sizes(), self.options().dtype(at::kDouble));
       } else {
         tmp_tensor = at::empty(self.sizes(), self.options().dtype(at::kFloat));
       }
 
       scalar_t *self_ptr = self.data_ptr<scalar_t>();
-      using tmp_scalar_t = typename std::conditional_t<std::is_same<scalar_t, double>::value, double, float>;
+      using tmp_scalar_t = typename std::conditional_t<std::is_same_v<scalar_t, double>, double, float>;
       tmp_scalar_t *sample_ptr = tmp_tensor.data_ptr<tmp_scalar_t>();
 
       // Intel MKL vRngExponential variate originally does not exclude 0.
@@ -159,7 +159,7 @@ void exponential_kernel(TensorIteratorBase &iter, double lambda, std::optional<G
         int64_t len = end - begin;
         if (len > 0) {
           VSLStreamStatePtr stream;
-          if constexpr (std::is_same<scalar_t, double>::value) {
+          if constexpr (std::is_same_v<scalar_t, double>) {
             vslNewStream(&stream, VSL_BRNG_MCG31, seed);
             vslSkipAheadStream(stream, begin);
             vdRngExponential(VSL_RNG_METHOD_EXPONENTIAL_ICDF, stream, len,
@@ -235,16 +235,16 @@ static void random_full_64_bits_range_kernel(TensorIteratorBase& iter, std::opti
 
 } // namespace (anonymous)
 
-REGISTER_DISPATCH(bernoulli_tensor_stub, &bernoulli_tensor_kernel);
-REGISTER_DISPATCH(bernoulli_scalar_stub, &bernoulli_scalar_kernel);
-REGISTER_DISPATCH(cauchy_stub, &cauchy_kernel);
-REGISTER_DISPATCH(exponential_stub, &exponential_kernel);
-REGISTER_DISPATCH(geometric_stub, &geometric_kernel);
-REGISTER_DISPATCH(log_normal_stub, &log_normal_kernel);
-REGISTER_DISPATCH(normal_stub, &normal_kernel);
-REGISTER_DISPATCH(uniform_stub, &uniform_kernel);
-REGISTER_DISPATCH(random_from_to_stub, &random_from_to_kernel);
-REGISTER_DISPATCH(random_full_64_bits_range_stub, &random_full_64_bits_range_kernel);
-REGISTER_DISPATCH(random_stub, &random_kernel);
+REGISTER_DISPATCH(bernoulli_tensor_stub, &bernoulli_tensor_kernel)
+REGISTER_DISPATCH(bernoulli_scalar_stub, &bernoulli_scalar_kernel)
+REGISTER_DISPATCH(cauchy_stub, &cauchy_kernel)
+REGISTER_DISPATCH(exponential_stub, &exponential_kernel)
+REGISTER_DISPATCH(geometric_stub, &geometric_kernel)
+REGISTER_DISPATCH(log_normal_stub, &log_normal_kernel)
+REGISTER_DISPATCH(normal_stub, &normal_kernel)
+REGISTER_DISPATCH(uniform_stub, &uniform_kernel)
+REGISTER_DISPATCH(random_from_to_stub, &random_from_to_kernel)
+REGISTER_DISPATCH(random_full_64_bits_range_stub, &random_full_64_bits_range_kernel)
+REGISTER_DISPATCH(random_stub, &random_kernel)
 
 } // namespace at::native
