@@ -287,8 +287,7 @@ template<typename scalar_t>
 void infer_bin_edges_from_input(const Tensor& input, const int64_t N,
         std::vector<double> &leftmost_edges, std::vector<double> &rightmost_edges) {
     // Calls aminmax on input with dim=0, reducing all but the innermost dimension of input.
-    Tensor min, max;
-    std::tie(min, max) = aminmax(input, 0);
+    auto [min, max] = aminmax(input, 0);
 
     TORCH_INTERNAL_ASSERT(min.is_contiguous() && max.is_contiguous());
 
@@ -308,8 +307,8 @@ static void histogram_select_outer_bin_edges_impl(const Tensor& input, const int
 
 } // namespace
 
-REGISTER_DISPATCH(histogramdd_stub, &histogramdd_kernel_impl);
-REGISTER_DISPATCH(histogramdd_linear_stub, &histogramdd_linear_kernel_impl);
-REGISTER_DISPATCH(histogram_select_outer_bin_edges_stub, &histogram_select_outer_bin_edges_impl);
+REGISTER_DISPATCH(histogramdd_stub, &histogramdd_kernel_impl)
+REGISTER_DISPATCH(histogramdd_linear_stub, &histogramdd_linear_kernel_impl)
+REGISTER_DISPATCH(histogram_select_outer_bin_edges_stub, &histogram_select_outer_bin_edges_impl)
 
 } // namespace at::native
