@@ -36,6 +36,7 @@
 #include <ATen/native/TensorIteratorDynamicCasting.h>
 #include <ATen/cpu/vec/vec.h>
 
+#include <tuple>
 #include <utility>
 
 namespace at::native { inline namespace CPU_CAPABILITY {
@@ -171,7 +172,7 @@ multiple_outputs_loop(char* C10_RESTRICT data[], const int64_t* strides_, int64_
   using traits = function_traits<func_t>;
 
   using result_type = typename traits::result_type;
-  constexpr int num_outputs = std::tuple_size<result_type>::value;
+  constexpr int num_outputs = std::tuple_size_v<result_type>;
   constexpr int ntensors = traits::arity + num_outputs;
 
   // Copying strides to temporary array helps auto vectorization in older GCC
@@ -207,7 +208,7 @@ vectorized_loop(char** C10_RESTRICT data_, int64_t n, int64_t S, func_t&& op, ve
     data[arg] = data_[arg];
   }
 
-  Vec opt_scalar = Vec(S > 0 ? *(scalar_t*)data[S] : scalar_t(0));
+  Vec opt_scalar = Vec(S > 0 ? c10::load((scalar_t*)data[S]) : scalar_t(0));
   int64_t i = 0;
   for (; i <= n - 2 * Vec::size(); i += 2 * Vec::size()) {
     auto args1 = dereference_vec<traits>(&data[1], opt_scalar, S, i);
@@ -271,7 +272,7 @@ struct VectorizedLoop2d {
     const int64_t *outer_strides = &strides[ntensors];
 
     if (is_contiguous<traits>(strides)) {
-      for (const auto i C10_UNUSED : c10::irange(size1)) {
+      for ([[maybe_unused]] const auto i : c10::irange(size1)) {
         vectorized_loop(data.data(), size0, 0, op, vop);
         advance(data, outer_strides);
       }
@@ -279,12 +280,12 @@ struct VectorizedLoop2d {
       using Indices = std::make_index_sequence<traits::arity>;
       unroll_contiguous_scalar_checks<traits>(strides, Indices{}, [&](size_t idx) {
         if (idx) {
-          for (const auto i C10_UNUSED : c10::irange(size1)) {
+          for ([[maybe_unused]] const auto i : c10::irange(size1)) {
             vectorized_loop(data.data(), size0, idx, op, vop);
             advance(data, outer_strides);
           }
         } else {
-          for (const auto i C10_UNUSED : c10::irange(size1)) {
+          for ([[maybe_unused]] const auto i : c10::irange(size1)) {
             basic_loop(data.data(), strides, 0, size0, op);
             advance(data, outer_strides);
           }
