@@ -5,34 +5,34 @@
 #include <ATen/native/TensorIterator.h>
 #include <ATen/native/cpu/Loops.h>
 #include <c10/core/Scalar.h>
-
+#include <ATen/cpu/vec/functional.h>
 namespace at::native {
 namespace {
 
 static void addcmul_cpu_kernel(TensorIteratorBase& iter, const Scalar& value) {
   ScalarType dtype = iter.common_dtype();
-  if (dtype == kBFloat16) {
-    float float_val = value.to<float>();
-    auto float_vec = Vectorized<float>(float_val);
-    cpu_kernel_vec(
-        iter,
-        [=](BFloat16 self_val, BFloat16 t1_val, BFloat16 t2_val) -> BFloat16 {
-          return float(self_val) + float_val * float(t1_val) * float(t2_val);
-        },
-        [=](Vectorized<BFloat16> self_vec,
-          Vectorized<BFloat16> t1_vec,
-          Vectorized<BFloat16> t2_vec) {
-          Vectorized<float> self_vec0, self_vec1;
-          std::tie(self_vec0, self_vec1) = convert_bfloat16_float(self_vec);
-          Vectorized<float> t1_vec0, t1_vec1, t2_vec0, t2_vec1;
-          std::tie(t1_vec0, t1_vec1) = convert_bfloat16_float(t1_vec);
-          std::tie(t2_vec0, t2_vec1) = convert_bfloat16_float(t2_vec);
-          self_vec0 = self_vec0 + float_vec * t1_vec0 * t2_vec0;
-          self_vec1 = self_vec1 + float_vec * t1_vec1 * t2_vec1;
-          return convert_float_bfloat16(self_vec0, self_vec1);
-        });
+  if (at::isReducedFloatingType(dtype)) {
+    AT_DISPATCH_REDUCED_FLOATING_TYPES(dtype, "addcmul_cpu_out", [&]() {
+      float float_val = value.to<float>();
+      auto float_vec = Vectorized<float>(float_val);
+      cpu_kernel_vec(
+          iter,
+          [=](scalar_t self_val, scalar_t t1_val, scalar_t t2_val) -> scalar_t {
+            return float(self_val) + float_val * float(t1_val) * float(t2_val);
+          },
+          [=](Vectorized<scalar_t> self_vec,
+            Vectorized<scalar_t> t1_vec,
+            Vectorized<scalar_t> t2_vec) -> Vectorized<scalar_t> {
+            auto [self_vec0, self_vec1] = convert_to_float<scalar_t>(self_vec);
+            auto [t1_vec0, t1_vec1] = convert_to_float<scalar_t>(t1_vec);
+            auto [t2_vec0, t2_vec1] = convert_to_float<scalar_t>(t2_vec);
+            self_vec0 = self_vec0 + float_vec * t1_vec0 * t2_vec0;
+            self_vec1 = self_vec1 + float_vec * t1_vec1 * t2_vec1;
+            return convert_from_float<scalar_t>(self_vec0, self_vec1);
+          });
+    });
   } else {
-    AT_DISPATCH_ALL_TYPES_AND_COMPLEX_AND2(at::ScalarType::ComplexHalf, at::ScalarType::Half,
+    AT_DISPATCH_ALL_TYPES_AND_COMPLEX_AND(at::ScalarType::ComplexHalf,
                                            dtype, "addcmul_cpu_out", [&] {
       scalar_t scalar_val = value.to<scalar_t>();
       auto scalar_vec = Vectorized<scalar_t>(scalar_val);
@@ -52,26 +52,26 @@ static void addcmul_cpu_kernel(TensorIteratorBase& iter, const Scalar& value) {
 
 static void addcdiv_cpu_kernel(TensorIteratorBase& iter, const Scalar& value) {
   ScalarType dtype = iter.common_dtype();
-  if (dtype == kBFloat16) {
-    float float_val = value.to<float>();
-    auto float_vec = Vectorized<float>(float_val);
-    cpu_kernel_vec(
-        iter,
-        [=](BFloat16 self_val, BFloat16 t1_val, BFloat16 t2_val) -> BFloat16 {
-          return float(self_val) + float_val * float(t1_val) / float(t2_val);
-        },
-        [=](Vectorized<BFloat16> self_vec,
-            Vectorized<BFloat16> t1_vec,
-            Vectorized<BFloat16> t2_vec) {
-            Vectorized<float> self_vec0, self_vec1;
-          std::tie(self_vec0, self_vec1) = convert_bfloat16_float(self_vec);
-          Vectorized<float> t1_vec0, t1_vec1, t2_vec0, t2_vec1;
-          std::tie(t1_vec0, t1_vec1) = convert_bfloat16_float(t1_vec);
-          std::tie(t2_vec0, t2_vec1) = convert_bfloat16_float(t2_vec);
-          self_vec0 = self_vec0 + float_vec * t1_vec0 / t2_vec0;
-          self_vec1 = self_vec1 + float_vec * t1_vec1 / t2_vec1;
-          return convert_float_bfloat16(self_vec0, self_vec1);
-        });
+  if (at::isReducedFloatingType(dtype)) {
+    AT_DISPATCH_REDUCED_FLOATING_TYPES(dtype, "addcdiv_cpu_out", [&]() {
+      float float_val = value.to<float>();
+      auto float_vec = Vectorized<float>(float_val);
+      cpu_kernel_vec(
+          iter,
+          [=](scalar_t self_val, scalar_t t1_val, scalar_t t2_val) -> scalar_t {
+            return float(self_val) + float_val * float(t1_val) / float(t2_val);
+          },
+          [=](Vectorized<scalar_t> self_vec,
+              Vectorized<scalar_t> t1_vec,
+              Vectorized<scalar_t> t2_vec) -> Vectorized<scalar_t> {
+              auto [self_vec0, self_vec1] = convert_to_float<scalar_t>(self_vec);
+              auto [t1_vec0, t1_vec1] = convert_to_float<scalar_t>(t1_vec);
+              auto [t2_vec0, t2_vec1] = convert_to_float<scalar_t>(t2_vec);
+              self_vec0 = self_vec0 + float_vec * t1_vec0 / t2_vec0;
+              self_vec1 = self_vec1 + float_vec * t1_vec1 / t2_vec1;
+              return convert_from_float<scalar_t>(self_vec0, self_vec1);
+          });
+    });
   } else {
     AT_DISPATCH_ALL_TYPES_AND_COMPLEX(dtype, "addcdiv_cpu_out", [&] {
       scalar_t scalar_val = value.to<scalar_t>();
@@ -117,10 +117,9 @@ static void smooth_l1_backward_cpu_kernel(TensorIterator& iter, const Scalar& no
         // 1        if  x >= beta
         // -1       if x <= -beta
         // x / beta if |x| < beta
-        Vectorized<float> input0, input1, target0, target1, grad_output0, grad_output1;
-        std::tie(input0, input1) = convert_bfloat16_float(input);
-        std::tie(target0, target1) = convert_bfloat16_float(target);
-        std::tie(grad_output0, grad_output1) = convert_bfloat16_float(grad_output);
+        auto [input0, input1] = convert_bfloat16_float(input);
+        auto [target0, target1] = convert_bfloat16_float(target);
+        auto [grad_output0, grad_output1] = convert_bfloat16_float(grad_output);
         auto x = input0 - target0;
         auto pos_or_neg_1_vec = Vectorized<float>::blendv(
             neg_1_vec, pos_1_vec, x > zero_vec);
@@ -218,7 +217,7 @@ static void huber_backward_cpu_kernel(TensorIterator& iter, const Scalar& norm,
 
 static void mse_backward_cpu_kernel(TensorIterator& iter, const Scalar& value) {
   ScalarType dtype = iter.dtype(0);
-  AT_DISPATCH_ALL_TYPES(dtype, "mse_backward_cpu_out", [&] {
+  AT_DISPATCH_ALL_TYPES_AND2(kHalf, kBFloat16, dtype, "mse_backward_cpu_out", [&] {
     scalar_t scalar_val = value.to<scalar_t>();
     auto scalar_vec = Vectorized<scalar_t>(scalar_val);
     cpu_kernel_vec(
@@ -236,10 +235,10 @@ static void mse_backward_cpu_kernel(TensorIterator& iter, const Scalar& value) {
 
 } // anonymous namespace
 
-REGISTER_DISPATCH(addcmul_stub, &addcmul_cpu_kernel);
-REGISTER_DISPATCH(addcdiv_stub, &addcdiv_cpu_kernel);
-REGISTER_DISPATCH(smooth_l1_backward_stub, &smooth_l1_backward_cpu_kernel);
-REGISTER_DISPATCH(huber_backward_stub, &huber_backward_cpu_kernel);
-REGISTER_DISPATCH(mse_backward_stub, &mse_backward_cpu_kernel);
+REGISTER_DISPATCH(addcmul_stub, &addcmul_cpu_kernel)
+REGISTER_DISPATCH(addcdiv_stub, &addcdiv_cpu_kernel)
+REGISTER_DISPATCH(smooth_l1_backward_stub, &smooth_l1_backward_cpu_kernel)
+REGISTER_DISPATCH(huber_backward_stub, &huber_backward_cpu_kernel)
+REGISTER_DISPATCH(mse_backward_stub, &mse_backward_cpu_kernel)
 
 } // namespace at::native
