@@ -34,11 +34,11 @@ public:
   template <typename scalar_t>
   constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
     using opmath_t = at::opmath_type<scalar_t>;
-    *self_data *= opmath_t(*src_data);
+    *self_data *= opmath_t(c10::load(src_data));
   }
 
   constexpr void operator() (bool * self_data, bool * src_data) const {
-    *self_data = *self_data && *src_data;
+    *self_data = c10::load(self_data) && c10::load(src_data);
   }
 };
 static ReduceMultiply reduce_multiply;
@@ -48,7 +48,7 @@ public:
   template <typename scalar_t>
   constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
     using opmath_t = at::opmath_type<scalar_t>;
-    *self_data += opmath_t(*src_data);
+    *self_data += opmath_t(c10::load(src_data));
   }
 };
 static ReduceAdd reduce_add;
@@ -58,7 +58,7 @@ public:
   template <typename scalar_t>
   constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
     using opmath_t = at::opmath_type<scalar_t>;
-    *self_data += opmath_t(*src_data);
+    *self_data += opmath_t(c10::load(src_data));
   }
 };
 static ReduceMean reduce_mean;
@@ -68,7 +68,9 @@ public:
   template <typename scalar_t>
   constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
     using opmath_t = at::opmath_type<scalar_t>;
-    *self_data = at::_isnan<scalar_t>(*src_data) ? opmath_t(*src_data) : std::max(*self_data, opmath_t(*src_data));
+    auto self_value = c10::load(self_data);
+    auto src_value = c10::load(src_data);
+    *self_data = at::_isnan<scalar_t>(src_value) ? opmath_t(src_value) : std::max(self_value, opmath_t(src_value));
   }
 };
 static ReduceMaximum reduce_maximum;
@@ -78,7 +80,9 @@ public:
   template <typename scalar_t>
   constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
     using opmath_t = at::opmath_type<scalar_t>;
-    *self_data = at::_isnan<scalar_t>(*src_data) ? opmath_t(*src_data) : std::min(*self_data, opmath_t(*src_data));
+    auto self_value = c10::load(self_data);
+    auto src_value = c10::load(src_data);
+    *self_data = at::_isnan<scalar_t>(src_value) ? opmath_t(src_value) : std::min(self_value, opmath_t(src_value));
   }
 };
 static ReduceMinimum reduce_minimum;
@@ -88,7 +92,7 @@ public:
   template <typename scalar_t>
   constexpr void operator() (at::opmath_type<scalar_t> * self_data, scalar_t * src_data) const {
     using opmath_t = at::opmath_type<scalar_t>;
-    *self_data = opmath_t(*src_data);
+    *self_data = opmath_t(c10::load(src_data));
   }
 };
 static TensorAssign tensor_assign;
@@ -183,8 +187,7 @@ struct cpu_scatter_gather_base_kernel {
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
-      // NOLINTNEXTLINE(bugprone-argument-comment)
-      .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
+      .declare_static_shape(index.sizes(), /*squash_dims=*/dim)
       .add_output(buffer)
       .add_const_input(index)
       .build();
@@ -215,7 +218,7 @@ struct cpu_scatter_gather_base_kernel {
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
           if (dim== buffer.dim() - 1) {
-            for (const auto nelem C10_UNUSED : c10::irange(n)) {
+            for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
               loop_func.template operator()<scalar_t, func_t>(
@@ -232,7 +235,7 @@ struct cpu_scatter_gather_base_kernel {
             for (const auto i : c10::irange(index_dim_size)) {
               auto* self_data = self_data_bytes;
               auto* index_data = (char*)((int64_t*)index_data_bytes + i * index_dim_stride);
-              for (const auto nelem C10_UNUSED : c10::irange(n)) {
+              for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
                 int64_t idx_dim = *(int64_t*)index_data;
                 // we are not putting idx_dim in the error message because it disables
                 // loop optimization in clang-7
@@ -270,8 +273,7 @@ struct cpu_scatter_gather_base_kernel {
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
-      // NOLINTNEXTLINE(bugprone-argument-comment)
-      .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
+      .declare_static_shape(index.sizes(), /*squash_dims=*/dim)
       .add_output(buffer)
       .add_const_input(src)
       .add_const_input(index)
@@ -306,7 +308,7 @@ struct cpu_scatter_gather_base_kernel {
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
           if (dim== buffer.dim() - 1) {
-            for (const auto nelem C10_UNUSED : c10::irange(n)) {
+            for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
               loop_func.template operator()<scalar_t, func_t>(
@@ -327,7 +329,7 @@ struct cpu_scatter_gather_base_kernel {
               auto* self_data = self_data_bytes;
               auto* index_data = (char*)((int64_t*)index_data_bytes + i * index_dim_stride);
               auto* src_data = src_data_bytes;
-              for (const auto nelem C10_UNUSED : c10::irange(n)) {
+              for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
                 int64_t idx_dim = *(int64_t*)index_data;
                 // we are not putting idx_dim in the error message because it disables
                 // loop optimization in clang-7
@@ -366,8 +368,7 @@ struct cpu_scatter_gather_base_kernel {
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
-      // NOLINTNEXTLINE(bugprone-argument-comment)
-      .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
+      .declare_static_shape(index.sizes(), /*squash_dims=*/dim)
       .add_output(buffer)
       .add_const_input(src)
       .add_const_input(index)
@@ -402,7 +403,7 @@ struct cpu_scatter_gather_base_kernel {
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
           if (dim== buffer.dim() - 1) {
-            for (const auto nelem C10_UNUSED : c10::irange(n)) {
+            for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
               loop_func.template operator()<scalar_t, ReduceMean>(
@@ -423,7 +424,7 @@ struct cpu_scatter_gather_base_kernel {
               auto* self_data = self_data_bytes;
               auto* index_data = (char*)((int64_t*)index_data_bytes + i * index_dim_stride);
               auto* src_data = src_data_bytes;
-              for (const auto nelem C10_UNUSED : c10::irange(n)) {
+              for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
                 int64_t idx_dim = *(int64_t*)index_data;
                 // we are not putting idx_dim in the error message because it disables
                 // loop optimization in clang-7
@@ -461,8 +462,7 @@ struct cpu_scatter_gather_base_kernel {
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
-      // NOLINTNEXTLINE(bugprone-argument-comment)
-      .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
+      .declare_static_shape(index.sizes(), /*squash_dims=*/dim)
       .add_output(buffer)
       .add_const_input(src)
       .add_const_input(index)
@@ -497,7 +497,7 @@ struct cpu_scatter_gather_base_kernel {
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
           if (dim== buffer.dim() - 1) {
-            for (const auto nelem C10_UNUSED : c10::irange(n)) {
+            for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
               loop_func.template operator()<scalar_t, ReduceMaximum>(
@@ -518,7 +518,7 @@ struct cpu_scatter_gather_base_kernel {
               auto* self_data = self_data_bytes;
               auto* index_data = (char*)((int64_t*)index_data_bytes + i * index_dim_stride);
               auto* src_data = src_data_bytes;
-              for (const auto nelem C10_UNUSED : c10::irange(n)) {
+              for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
                 int64_t idx_dim = *(int64_t*)index_data;
                 // we are not putting idx_dim in the error message because it disables
                 // loop optimization in clang-7
@@ -557,8 +557,7 @@ struct cpu_scatter_gather_base_kernel {
     auto iter = TensorIteratorConfig()
       .check_all_same_dtype(false)
       .resize_outputs(false)
-      // NOLINTNEXTLINE(bugprone-argument-comment)
-      .declare_static_shape(index.sizes(), /*squash_dim=*/dim)
+      .declare_static_shape(index.sizes(), /*squash_dims=*/dim)
       .add_output(buffer)
       .add_const_input(src)
       .add_const_input(index)
@@ -593,7 +592,7 @@ struct cpu_scatter_gather_base_kernel {
           // vs dim-TensorIterator loop order depending on
           // whether dim is the last dimension
           if (dim== buffer.dim() - 1) {
-            for (const auto nelem C10_UNUSED : c10::irange(n)) {
+            for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
               // dim loop is a separate code block
               // for better performance
               loop_func.template operator()<scalar_t, ReduceMinimum>(
@@ -614,7 +613,7 @@ struct cpu_scatter_gather_base_kernel {
               auto* self_data = self_data_bytes;
               auto* index_data = (char*)((int64_t*)index_data_bytes + i * index_dim_stride);
               auto* src_data = src_data_bytes;
-              for (const auto nelem C10_UNUSED : c10::irange(n)) {
+              for ([[maybe_unused]] const auto nelem : c10::irange(n)) {
                 int64_t idx_dim = *(int64_t*)index_data;
                 // we are not putting idx_dim in the error message because it disables
                 // loop optimization in clang-7
@@ -955,17 +954,17 @@ void scatter_scalar_reduce_cpu_kernel(const Tensor& self, const int64_t dim, con
 
 } // anonymous namespace
 
-REGISTER_DISPATCH(gather_stub, &gather_cpu_kernel);
-REGISTER_DISPATCH(scatter_stub, &scatter_cpu_kernel);
-REGISTER_DISPATCH(scatter_fill_stub, &scatter_fill_cpu_kernel);
-REGISTER_DISPATCH(scatter_add_stub, &scatter_add_cpu_kernel);
-REGISTER_DISPATCH(scatter_reduce_stub, &scatter_reduce_cpu_kernel);
-REGISTER_DISPATCH(scatter_scalar_reduce_stub, &scatter_scalar_reduce_cpu_kernel);
-REGISTER_DISPATCH(scatter_reduce_two_stub, &scatter_reduce_two_cpu_kernel);
+REGISTER_DISPATCH(gather_stub, &gather_cpu_kernel)
+REGISTER_DISPATCH(scatter_stub, &scatter_cpu_kernel)
+REGISTER_DISPATCH(scatter_fill_stub, &scatter_fill_cpu_kernel)
+REGISTER_DISPATCH(scatter_add_stub, &scatter_add_cpu_kernel)
+REGISTER_DISPATCH(scatter_reduce_stub, &scatter_reduce_cpu_kernel)
+REGISTER_DISPATCH(scatter_scalar_reduce_stub, &scatter_scalar_reduce_cpu_kernel)
+REGISTER_DISPATCH(scatter_reduce_two_stub, &scatter_reduce_two_cpu_kernel)
 
 // fast paths for GNN usage
-REGISTER_DISPATCH(scatter_add_expanded_index_stub, &scatter_add_expanded_index_kernel);
-REGISTER_DISPATCH(scatter_reduce_expanded_index_stub, &scatter_reduce_expanded_index_kernel);
-REGISTER_DISPATCH(gather_expanded_index_stub, &gather_expanded_index_kernel);
+REGISTER_DISPATCH(scatter_add_expanded_index_stub, &scatter_add_expanded_index_kernel)
+REGISTER_DISPATCH(scatter_reduce_expanded_index_stub, &scatter_reduce_expanded_index_kernel)
+REGISTER_DISPATCH(gather_expanded_index_stub, &gather_expanded_index_kernel)
 
 } // namespace at::native
