@@ -7,7 +7,7 @@
 #define USE_CUSOLVER_64_BIT
 #endif
 
-#if defined(CUDART_VERSION) || defined(ROCM_VERSION) && ROCM_VERSION >= 50300
+#if defined(CUDART_VERSION) || defined(USE_ROCM)
 
 namespace at {
 namespace cuda {
@@ -18,7 +18,7 @@ namespace solver {
 
 template<class Dtype>
 void getrf(CUDASOLVER_GETRF_ARGTYPES(Dtype)) {
-  TORCH_CHECK(false, "at::cuda::solver::getrf: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::getrf: not implemented");
 }
 template<>
 void getrf<float>(CUDASOLVER_GETRF_ARGTYPES(float));
@@ -35,7 +35,7 @@ void getrf<c10::complex<float>>(CUDASOLVER_GETRF_ARGTYPES(c10::complex<float>));
 
 template<class Dtype>
 void getrs(CUDASOLVER_GETRS_ARGTYPES(Dtype)) {
-  TORCH_CHECK(false, "at::cuda::solver::getrs: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::getrs: not implemented");
 }
 template<>
 void getrs<float>(CUDASOLVER_GETRS_ARGTYPES(float));
@@ -51,10 +51,8 @@ void getrs<c10::complex<float>>(CUDASOLVER_GETRS_ARGTYPES(c10::complex<float>));
 
 template <class Dtype>
 void sytrf_bufferSize(CUDASOLVER_SYTRF_BUFFER_ARGTYPES(Dtype)) {
-  TORCH_CHECK(
-      false,
-      "at::cuda::solver::sytrf_bufferSize: not implemented for ",
-      typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype),
+      "at::cuda::solver::sytrf_bufferSize: not implemented");
 }
 template <>
 void sytrf_bufferSize<float>(CUDASOLVER_SYTRF_BUFFER_ARGTYPES(float));
@@ -73,10 +71,8 @@ void sytrf_bufferSize<c10::complex<float>>(
 
 template <class Dtype>
 void sytrf(CUDASOLVER_SYTRF_ARGTYPES(Dtype)) {
-  TORCH_CHECK(
-      false,
-      "at::cuda::solver::sytrf: not implemented for ",
-      typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype),
+      "at::cuda::solver::sytrf: not implemented");
 }
 template <>
 void sytrf<float>(CUDASOLVER_SYTRF_ARGTYPES(float));
@@ -93,7 +89,7 @@ void sytrf<c10::complex<float>>(CUDASOLVER_SYTRF_ARGTYPES(c10::complex<float>));
 
 template<class Dtype>
 void gesvd_buffersize(CUDASOLVER_GESVD_BUFFERSIZE_ARGTYPES()) {
-  TORCH_CHECK(false, "at::cuda::solver::gesvd_buffersize: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::gesvd_buffersize: not implemented");
 }
 template<>
 void gesvd_buffersize<float>(CUDASOLVER_GESVD_BUFFERSIZE_ARGTYPES());
@@ -111,7 +107,7 @@ void gesvd_buffersize<c10::complex<double>>(CUDASOLVER_GESVD_BUFFERSIZE_ARGTYPES
 
 template<class Dtype, class Vtype>
 void gesvd(CUDASOLVER_GESVD_ARGTYPES(Dtype, Vtype)) {
-  TORCH_CHECK(false, "at::cuda::solver::gesvd: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::gesvd: not implemented");
 }
 template<>
 void gesvd<float>(CUDASOLVER_GESVD_ARGTYPES(float, float));
@@ -129,7 +125,7 @@ void gesvd<c10::complex<double>>(CUDASOLVER_GESVD_ARGTYPES(c10::complex<double>,
 
 template<class Dtype, class Vtype>
 void gesvdj_buffersize(CUDASOLVER_GESVDJ_BUFFERSIZE_ARGTYPES(Dtype, Vtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::gesvdj_buffersize: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::gesvdj_buffersize: not implemented");
 }
 template<>
 void gesvdj_buffersize<float>(CUDASOLVER_GESVDJ_BUFFERSIZE_ARGTYPES(float, float));
@@ -147,7 +143,7 @@ void gesvdj_buffersize<c10::complex<double>>(CUDASOLVER_GESVDJ_BUFFERSIZE_ARGTYP
 
 template<class Dtype, class Vtype>
 void gesvdj(CUDASOLVER_GESVDJ_ARGTYPES(Dtype, Vtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::gesvdj: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::gesvdj: not implemented");
 }
 template<>
 void gesvdj<float>(CUDASOLVER_GESVDJ_ARGTYPES(float, float));
@@ -165,7 +161,7 @@ void gesvdj<c10::complex<double>>(CUDASOLVER_GESVDJ_ARGTYPES(c10::complex<double
 
 template<class Dtype, class Vtype>
 void gesvdjBatched(CUDASOLVER_GESVDJ_BATCHED_ARGTYPES(Dtype, Vtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::gesvdj: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::gesvdj: not implemented");
 }
 template<>
 void gesvdjBatched<float>(CUDASOLVER_GESVDJ_BATCHED_ARGTYPES(float, float));
@@ -183,7 +179,7 @@ void gesvdjBatched<c10::complex<double>>(CUDASOLVER_GESVDJ_BATCHED_ARGTYPES(c10:
 
 template<class Dtype, class Vtype>
 void gesvdaStridedBatched_buffersize(CUDASOLVER_GESVDA_STRIDED_BATCHED_BUFFERSIZE_ARGTYPES(Dtype, Vtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::gesvdaStridedBatched_buffersize: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::gesvdaStridedBatched_buffersize: not implemented");
 }
 template<>
 void gesvdaStridedBatched_buffersize<float>(CUDASOLVER_GESVDA_STRIDED_BATCHED_BUFFERSIZE_ARGTYPES(float, float));
@@ -203,7 +199,7 @@ void gesvdaStridedBatched_buffersize<c10::complex<double>>(CUDASOLVER_GESVDA_STR
 
 template<class Dtype, class Vtype>
 void gesvdaStridedBatched(CUDASOLVER_GESVDA_STRIDED_BATCHED_ARGTYPES(Dtype, Vtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::gesvdaStridedBatched: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::gesvdaStridedBatched: not implemented");
 }
 template<>
 void gesvdaStridedBatched<float>(CUDASOLVER_GESVDA_STRIDED_BATCHED_ARGTYPES(float, float));
@@ -220,7 +216,7 @@ void gesvdaStridedBatched<c10::complex<double>>(CUDASOLVER_GESVDA_STRIDED_BATCHE
 
 template<class Dtype>
 void potrf(CUDASOLVER_POTRF_ARGTYPES(Dtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::potrf: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::potrf: not implemented");
 }
 template<>
 void potrf<float>(CUDASOLVER_POTRF_ARGTYPES(float));
@@ -237,7 +233,7 @@ void potrf<c10::complex<double>>(CUDASOLVER_POTRF_ARGTYPES(c10::complex<double>)
 
 template<class Dtype>
 void potrf_buffersize(CUDASOLVER_POTRF_BUFFERSIZE_ARGTYPES(Dtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::potrf_buffersize: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::potrf_buffersize: not implemented");
 }
 template<>
 void potrf_buffersize<float>(CUDASOLVER_POTRF_BUFFERSIZE_ARGTYPES(float));
@@ -254,7 +250,7 @@ void potrf_buffersize<c10::complex<double>>(CUDASOLVER_POTRF_BUFFERSIZE_ARGTYPES
 
 template<class Dtype>
 void potrfBatched(CUDASOLVER_POTRF_BATCHED_ARGTYPES(Dtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::potrfBatched: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::potrfBatched: not implemented");
 }
 template<>
 void potrfBatched<float>(CUDASOLVER_POTRF_BATCHED_ARGTYPES(float));
@@ -270,10 +266,8 @@ void potrfBatched<c10::complex<double>>(CUDASOLVER_POTRF_BATCHED_ARGTYPES(c10::c
 
 template <class scalar_t>
 void geqrf_bufferSize(CUDASOLVER_GEQRF_BUFFERSIZE_ARGTYPES(scalar_t)) {
-  TORCH_CHECK(
-      false,
-      "at::cuda::solver::geqrf_bufferSize: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::geqrf_bufferSize: not implemented");
 }
 template <>
 void geqrf_bufferSize<float>(CUDASOLVER_GEQRF_BUFFERSIZE_ARGTYPES(float));
@@ -292,10 +286,8 @@ void geqrf_bufferSize<c10::complex<double>>(
 
 template <class scalar_t>
 void geqrf(CUDASOLVER_GEQRF_ARGTYPES(scalar_t)) {
-  TORCH_CHECK(
-      false,
-      "at::cuda::solver::geqrf: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::geqrf: not implemented");
 }
 template <>
 void geqrf<float>(CUDASOLVER_GEQRF_ARGTYPES(float));
@@ -312,7 +304,7 @@ void geqrf<c10::complex<double>>(
 
 template<class Dtype>
 void potrs(CUDASOLVER_POTRS_ARGTYPES(Dtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::potrs: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::potrs: not implemented");
 }
 template<>
 void potrs<float>(CUDASOLVER_POTRS_ARGTYPES(float));
@@ -329,7 +321,7 @@ void potrs<c10::complex<double>>(CUDASOLVER_POTRS_ARGTYPES(c10::complex<double>)
 
 template<class Dtype>
 void potrsBatched(CUDASOLVER_POTRS_BATCHED_ARGTYPES(Dtype)) {
-  TORCH_INTERNAL_ASSERT(false, "at::cuda::solver::potrsBatched: not implemented for ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::potrsBatched: not implemented");
 }
 template<>
 void potrsBatched<float>(CUDASOLVER_POTRS_BATCHED_ARGTYPES(float));
@@ -347,10 +339,7 @@ void potrsBatched<c10::complex<double>>(CUDASOLVER_POTRS_BATCHED_ARGTYPES(c10::c
 
 template <class Dtype>
 void orgqr_buffersize(CUDASOLVER_ORGQR_BUFFERSIZE_ARGTYPES(Dtype)) {
-  TORCH_CHECK(
-      false,
-      "at::cuda::solver::orgqr_buffersize: not implemented for ",
-      typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::orgqr_buffersize: not implemented");
 }
 template <>
 void orgqr_buffersize<float>(CUDASOLVER_ORGQR_BUFFERSIZE_ARGTYPES(float));
@@ -368,10 +357,7 @@ void orgqr_buffersize<c10::complex<double>>(CUDASOLVER_ORGQR_BUFFERSIZE_ARGTYPES
 
 template <class Dtype>
 void orgqr(CUDASOLVER_ORGQR_ARGTYPES(Dtype)) {
-  TORCH_CHECK(
-      false,
-      "at::cuda::solver::orgqr: not implemented for ",
-      typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "at::cuda::solver::orgqr: not implemented");
 }
 template <>
 void orgqr<float>(CUDASOLVER_ORGQR_ARGTYPES(float));
@@ -389,10 +375,8 @@ void orgqr<c10::complex<double>>(CUDASOLVER_ORGQR_ARGTYPES(c10::complex<double>)
 
 template <class Dtype>
 void ormqr_bufferSize(CUDASOLVER_ORMQR_BUFFERSIZE_ARGTYPES(Dtype)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::ormqr_bufferSize: not implemented for ",
-      typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype),
+      "at::cuda::solver::ormqr_bufferSize: not implemented");
 }
 template <>
 void ormqr_bufferSize<float>(CUDASOLVER_ORMQR_BUFFERSIZE_ARGTYPES(float));
@@ -412,10 +396,8 @@ void ormqr_bufferSize<c10::complex<double>>(
 
 template <class Dtype>
 void ormqr(CUDASOLVER_ORMQR_ARGTYPES(Dtype)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::ormqr: not implemented for ",
-      typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype),
+      "at::cuda::solver::ormqr: not implemented");
 }
 template <>
 void ormqr<float>(CUDASOLVER_ORMQR_ARGTYPES(float));
@@ -431,7 +413,8 @@ void ormqr<c10::complex<double>>(
 
 template<class Dtype>
 cudaDataType get_cusolver_datatype() {
-  TORCH_CHECK(false, "cusolver doesn't support data type ", typeid(Dtype).name());
+  static_assert(false&&sizeof(Dtype), "cusolver doesn't support data type");
+  return {};
 }
 template<> cudaDataType get_cusolver_datatype<float>();
 template<> cudaDataType get_cusolver_datatype<double>();
@@ -459,10 +442,8 @@ void xpotrs(
 
 template <class scalar_t, class value_t = scalar_t>
 void syevd_bufferSize(CUDASOLVER_SYEVD_BUFFERSIZE_ARGTYPES(scalar_t, value_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::syevd_bufferSize: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::syevd_bufferSize: not implemented");
 }
 
 template <>
@@ -485,10 +466,8 @@ void syevd_bufferSize<c10::complex<double>, double>(
 
 template <class scalar_t, class value_t = scalar_t>
 void syevd(CUDASOLVER_SYEVD_ARGTYPES(scalar_t, value_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::syevd: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::syevd: not implemented");
 }
 
 template <>
@@ -509,10 +488,8 @@ void syevd<c10::complex<double>, double>(
 
 template <class scalar_t, class value_t = scalar_t>
 void syevj_bufferSize(CUDASOLVER_SYEVJ_BUFFERSIZE_ARGTYPES(scalar_t, value_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::syevj_bufferSize: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::syevj_bufferSize: not implemented");
 }
 
 template <>
@@ -535,10 +512,7 @@ void syevj_bufferSize<c10::complex<double>, double>(
 
 template <class scalar_t, class value_t = scalar_t>
 void syevj(CUDASOLVER_SYEVJ_ARGTYPES(scalar_t, value_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::syevj: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t), "at::cuda::solver::syevj: not implemented");
 }
 
 template <>
@@ -560,10 +534,8 @@ void syevj<c10::complex<double>, double>(
 template <class scalar_t, class value_t = scalar_t>
 void syevjBatched_bufferSize(
     CUDASOLVER_SYEVJ_BATCHED_BUFFERSIZE_ARGTYPES(scalar_t, value_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::syevjBatched_bufferSize: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::syevjBatched_bufferSize: not implemented");
 }
 
 template <>
@@ -586,10 +558,8 @@ void syevjBatched_bufferSize<c10::complex<double>, double>(
 
 template <class scalar_t, class value_t = scalar_t>
 void syevjBatched(CUDASOLVER_SYEVJ_BATCHED_ARGTYPES(scalar_t, value_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::syevjBatched: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::syevjBatched: not implemented");
 }
 
 template <>
@@ -612,10 +582,8 @@ void syevjBatched<c10::complex<double>, double>(
 
 template <class scalar_t>
 void xgeqrf_bufferSize(CUDASOLVER_XGEQRF_BUFFERSIZE_ARGTYPES(scalar_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::xgeqrf_bufferSize: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::xgeqrf_bufferSize: not implemented");
 }
 
 template <>
@@ -637,10 +605,7 @@ void xgeqrf_bufferSize<c10::complex<double>>(
 
 template <class scalar_t>
 void xgeqrf(CUDASOLVER_XGEQRF_ARGTYPES(scalar_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::xgeqrf: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t), "at::cuda::solver::xgeqrf: not implemented");
 }
 
 template <>
@@ -663,10 +628,8 @@ void xgeqrf<c10::complex<double>>(
 template <class scalar_t, class value_t = scalar_t>
 void xsyevd_bufferSize(
     CUDASOLVER_XSYEVD_BUFFERSIZE_ARGTYPES(scalar_t, value_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::xsyevd_bufferSize: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::xsyevd_bufferSize: not implemented");
 }
 
 template <>
@@ -691,10 +654,8 @@ void xsyevd_bufferSize<c10::complex<double>, double>(
 
 template <class scalar_t, class value_t = scalar_t>
 void xsyevd(CUDASOLVER_XSYEVD_ARGTYPES(scalar_t, value_t)) {
-  TORCH_INTERNAL_ASSERT(
-      false,
-      "at::cuda::solver::xsyevd: not implemented for ",
-      typeid(scalar_t).name());
+  static_assert(false&&sizeof(scalar_t),
+      "at::cuda::solver::xsyevd: not implemented");
 }
 
 template <>
