@@ -1,6 +1,12 @@
-#include <ATen/ATen.h>
+#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
+#include <ATen/core/Tensor.h>
 #include <ATen/Config.h>
+
+#ifndef AT_PER_OPERATOR_HEADERS
 #include <ATen/NativeFunctions.h>
+#else
+#include <ATen/ops/_softmax_native.h>         // for mkldnn_softmax
+#endif
 
 #if !AT_MKLDNN_ENABLED()
 
@@ -21,8 +27,7 @@ Tensor mkldnn_softmax(
 
 #include <ATen/native/mkldnn/MKLDNNCommon.h>
 
-namespace at {
-namespace native {
+namespace at::native {
 
 Tensor mkldnn_softmax(
     const Tensor& self,
@@ -39,7 +44,6 @@ Tensor mkldnn_softmax(
                                  self.options().device_opt());
 }
 
-} // namespace native
 } // namespace at
 
 #endif // AT_MKLDNN_ENABLED
