@@ -1,6 +1,6 @@
 #include <ATen/NestedTensorImpl.h>
 #include <ATen/native/nested/NestedTensorUtils.h>
-#include <c10/util/Optional.h>
+#include <optional>
 
 #ifndef AT_PER_OPERATOR_HEADERS
 #include <ATen/NativeFunctions.h>
@@ -12,8 +12,7 @@
 #include <ATen/ops/split_with_sizes_native.h>
 #endif
 
-namespace at {
-namespace native {
+namespace at::native {
 
 /**
  * Thin wrapper around get_nested_sizes that is registered as a native function
@@ -61,7 +60,7 @@ std::vector<int64_t> NestedTensor_get_max_size(const NestedTensorImpl& nt) {
 int64_t get_consistent_last_dim_of_nested_tensor(const NestedTensorImpl& nt) {
   std::optional<int64_t> last_dim = nt.opt_size(-1);
   TORCH_CHECK(
-      last_dim != c10::nullopt,
+      last_dim != std::nullopt,
       "Expected all tensors in nested tensor to have the same trailing dimension, instead last dimension equals: ",
       nt.get_nested_sizes().select(1, -1));
   return *last_dim;
@@ -167,5 +166,4 @@ std::vector<Tensor> split_with_sizes_nested(
   return splits;
 }
 
-} // namespace native
-} // namespace at
+} // namespace at::native
