@@ -400,7 +400,7 @@ enum pytorch_qnnp_status qnnpackConv(
         threadpool);
     if (status != pytorch_qnnp_status_success) {
       pytorch_qnnp_log_error(
-          "failed to run covolution op setup to setup indirection buffer.");
+          "failed to run convolution op setup to setup indirection buffer.");
       return status;
     }
   }
