@@ -3,10 +3,7 @@
 #include <ATen/Tensor.h>
 #include <ATen/core/Scalar.h>
 
-namespace at {
-namespace native {
-namespace sparse {
-namespace impl {
+namespace at::native::sparse::impl {
 
 TORCH_API Tensor& _compressed_row_strided_mm_out(
     const Tensor& compressed_row_sparse,
@@ -45,7 +42,4 @@ void triangular_solve_out_sparse_csr(
     bool unitriangular);
 
 } // namespace cpu
-} // namespace impl
-} // namespace sparse
-} // namespace native
-} // namespace at
+} // namespace at::native::sparse::impl
