@@ -249,9 +249,9 @@ Tensor& addmv_out_sparse_compressed_cuda(
           at::native::scalar_tensor(
               beta,
               self.scalar_type(),
-              c10::nullopt /* layout */,
+              std::nullopt /* layout */,
               at::kCPU,
-              c10::nullopt /* pin_memory */));
+              std::nullopt /* pin_memory */));
     }
   }
 
