@@ -23,11 +23,7 @@
 #include <c10/cuda/CUDACachingAllocator.h>
 #include <c10/util/MaybeOwned.h>
 
-namespace at {
-namespace native {
-namespace sparse {
-namespace impl {
-namespace cuda {
+namespace at::native::sparse::impl::cuda {
 
 namespace {
 
@@ -464,6 +460,7 @@ void block_sparse_mv(
 }
 
 void block_sparse_mm(
+    const Tensor& input,
     const at::sparse_csr::SparseCsrTensor& mat1,
     const Tensor& mat2,
     const Scalar& beta,
@@ -484,9 +481,11 @@ void block_sparse_mm(
   // NOTE: the code below allows arbitrary block sizes
   // and might be potentially faster than cuSPARSE implementation
   // especially for not very sparse inputs.
-  if (mat1.scalar_type() == ScalarType::Half || mat1.scalar_type() == ScalarType::BFloat16) {
+  if (mat1.scalar_type() == ScalarType::Half
+      || mat1.scalar_type() == ScalarType::BFloat16
+      || mat1.scalar_type() == ScalarType::Float) {
     at::native::sparse::impl::_compressed_row_strided_addmm_out(
-        result,
+        input,
         mat1,
         mat2,
         /*beta=*/beta,
@@ -497,6 +496,10 @@ void block_sparse_mm(
     return;
   }
 
+  if (beta.toComplexDouble() != 0. && !result.is_same(input)) {
+    result.copy_(input);
+  }
+
   const cusparseDirection_t block_layout = mat1.values().is_contiguous()
       ? CUSPARSE_DIRECTION_ROW
       : CUSPARSE_DIRECTION_COLUMN;
@@ -579,13 +582,13 @@ void spmm(
     const Scalar& beta,
     const Scalar& alpha,
     const Tensor& result) {
-#if !(AT_USE_CUSPARSE_GENERIC_API() || AT_USE_HIPSPARSE_GENERIC_52_API())
+#if !(AT_USE_CUSPARSE_GENERIC_API() || AT_USE_HIPSPARSE_GENERIC_API())
   addmm_out_legacy(mat1, mat2, beta, alpha, result);
 #else
   c10::MaybeOwned<Tensor> result_ = prepare_dense_matrix_for_cusparse(result);
   c10::MaybeOwned<Tensor> mat2_ = prepare_dense_matrix_for_cusparse(mat2);
 
-  // Here subscript "c" stands for column-major, substript "r" stands for
+  // Here subscript "c" stands for column-major, subscript "r" stands for
   // row-major order Both orders are supported by cuSPARSE. For mixed input we
   // need to cast 'mat2' to order of 'result'. We compute
   // result = mat1 @ op(mat2) + result.
@@ -629,7 +632,7 @@ void spmm(
   auto algorithm = CUSPARSE_SPMM_CSR_ALG2;
 #endif
 
-  auto descB = at::cuda::sparse::CuSparseDnMatDescriptor(
+  auto descB = at::cuda::sparse::CuSparseConstDnMatDescriptor(
       transpose_B ? mat2_->mT() : *mat2_);
   auto descC = at::cuda::sparse::CuSparseDnMatDescriptor(*result_);
 
@@ -652,7 +655,7 @@ void spmm(
             opB,
             &alpha_,
             descA.descriptor(),
-            descB.descriptor(),
+            descB.unsafe_mutable_descriptor(),
             &beta_,
             descC.descriptor(),
             compute_type,
@@ -669,7 +672,7 @@ void spmm(
             opB,
             &alpha_,
             descA.descriptor(),
-            descB.descriptor(),
+            descB.unsafe_mutable_descriptor(),
             &beta_,
             descC.descriptor(),
             compute_type,
@@ -689,13 +692,6 @@ void spgemm(
     const Scalar& beta,
     const Scalar& alpha,
     const at::sparse_csr::SparseCsrTensor& C) {
-#if defined(USE_ROCM) && ROCM_VERSION < 50200
-  TORCH_CHECK(
-      false,
-      "Calling addmm with sparse GPU tensors requires compiling ",
-      "PyTorch with ROCm 5.2+. ",
-      "Please use PyTorch built with newer ROCm version.");
-#else
   // older versions of cusparse on Windows segfault for complex128 dtype
 #if defined(_WIN32) && defined(CUSPARSE_VERSION) && CUSPARSE_VERSION < 11400
   TORCH_CHECK(
@@ -806,8 +802,7 @@ void spgemm(
             buffer2.get()));
 
         // Get how many specified elements are there in C
-        int64_t C_num_rows, C_num_cols, C_nnz;
-        std::tie(C_num_rows, C_num_cols, C_nnz) = descC.get_size();
+        auto [C_num_rows, C_num_cols, C_nnz] = descC.get_size();
 
         TORCH_INTERNAL_ASSERT_DEBUG_ONLY(C_num_rows == m);
         TORCH_INTERNAL_ASSERT_DEBUG_ONLY(C_num_cols == n);
@@ -832,12 +827,12 @@ void spgemm(
             CUSPARSE_SPGEMM_DEFAULT,
             spgemm_desc.descriptor()));
       });
-#endif
 }
 
 } // anonymous namespace
 
 void addmm_out_sparse_csr(
+    const Tensor& input,
     const Tensor& mat1,
     const Tensor& mat2,
     const Scalar& beta,
@@ -853,6 +848,39 @@ void addmm_out_sparse_csr(
   // Valid combinations terminate in a return
   // Invalid combinations are omitted and will fall though to the TORCH check
   // generating an informative error message
+
+  // mm functions that copy input to result when needed (e.g. mm
+  // triton kernels do not require result being initialized with
+  // input):
+  if (mat1.layout() == kSparseBsr) {
+    if (mat2.layout() == kStrided) {
+      if (result.layout() == kStrided)
+        return block_sparse_mm(input, mat1, mat2, beta, alpha, result);
+    }
+  }
+
+  if (mat1.layout() == kStrided) {
+    if (mat2.layout() == kSparseBsc) {
+      if (result.layout() == kStrided) {
+        auto result_t = result.transpose(-2, -1);
+        auto input_t = (result.is_same(input) ? result_t : input.transpose(-2, -1));
+        return block_sparse_mm(
+            input_t,
+            mat2.transpose(-2, -1),
+            mat1.transpose(-2, -1),
+            beta,
+            alpha,
+            result_t);
+      }
+    }
+  }
+
+  // copy input to result:
+  if (beta.toComplexDouble() != 0. && !result.is_same(input)) {
+    result.copy_(input);
+  }
+
+  // mm functions that assume that result contains input:
   if (mat1.layout() == kStrided) {
     if (mat2.layout() == kSparseCsr) {
       if (result.layout() == kStrided) {
@@ -875,16 +903,6 @@ void addmm_out_sparse_csr(
             result.transpose(-2, -1));
       }
     }
-    if (mat2.layout() == kSparseBsc) {
-      if (result.layout() == kStrided) {
-        return block_sparse_mm(
-            mat2.transpose(-2, -1),
-            mat1.transpose(-2, -1),
-            beta,
-            alpha,
-            result.transpose(-2, -1));
-      }
-    }
   }
   if (mat1.layout() == kSparseCsr) {
     if (mat2.layout() == kStrided) {
@@ -933,12 +951,6 @@ void addmm_out_sparse_csr(
       }
     }
   }
-  if (mat1.layout() == kSparseBsr) {
-    if (mat2.layout() == kStrided) {
-      if (result.layout() == kStrided)
-        return block_sparse_mm(mat1, mat2, beta, alpha, result);
-    }
-  }
   TORCH_CHECK(
       false,
       "addmm: computation on CUDA is not implemented for ",
@@ -1406,7 +1418,7 @@ void sampled_addmm_out_sparse_csr(
     const Scalar& beta,
     const Scalar& alpha,
     const at::sparse_csr::SparseCsrTensor& C) {
-#if !(AT_USE_CUSPARSE_GENERIC_SDDMM() || AT_USE_HIPSPARSE_GENERIC_52_API())
+#if !(AT_USE_CUSPARSE_GENERIC_SDDMM() || AT_USE_HIPSPARSE_GENERIC_API())
   TORCH_CHECK(
       false,
       "Calling sampled_addmm with sparse GPU tensors requires compiling ",
@@ -1434,8 +1446,8 @@ void sampled_addmm_out_sparse_csr(
         // ** On entry to cusparseSDDMM_bufferSize(): batched SDDMM is not supported
         // So we need to resort to the for loop
         for (const auto i : c10::irange(batchCount(A))) {
-          auto descA = at::cuda::sparse::CuSparseDnMatDescriptor(*A_, /*batch_offset=*/i);
-          auto descB = at::cuda::sparse::CuSparseDnMatDescriptor(*B_, /*batch_offset=*/i);
+          auto descA = at::cuda::sparse::CuSparseConstDnMatDescriptor(*A_, /*batch_offset=*/i);
+          auto descB = at::cuda::sparse::CuSparseConstDnMatDescriptor(*B_, /*batch_offset=*/i);
           auto descC = at::cuda::sparse::CuSparseSpMatCsrDescriptor(C, /*batch_offset=*/i);
 
           auto beta_ = beta.to<scalar_t>();
@@ -1448,8 +1460,8 @@ void sampled_addmm_out_sparse_csr(
               opA,
               opB,
               &alpha_,
-              descA.descriptor(),
-              descB.descriptor(),
+              descA.unsafe_mutable_descriptor(),
+              descB.unsafe_mutable_descriptor(),
               &beta_,
               descC.descriptor(),
               compute_type,
@@ -1465,8 +1477,8 @@ void sampled_addmm_out_sparse_csr(
               opA,
               opB,
               &alpha_,
-              descA.descriptor(),
-              descB.descriptor(),
+              descA.unsafe_mutable_descriptor(),
+              descB.unsafe_mutable_descriptor(),
               &beta_,
               descC.descriptor(),
               compute_type,
@@ -1478,8 +1490,8 @@ void sampled_addmm_out_sparse_csr(
               opA,
               opB,
               &alpha_,
-              descA.descriptor(),
-              descB.descriptor(),
+              descA.unsafe_mutable_descriptor(),
+              descB.unsafe_mutable_descriptor(),
               &beta_,
               descC.descriptor(),
               compute_type,
@@ -1490,8 +1502,4 @@ void sampled_addmm_out_sparse_csr(
 #endif
 }
 
-} // namespace cuda
-} // namespace impl
-} // namespace sparse
-} // namespace native
-} // namespace at
+} // namespace at::native::sparse::impl::cuda
