@@ -1,13 +1,16 @@
 #pragma once
 
+// @lint-ignore-every CLANGTIDY facebook-hte-BadMemberName
+
 #ifdef USE_VULKAN_API
 
-#include <ATen/native/vulkan/api/Common.h>
+#include <ATen/native/vulkan/api/vk_api.h>
+
 #include <ATen/native/vulkan/api/Descriptor.h>
 #include <ATen/native/vulkan/api/Pipeline.h>
 #include <ATen/native/vulkan/api/Resource.h>
 #include <ATen/native/vulkan/api/Shader.h>
-#include <c10/util/ArrayRef.h>
+#include <ATen/native/vulkan/api/Utils.h>
 
 namespace at {
 namespace native {
@@ -16,9 +19,7 @@ namespace api {
 
 class CommandBuffer final {
  public:
-  explicit CommandBuffer(
-      const VkCommandBuffer,
-      const VkCommandBufferUsageFlags);
+  explicit CommandBuffer(VkCommandBuffer, const VkCommandBufferUsageFlags);
 
   CommandBuffer(const CommandBuffer&) = delete;
   CommandBuffer& operator=(const CommandBuffer&) = delete;
@@ -80,13 +81,10 @@ class CommandBuffer final {
   void begin();
   void end();
 
-  void bind_pipeline(
-      const VkPipeline,
-      const VkPipelineLayout,
-      const utils::uvec3);
-  void bind_descriptors(const VkDescriptorSet);
+  void bind_pipeline(VkPipeline, VkPipelineLayout, const utils::uvec3);
+  void bind_descriptors(VkDescriptorSet);
 
-  void insert_barrier(const PipelineBarrier& pipeline_barrier);
+  void insert_barrier(PipelineBarrier& pipeline_barrier);
   void dispatch(const utils::uvec3&);
 
   void copy_buffer_to_buffer(
@@ -117,8 +115,8 @@ class CommandBuffer final {
       const api::utils::uvec3&,
       const api::utils::uvec3&);
 
-  void write_timestamp(const VkQueryPool, const uint32_t) const;
-  void reset_querypool(const VkQueryPool, const uint32_t, const uint32_t) const;
+  void write_timestamp(VkQueryPool, const uint32_t) const;
+  void reset_querypool(VkQueryPool, const uint32_t, const uint32_t) const;
 
   VkCommandBuffer get_submit_handle(const bool final_use = false);
 
@@ -134,10 +132,7 @@ struct CommandPoolConfig final {
 
 class CommandPool final {
  public:
-  explicit CommandPool(
-      const VkDevice,
-      const uint32_t,
-      const CommandPoolConfig&);
+  explicit CommandPool(VkDevice, const uint32_t, const CommandPoolConfig&);
 
   CommandPool(const CommandPool&) = delete;
   CommandPool& operator=(const CommandPool&) = delete;
