@@ -1,6 +1,5 @@
 #include <ATen/native/vulkan/api/Tensor.h>
 #include <ATen/native/vulkan/api/Utils.h>
-#include <c10/util/accumulate.h>
 
 namespace at {
 namespace native {
@@ -8,75 +7,45 @@ namespace vulkan {
 
 namespace {
 
-/**
- * Determines an appropriate GPU Memory Layout qualifier based on the the
- * StorageType requested and the c10::MemoryFormat specified.
- */
-api::GPUMemoryLayout get_gpu_memory_layout(
-    const api::StorageType storage_type,
-    const c10::MemoryFormat memory_format) {
-  if (storage_type == api::StorageType::BUFFER) {
-    switch (memory_format) {
-      case c10::MemoryFormat::Contiguous:
-        return api::GPUMemoryLayout::TENSOR_WIDTH_PACKED;
-      case c10::MemoryFormat::ChannelsLast:
-        return api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED;
-      default:
-        TORCH_CHECK(false, "Invalid memory format used to create vTensor!");
-    }
-  }
-  // For texture storage, always return a memory layout that packs the channels
-  // dimension. for now. With the way texture storage currently works, for 2-dim
-  // tensors, a channel dimension is added, as well as 3 channels of zero
-  // padding resulting in a final shape of {4, H, W}. For 1-dim tensors, it is
-  // unsqueezed to size {1, 1, L} and 3 channels of zero padding are added to
-  // produce a final size of {4, 1, L}. This is to ensure that physical texture
-  // positions correspond directly to logical tensor coordinates (so
-  // texelFetch(ivec3(x, y, 0), 0) will correspond to tensor[y, x].
-  //
-  // TODO(ssjia): have 2D and 1D tensors use TENSOR_WIDTH_PACKED by default.
-  return api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED;
-}
-
 /*
  * Calculates the strides of a contiguous tensor. empty_tensor_restride from
  * TensorImpl.h was used as a reference.
  */
-c10::SmallVector<int64_t, 6u> calc_contiguous_strides(const IntArrayRef sizes) {
-  int64_t ndim = sizes.size();
-  c10::SmallVector<int64_t, 6u> strides(ndim);
+std::vector<int64_t> calc_contiguous_strides(
+    const std::vector<int64_t>& sizes) {
+  int64_t ndim = static_cast<int64_t>(sizes.size());
+  std::vector<int64_t> strides(ndim);
 
   int64_t running_product = 1;
   if (ndim >= 1) {
-    strides[ndim - 1] = running_product;
-    for (int i = sizes.size() - 2; i >= 0; --i) {
-      running_product *= sizes[i + 1];
-      strides[i] = running_product;
+    strides.at(ndim - 1) = running_product;
+    for (int i = static_cast<int>(sizes.size()) - 2; i >= 0; --i) {
+      running_product *= sizes.at(i + 1);
+      strides.at(i) = running_product;
     }
   }
 
   return strides;
 }
 
-c10::SmallVector<int64_t, 6u> calc_channels_last_strides(
-    const IntArrayRef sizes) {
-  c10::SmallVector<int64_t, 6u> strides(sizes.size());
+std::vector<int64_t> calc_channels_last_strides(
+    const std::vector<int64_t>& sizes) {
+  std::vector<int64_t> strides(sizes.size());
 
   switch (sizes.size()) {
     case 4:
-      strides[1] = 1;
-      strides[3] = sizes[1];
-      strides[2] = strides[3] * sizes[3];
-      strides[0] = strides[2] * sizes[2];
+      strides.at(1) = 1;
+      strides.at(3) = sizes.at(1);
+      strides.at(2) = strides.at(3) * sizes.at(3);
+      strides.at(0) = strides.at(2) * sizes.at(2);
       return strides;
     case 3:
-      strides[0] = 1;
-      strides[2] = sizes[0];
-      strides[1] = strides[2] * sizes[2];
+      strides.at(0) = 1;
+      strides.at(2) = sizes.at(0);
+      strides.at(1) = strides.at(2) * sizes.at(2);
       return strides;
     default:
-      TORCH_CHECK(
-          false, "ChannelsLast format only available for 3 <= ndim <= 4!");
+      VK_THROW("ChannelsLast format only available for 3 <= ndim <= 4!");
   }
 
   return strides;
@@ -87,8 +56,8 @@ c10::SmallVector<int64_t, 6u> calc_channels_last_strides(
  * that strides are only valid for vTensors that are backed by buffer storage;
  * if texture storage is used then the strides are invalid and set to zeros.
  */
-c10::SmallVector<int64_t, 6u> calc_strides(
-    const IntArrayRef sizes,
+std::vector<int64_t> calc_strides(
+    const std::vector<int64_t>& sizes,
     const api::GPUMemoryLayout memory_layout,
     const api::StorageType storage_type) {
   switch (storage_type) {
@@ -101,14 +70,14 @@ c10::SmallVector<int64_t, 6u> calc_strides(
           return calc_channels_last_strides(sizes);
           break;
         default:
-          TORCH_CHECK(false, "Invalid memory format used to create vTensor!");
+          VK_THROW("Invalid memory format used to create vTensor!");
       }
       break;
     case api::StorageType::TEXTURE_3D:
     case api::StorageType::TEXTURE_2D:
-      return c10::SmallVector<int64_t, 6u>(sizes.size());
+      return std::vector<int64_t>(sizes.size());
     default:
-      TORCH_CHECK(false, "Invalid storage type used to create vTensor!");
+      VK_THROW("Invalid storage type used to create vTensor!");
   }
 }
 
@@ -120,160 +89,61 @@ c10::SmallVector<int64_t, 6u> calc_strides(
  * returns a sizes array describing the dimensions of the memory used to store
  * the tensor data on the GPU.
  */
-c10::SmallVector<int64_t, 6u> calc_gpu_sizes(
-    const IntArrayRef sizes,
+std::vector<int64_t> calc_gpu_sizes(
+    const std::vector<int64_t>& sizes,
     const api::GPUMemoryLayout memory_layout,
     const api::StorageType storage_type) {
-  size_t ndim = sizes.size();
+  VK_CHECK_COND(storage_type != api::StorageType::UNKNOWN);
 
-  TORCH_CHECK(storage_type != api::StorageType::UNKNOWN);
-
-  // For buffer formats, the innermost dim (i.e. where the stride is 1) will be
-  // aligned up. Which dim is the innermost is described by the GPUMemoryLayout.
+  std::vector<int64_t> gpu_sizes;
   if (storage_type == api::StorageType::BUFFER) {
-    c10::SmallVector<int64_t, 6u> gpu_sizes{sizes};
-
-    switch (memory_layout) {
-      case api::GPUMemoryLayout::TENSOR_WIDTH_PACKED:
-        gpu_sizes[ndim - 1] = api::utils::align_up(sizes[ndim - 1], INT64_C(4));
-        break;
-
-      case api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED:
-        switch (ndim) {
-          case 3:
-            gpu_sizes[0] = api::utils::align_up(sizes[0], INT64_C(4));
-            break;
-
-          case 4:
-            gpu_sizes[1] = api::utils::align_up(sizes[1], INT64_C(4));
-            break;
-        }
-        break;
-
-      default:
-        TORCH_CHECK(false, "Invalid memory format used to create vTensor!");
-        break;
+    gpu_sizes.resize(sizes.size());
+    for (size_t i = 0; i < sizes.size(); i++) {
+      gpu_sizes.at(i) = sizes.at(i);
     }
-
-    return gpu_sizes;
   }
-  // If StorageType is not BUFFER, that means TEXTURE storage will be used. For
-  // texture storage, the returned gpu_sizes will be at least 3 dimensional to
-  // represent the extents of the image texture that will be allocated. For 4
-  // dimensional tensors, The gpu_sizes will also be 4 dimensional in order to
-  // preserve the size of the batch dim to facilitate conversion between logical
-  // tensor coordinates and physical texel positions. Based on the GPU memory
-  // layout, whichever dimension is packed will be aligned up to the next
-  // multiple of 4, as each texel shall store 4 consecutive elements from the
-  // packed dimension.
+  // For texture storage, tensors are typically stored using 3D image textures.
+  // Batches are stacked along the depth dimension. To represent the physical
+  // 3 dimensionality of the image texture (with concatenated batches) GPU sizes
+  // will be fixed to 4 dimensions when using texture storage.
   else {
-    TORCH_CHECK(
-        ndim >= 1 && ndim <= 4,
-        "Texture storage only valid for 1 <= ndim <= 4!");
+    VK_CHECK_COND(
+        sizes.size() >= 0 && sizes.size() <= 4,
+        "Texture storage only valid for 0 <= ndim <= 4, received: ",
+        sizes.size());
 
-    c10::SmallVector<int64_t, 6u> gpu_sizes(ndim == 4 ? 4 : 3);
-
-    // Channel dim will be be aligned to the next multiple of 4
-    switch (ndim) {
-      case 1:
-        switch (memory_layout) {
-          case api::GPUMemoryLayout::TENSOR_WIDTH_PACKED:
-            gpu_sizes[0] = 1;
-            gpu_sizes[1] = 1;
-            gpu_sizes[2] = api::utils::align_up(sizes[0], INT64_C(4));
-            break;
-          case api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED:
-            // 1-dimension tensors are interpreted as 3-dimensional tensors with
-            // size {1, 1, L} when stored as image textures, thus channel
-            // packing is valid even though the original tensor does not
-            // technically have a channels dimension. In this mode, 3 channels
-            // of zero padding are added to the unsqueezed size of {1, 1, L}
-            // producing a final shape of {4, 1, L}.
-            gpu_sizes[0] = 4;
-            gpu_sizes[1] = 1;
-            gpu_sizes[2] = sizes[0];
-            break;
-          default:
-            TORCH_CHECK(false, "Invalid memory format used to create vTensor!");
-        }
-        break;
-
-      case 2:
-        switch (memory_layout) {
-          case api::GPUMemoryLayout::TENSOR_WIDTH_PACKED:
-            gpu_sizes[0] = 1;
-            gpu_sizes[1] = sizes[0];
-            gpu_sizes[2] = api::utils::align_up(sizes[1], INT64_C(4));
-            break;
-          case api::GPUMemoryLayout::TENSOR_HEIGHT_PACKED:
-            gpu_sizes[0] = 1;
-            gpu_sizes[1] = api::utils::align_up(sizes[0], INT64_C(4));
-            gpu_sizes[2] = sizes[1];
-            break;
-          case api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED:
-            // 2-dimension tensors are interpreted as 3-dimensional tensors with
-            // size {1, H, W} when stored as image textures, thus channel
-            // packing is valid even though the original tensor does not
-            // technically have a channels dimension. In this mode, 3 channels
-            // of zero padding are added to the unsqueezed size of {1, H, W}
-            // producing a final shape of {4, H, W}.
-            gpu_sizes[0] = 4;
-            gpu_sizes[1] = sizes[0];
-            gpu_sizes[2] = sizes[1];
-            break;
-          default:
-            TORCH_CHECK(false, "Invalid memory format used to create vTensor!");
-        }
-        break;
-
-      case 3:
-        switch (memory_layout) {
-          case api::GPUMemoryLayout::TENSOR_WIDTH_PACKED:
-            gpu_sizes[0] = sizes[0];
-            gpu_sizes[1] = sizes[1];
-            gpu_sizes[2] = api::utils::align_up(sizes[2], INT64_C(4));
-            break;
-          case api::GPUMemoryLayout::TENSOR_HEIGHT_PACKED:
-            gpu_sizes[0] = sizes[0];
-            gpu_sizes[1] = api::utils::align_up(sizes[1], INT64_C(4));
-            gpu_sizes[2] = sizes[2];
-            break;
-          case api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED:
-            gpu_sizes[0] = api::utils::align_up(sizes[0], INT64_C(4));
-            gpu_sizes[1] = sizes[1];
-            gpu_sizes[2] = sizes[2];
-            break;
-          default:
-            TORCH_CHECK(false, "Invalid memory format used to create vTensor!");
-        }
-        break;
-
-      case 4:
-        switch (memory_layout) {
-          case api::GPUMemoryLayout::TENSOR_WIDTH_PACKED:
-            gpu_sizes[0] = sizes[0];
-            gpu_sizes[1] = sizes[1];
-            gpu_sizes[2] = sizes[3];
-            gpu_sizes[3] = api::utils::align_up(sizes[3], INT64_C(4));
-          case api::GPUMemoryLayout::TENSOR_HEIGHT_PACKED:
-            gpu_sizes[0] = sizes[0];
-            gpu_sizes[1] = sizes[1];
-            gpu_sizes[2] = api::utils::align_up(sizes[2], INT64_C(4));
-            gpu_sizes[3] = sizes[3];
-            break;
-          case api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED:
-            gpu_sizes[0] = sizes[0];
-            gpu_sizes[1] = api::utils::align_up(sizes[1], INT64_C(4));
-            gpu_sizes[2] = sizes[2];
-            gpu_sizes[3] = sizes[3];
-            break;
-          default:
-            TORCH_CHECK(false, "Invalid memory format used to create vTensor!");
-        }
-        break;
-    }
-    return gpu_sizes;
+    gpu_sizes.resize(4);
+    gpu_sizes.at(0) = api::utils::val_at(-4, sizes);
+    gpu_sizes.at(1) = api::utils::val_at(-3, sizes);
+    gpu_sizes.at(2) = api::utils::val_at(-2, sizes);
+    gpu_sizes.at(3) = api::utils::val_at(-1, sizes);
   }
+
+  size_t ndim = gpu_sizes.size();
+  switch (memory_layout) {
+    case api::GPUMemoryLayout::TENSOR_WIDTH_PACKED:
+      if (ndim >= 1) {
+        gpu_sizes.at(ndim - 1) =
+            api::utils::align_up(api::utils::val_at(-1, sizes), INT64_C(4));
+      }
+      break;
+
+    case api::GPUMemoryLayout::TENSOR_HEIGHT_PACKED:
+      if (ndim >= 2) {
+        gpu_sizes.at(ndim - 2) =
+            api::utils::align_up(api::utils::val_at(-2, sizes), INT64_C(4));
+      }
+      break;
+
+    case api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED:
+      if (ndim >= 3) {
+        gpu_sizes.at(ndim - 3) =
+            api::utils::align_up(api::utils::val_at(-3, sizes), INT64_C(4));
+      }
+      break;
+  }
+
+  return gpu_sizes;
 }
 
 /*
@@ -281,7 +151,7 @@ c10::SmallVector<int64_t, 6u> calc_gpu_sizes(
  * created to store a tensor of a given size.
  */
 api::utils::uvec3 create_image_extents(
-    const IntArrayRef gpu_sizes,
+    const std::vector<int64_t>& gpu_sizes,
     const api::StorageType storage_type,
     const api::GPUMemoryLayout memory_layout) {
   size_t ndim = gpu_sizes.size();
@@ -290,25 +160,31 @@ api::utils::uvec3 create_image_extents(
     // image extents do not apply to buffer storage
     return {0u, 0u, 0u};
   } else {
-    TORCH_CHECK(
+    VK_CHECK_COND(
         ndim >= 1 && ndim <= 4,
         "Texture storage only valid for 1 <= ndim <= 4!");
 
-    uint32_t width = api::utils::val_at(-1, gpu_sizes);
-    uint32_t height = api::utils::val_at(-2, gpu_sizes);
-    uint32_t channels = api::utils::val_at(-3, gpu_sizes);
-    uint32_t batch = api::utils::val_at(-4, gpu_sizes);
+    using namespace api::utils;
+    uint32_t width = safe_downcast<uint32_t>(val_at(-1, gpu_sizes));
+    uint32_t height = safe_downcast<uint32_t>(val_at(-2, gpu_sizes));
+    uint32_t channels = safe_downcast<uint32_t>(val_at(-3, gpu_sizes));
+    uint32_t batch = safe_downcast<uint32_t>(val_at(-4, gpu_sizes));
 
     switch (memory_layout) {
       case api::GPUMemoryLayout::TENSOR_WIDTH_PACKED:
-        TORCH_CHECK(width % 4 == 0, "Channels must be divisible by 4!")
+        VK_CHECK_COND(width % 4 == 0, "Channels must be divisible by 4!");
         width /= 4;
+        break;
       case api::GPUMemoryLayout::TENSOR_HEIGHT_PACKED:
-        TORCH_CHECK(height % 4 == 0, "Channels must be divisible by 4!")
+        VK_CHECK_COND(height % 4 == 0, "Channels must be divisible by 4!");
         height /= 4;
+        break;
       case api::GPUMemoryLayout::TENSOR_CHANNELS_PACKED:
-        TORCH_CHECK(channels % 4 == 0, "Channels must be divisible by 4!")
+        VK_CHECK_COND(channels % 4 == 0, "Channels must be divisible by 4!");
         channels /= 4;
+        break;
+      default:
+        VK_THROW("Invalid memory format used!");
     }
 
     return {width, height, batch * channels};
@@ -317,18 +193,18 @@ api::utils::uvec3 create_image_extents(
 
 api::UniformParamsBuffer make_metadata_uniform(
     api::Context* const context,
-    const IntArrayRef sizes,
-    const IntArrayRef strides,
+    const std::vector<int64_t>& sizes,
+    const std::vector<int64_t>& strides,
     const api::StorageType storage_type) {
   if (storage_type != api::StorageType::BUFFER) {
     return api::UniformParamsBuffer();
   }
 
   vTensor::BufferMetadata metadata{
-      api::utils::make_nchw_uvec4(sizes),
-      api::utils::make_nchw_uvec4(strides),
+      api::utils::make_whcn_uvec4(sizes),
+      api::utils::make_whcn_uvec4(strides),
       api::utils::safe_downcast<uint32_t>(sizes.size()),
-      api::utils::safe_downcast<uint32_t>(c10::multiply_integers(sizes)),
+      api::utils::safe_downcast<uint32_t>(api::utils::multiply_integers(sizes)),
   };
 
   return api::UniformParamsBuffer(context, metadata);
@@ -342,52 +218,56 @@ api::UniformParamsBuffer make_metadata_uniform(
 
 vTensor::vTensor(
     api::Context* const context,
-    const IntArrayRef sizes,
-    const c10::ScalarType dtype,
+    const std::vector<int64_t>& sizes,
+    const api::ScalarType dtype,
     const api::StorageType storage_type,
-    const api::GPUMemoryLayout memory_layout)
+    const api::GPUMemoryLayout memory_layout,
+    const bool allocate_memory)
     : dtype_(dtype),
       memory_layout_(memory_layout),
       // Calculate sizes and strides
-      sizes_{sizes},
+      sizes_(sizes.begin(), sizes.end()),
       strides_{calc_strides(sizes, memory_layout_, storage_type)},
       gpu_sizes_{calc_gpu_sizes(sizes, memory_layout_, storage_type)},
       gpu_strides_{calc_strides(gpu_sizes_, memory_layout_, storage_type)},
-      // Vulkan uniform buffer containing sizes and stride info
-      metadata_uniform_{make_metadata_uniform(
-          context,
-          gpu_sizes_,
-          gpu_strides_,
-          storage_type)},
+      virtual_extents_(
+          create_image_extents(gpu_sizes_, storage_type, memory_layout)),
+      // Utility Uniform Buffers that can be passed to shaders as arguments
+      metadata_uniform_(),
+      cpu_sizes_uniform_(nullptr),
+      gpu_sizes_uniform_(nullptr),
+      extents_uniform_(nullptr),
       // Construct Tensor storage
       view_(std::make_shared<vTensorStorage>(
           context,
           storage_type,
           memory_layout_,
           gpu_sizes_,
-          dtype_)) {}
+          dtype_,
+          allocate_memory)) {}
 
 vTensor::vTensor(
     api::Context* const context,
-    const IntArrayRef sizes,
+    const std::vector<int64_t>& sizes,
     double q_scale,
     int64_t q_zero_point,
-    const c10::ScalarType dtype,
+    const api::ScalarType dtype,
     const api::StorageType storage_type,
     const api::GPUMemoryLayout memory_layout)
     : dtype_(dtype),
       memory_layout_(memory_layout),
       // Calculate sizes and strides
-      sizes_{sizes},
+      sizes_(sizes.begin(), sizes.end()),
       strides_{calc_strides(sizes, memory_layout_, storage_type)},
       gpu_sizes_{calc_gpu_sizes(sizes, memory_layout_, storage_type)},
       gpu_strides_{calc_strides(gpu_sizes_, memory_layout_, storage_type)},
+      virtual_extents_(
+          create_image_extents(gpu_sizes_, storage_type, memory_layout)),
       // Vulkan uniform buffer containing sizes and stride info
-      metadata_uniform_{make_metadata_uniform(
-          context,
-          gpu_sizes_,
-          gpu_strides_,
-          storage_type)},
+      metadata_uniform_(),
+      cpu_sizes_uniform_(nullptr),
+      gpu_sizes_uniform_(nullptr),
+      extents_uniform_(nullptr),
       // Quantization params
       is_quantized_{true},
       q_scale_{q_scale},
@@ -400,41 +280,9 @@ vTensor::vTensor(
           gpu_sizes_,
           dtype_)) {}
 
-vTensor::vTensor(
-    api::Context* const context,
-    const IntArrayRef sizes,
-    const c10::ScalarType dtype,
-    const api::StorageType storage_type,
-    const c10::MemoryFormat memory_format)
-    : vTensor(
-          context,
-          sizes,
-          dtype,
-          storage_type,
-          get_gpu_memory_layout(storage_type, memory_format)) {}
-
-vTensor::vTensor(
-    api::Context* const context,
-    const IntArrayRef sizes,
-    double q_scale,
-    int64_t q_zero_point,
-    const c10::ScalarType dtype,
-    const api::StorageType storage_type,
-    const c10::MemoryFormat memory_format)
-    : vTensor(
-          context,
-          sizes,
-          q_scale,
-          q_zero_point,
-          dtype,
-          storage_type,
-          get_gpu_memory_layout(storage_type, memory_format)) {}
-
 api::VulkanImage& vTensor::image(
     api::PipelineBarrier& pipeline_barrier,
     const api::PipelineStageFlags stage) const& {
-  TORCH_CHECK(view_->image_, "vTensor has empty image texture!");
-
   view_->transition(pipeline_barrier, stage, api::MemoryAccessType::READ);
   return view_->image_;
 }
@@ -443,8 +291,6 @@ api::VulkanImage& vTensor::image(
     api::PipelineBarrier& pipeline_barrier,
     const api::PipelineStageFlags stage,
     const api::MemoryAccessFlags access) & {
-  TORCH_CHECK(view_->image_, "vTensor has empty image texture!");
-
   view_->transition(pipeline_barrier, stage, access);
   return view_->image_;
 }
@@ -452,8 +298,6 @@ api::VulkanImage& vTensor::image(
 api::VulkanBuffer& vTensor::buffer(
     api::PipelineBarrier& pipeline_barrier,
     const api::PipelineStageFlags stage) const& {
-  TORCH_CHECK(view_->buffer_, "vTensor has empty buffer!");
-
   view_->transition(pipeline_barrier, stage, api::MemoryAccessType::READ);
   return view_->buffer_;
 }
@@ -462,30 +306,166 @@ api::VulkanBuffer& vTensor::buffer(
     api::PipelineBarrier& pipeline_barrier,
     const api::PipelineStageFlags stage,
     const api::MemoryAccessFlags access) & {
-  TORCH_CHECK(view_->buffer_, "vTensor has empty buffer!");
-
   view_->transition(pipeline_barrier, stage, access);
   return view_->buffer_;
 }
 
+api::VulkanBuffer& vTensor::buffer_metadata() {
+  if (!metadata_uniform_.buffer()) {
+    metadata_uniform_ = make_metadata_uniform(
+        view_->context_, gpu_sizes_, gpu_strides_, storage_type());
+  }
+  return metadata_uniform_.buffer();
+}
+
+std::shared_ptr<api::UniformParamsBuffer> vTensor::cpu_sizes_ubo() {
+  if (!cpu_sizes_uniform_) {
+    cpu_sizes_uniform_.reset(new api::UniformParamsBuffer(
+        view_->context_, api::utils::make_whcn_ivec4(sizes_)));
+  }
+  return cpu_sizes_uniform_;
+}
+
+std::shared_ptr<api::UniformParamsBuffer> vTensor::gpu_sizes_ubo() {
+  if (!gpu_sizes_uniform_) {
+    gpu_sizes_uniform_.reset(new api::UniformParamsBuffer(
+        view_->context_, api::utils::make_whcn_ivec4(gpu_sizes_)));
+  }
+  return gpu_sizes_uniform_;
+}
+
+std::shared_ptr<api::UniformParamsBuffer> vTensor::extents_ubo() {
+  if (!extents_uniform_) {
+    extents_uniform_.reset(new api::UniformParamsBuffer(
+        view_->context_,
+        api::utils::uvec4(
+            {view_->extents_.data[0],
+             view_->extents_.data[1],
+             view_->extents_.data[2],
+             1u})));
+  }
+  return extents_uniform_;
+}
+
 vTensor::BufferMetadata vTensor::get_cpu_buffer_metadata() const {
   return {
-      api::utils::make_nchw_uvec4(sizes_),
-      api::utils::make_nchw_uvec4(strides_),
+      api::utils::make_whcn_uvec4(sizes_),
+      api::utils::make_whcn_uvec4(strides_),
       api::utils::safe_downcast<uint32_t>(sizes_.size()),
-      api::utils::safe_downcast<uint32_t>(c10::multiply_integers(sizes_)),
+      api::utils::safe_downcast<uint32_t>(
+          api::utils::multiply_integers(sizes_)),
   };
 }
 
+VmaAllocationCreateInfo vTensor::get_allocation_create_info() const {
+  switch (storage_type()) {
+    case api::StorageType::BUFFER:
+      return view_->buffer_.allocation_create_info();
+    case api::StorageType::TEXTURE_2D:
+    case api::StorageType::TEXTURE_3D:
+      return view_->image_.allocation_create_info();
+    case api::StorageType::UNKNOWN:
+      break;
+  }
+  return {};
+}
+
+VkMemoryRequirements vTensor::get_memory_requirements() const {
+  switch (storage_type()) {
+    case api::StorageType::BUFFER:
+      return view_->buffer_.get_memory_requirements();
+    case api::StorageType::TEXTURE_2D:
+    case api::StorageType::TEXTURE_3D:
+      return view_->image_.get_memory_requirements();
+    case api::StorageType::UNKNOWN:
+      break;
+  }
+  return {};
+}
+
+void vTensor::bind_allocation(const api::MemoryAllocation& allocation) {
+  switch (storage_type()) {
+    case api::StorageType::BUFFER:
+      view_->buffer_.bind_allocation(allocation);
+      break;
+    case api::StorageType::TEXTURE_2D:
+    case api::StorageType::TEXTURE_3D:
+      view_->image_.bind_allocation(allocation);
+      break;
+    case api::StorageType::UNKNOWN:
+      break;
+  }
+}
+
+void vTensor::update_size_metadata(const std::vector<int64_t>& new_sizes) {
+  sizes_ = new_sizes;
+  gpu_sizes_ = calc_gpu_sizes(sizes_, memory_layout_, storage_type());
+  virtual_extents_ =
+      create_image_extents(gpu_sizes_, storage_type(), memory_layout_);
+
+  if (cpu_sizes_uniform_) {
+    cpu_sizes_uniform_->update(api::utils::make_whcn_ivec4(sizes_));
+  }
+
+  if (gpu_sizes_uniform_) {
+    gpu_sizes_uniform_->update(api::utils::make_whcn_ivec4(gpu_sizes_));
+  }
+
+  if (extents_uniform_) {
+    extents_uniform_->update(api::utils::uvec4(
+        {virtual_extents_.data[0],
+         virtual_extents_.data[1],
+         virtual_extents_.data[2],
+         1u}));
+  }
+}
+
+void vTensor::reallocate(const std::vector<int64_t>& new_sizes) {
+  update_size_metadata(new_sizes);
+  view_->discard_and_reallocate(
+      calc_gpu_sizes(new_sizes, memory_layout_, storage_type()),
+      memory_layout_,
+      dtype_);
+}
+
+void vTensor::virtual_resize(const std::vector<int64_t>& new_sizes) {
+  update_size_metadata(new_sizes);
+  if (storage_type() == api::StorageType::BUFFER) {
+    if (gpu_nbytes() > view_->buffer_.mem_size()) {
+      VK_THROW(
+          "Cannot virtual_resize a vTensor with sizes that require a larger "
+          "buffer! reallocate() should be used instead.");
+    }
+  } else {
+    bool valid_resize = true;
+    if (virtual_extents_.data[0] > view_->extents_.data[0]) {
+      valid_resize = false;
+    }
+    if (virtual_extents_.data[1] > view_->extents_.data[1]) {
+      valid_resize = false;
+    }
+    if (virtual_extents_.data[2] > view_->extents_.data[2]) {
+      valid_resize = false;
+    }
+
+    if (!valid_resize) {
+      VK_THROW(
+          "Cannot virtual_resize a vTensor with sizes that require a larger "
+          "image texture! reallocate() should be used instead.");
+    }
+  }
+}
+
 //
 // vTensorStorage
 //
 
-api::VulkanImage allocate_image(
+static api::VulkanImage allocate_image(
     api::Context* const context_ptr,
     api::utils::uvec3& extents,
     const api::StorageType storage_type,
-    const VkFormat image_format) {
+    const VkFormat image_format,
+    const bool allocate_memory) {
   api::Adapter* adapter_ptr = context_ptr->adapter_ptr();
 
   api::ImageSampler::Properties sampler_props{
@@ -521,14 +501,16 @@ api::VulkanImage allocate_image(
       image_view_type,
       sampler_props,
       sampler,
-      true);
+      /*allow_transfer = */ true,
+      /*allocate_memory = */ allocate_memory);
 }
 
-api::VulkanBuffer allocate_buffer(
+static api::VulkanBuffer allocate_buffer(
     api::Context* const context_ptr,
     const int64_t numel,
     const api::StorageType storage_type,
-    const c10::ScalarType dtype) {
+    const api::ScalarType dtype,
+    const bool allocate_memory) {
   api::Adapter* adapter_ptr = context_ptr->adapter_ptr();
 
   switch (storage_type) {
@@ -540,34 +522,46 @@ api::VulkanBuffer allocate_buffer(
   }
 
   return adapter_ptr->vma().create_storage_buffer(
-      c10::elementSize(dtype) * numel, true);
+      api::element_size(dtype) * numel, /*gpu_only = */ true, allocate_memory);
 }
 
 vTensorStorage::vTensorStorage(
     api::Context* const context,
     const api::StorageType storage_type,
     const api::GPUMemoryLayout gpu_memory_layout,
-    const IntArrayRef gpu_sizes,
-    const at::ScalarType dtype)
+    const std::vector<int64_t>& gpu_sizes,
+    const api::ScalarType dtype,
+    const bool allocate_memory)
     : context_(context),
       storage_type_{storage_type},
       extents_(
           create_image_extents(gpu_sizes, storage_type, gpu_memory_layout)),
-      buffer_length_{c10::multiply_integers(gpu_sizes)},
+      buffer_length_{api::utils::multiply_integers(gpu_sizes)},
       image_(allocate_image(
           context_,
           extents_,
           storage_type_,
-          api::vk_format(dtype))),
-      buffer_(allocate_buffer(context_, buffer_length_, storage_type_, dtype)),
+          api::to_vkformat(dtype),
+          allocate_memory)),
+      buffer_(allocate_buffer(
+          context_,
+          buffer_length_,
+          storage_type_,
+          dtype,
+          allocate_memory)),
       last_access_{} {}
 
 vTensorStorage::~vTensorStorage() {
+  flush();
+}
+
+void vTensorStorage::flush() {
   if (image_) {
     context_->register_image_cleanup(image_);
   } else if (buffer_) {
     context_->register_buffer_cleanup(buffer_);
   }
+  last_access_ = {};
 }
 
 void vTensorStorage::transition(
@@ -604,19 +598,19 @@ void vTensorStorage::transition(
     pipeline_barrier.stage.dst |= dst_stage;
 
     if (image_) {
-      pipeline_barrier.images.push_back(api::ImageMemoryBarrier(
+      pipeline_barrier.images.emplace_back(
           api::vk_access(prev_stage, prev_access),
           api::vk_access(cur_stage, cur_access),
           cur_layout,
           new_layout,
-          image_));
+          image_);
 
       image_.set_layout(new_layout);
     } else if (buffer_) {
-      pipeline_barrier.buffers.push_back(api::BufferMemoryBarrier(
+      pipeline_barrier.buffers.emplace_back(
           api::vk_access(prev_stage, prev_access),
           api::vk_access(cur_stage, cur_access),
-          buffer_));
+          buffer_);
     }
   }
 
@@ -650,13 +644,35 @@ void add_buffer_barrier(
     pipeline_barrier.stage.src |= src_stage;
     pipeline_barrier.stage.dst |= dst_stage;
 
-    pipeline_barrier.buffers.push_back(api::BufferMemoryBarrier(
+    pipeline_barrier.buffers.emplace_back(
         api::vk_access(prev_stage, prev_access),
         api::vk_access(cur_stage, cur_access),
-        buffer));
+        buffer);
   }
 }
 
+void vTensorStorage::discard_and_reallocate(
+    const std::vector<int64_t>& gpu_sizes,
+    const api::GPUMemoryLayout gpu_memory_layout,
+    const api::ScalarType dtype) {
+  const bool image_owns_memory = image_.owns_memory();
+  const bool buffer_owns_memory = buffer_.owns_memory();
+
+  flush();
+
+  extents_ = create_image_extents(gpu_sizes, storage_type_, gpu_memory_layout);
+  image_ = allocate_image(
+      context_,
+      extents_,
+      storage_type_,
+      api::to_vkformat(dtype),
+      image_owns_memory);
+
+  buffer_length_ = api::utils::multiply_integers(gpu_sizes);
+  buffer_ = allocate_buffer(
+      context_, buffer_length_, storage_type_, dtype, buffer_owns_memory);
+}
+
 } // namespace vulkan
 } // namespace native
 } // namespace at
