@@ -10,9 +10,9 @@ namespace at::native::xnnpack {
 c10::intrusive_ptr<LinearOpContext>
 XNNPackLinearOpContext::create_context(
     at::Tensor&& weight,
-    c10::optional<at::Tensor>&& bias,
-    const c10::optional<Scalar>& output_min,
-    const c10::optional<Scalar>& output_max) {
+    std::optional<at::Tensor>&& bias,
+    const std::optional<Scalar>& output_min,
+    const std::optional<Scalar>& output_max) {
   auto linear_op_context =
       c10::make_intrusive<XNNPackLinearOpContext>(
           std::move(weight),
@@ -46,13 +46,13 @@ Tensor XNNPackLinearOpContext::run(const Tensor& input) {
 
 c10::intrusive_ptr<Conv2dOpContext>
 XNNPackConv2dOpContext::create_context(at::Tensor&& weight,
-    c10::optional<at::Tensor>&& bias,
+    std::optional<at::Tensor>&& bias,
     std::vector<int64_t>&& padding,
     std::vector<int64_t>&& stride,
     std::vector<int64_t>&& dilation,
     int64_t groups,
-    const c10::optional<Scalar>& output_min,
-    const c10::optional<Scalar>& output_max) {
+    const std::optional<Scalar>& output_min,
+    const std::optional<Scalar>& output_max) {
   auto op_context =
       xnnpack::internal::convolution2d::create(
           weight,
@@ -89,14 +89,14 @@ XNNPackConv2dOpContext::create_context(at::Tensor&& weight,
 
 c10::intrusive_ptr<TransposeConv2dOpContext>
 XNNPackTransposeConv2dOpContext::create_context(at::Tensor&& weight,
-    c10::optional<at::Tensor>&& bias,
+    std::optional<at::Tensor>&& bias,
     std::vector<int64_t>&& padding,
     std::vector<int64_t>&& output_padding,
     std::vector<int64_t>&& stride,
     std::vector<int64_t>&& dilation,
     int64_t groups,
-    const c10::optional<Scalar>& output_min,
-    const c10::optional<Scalar>& output_max) {
+    const std::optional<Scalar>& output_min,
+    const std::optional<Scalar>& output_max) {
   auto op_context =
       xnnpack::internal::convolution2d::create(
           weight,
