@@ -1,6 +1,7 @@
 from .server import AverageBatchParameterServer, AverageParameterServer
 
+
 server_map = {
     "AverageParameterServer": AverageParameterServer,
-    "AverageBatchParameterServer": AverageBatchParameterServer
+    "AverageBatchParameterServer": AverageBatchParameterServer,
 }
