@@ -1,4 +1,3 @@
-import gc
 import sys
 
 from benchmark_base import BenchmarkBase
@@ -9,8 +8,15 @@ import torch
 class Benchmark(BenchmarkBase):
     N = 20
 
+    def __init__(self):
+        super().__init__(
+            category="update_hint",
+            backend="inductor",
+            device="cpu",
+        )
+
     def name(self):
-        return "update_hint_regression"
+        return f"{self.category()}_regression"
 
     def description(self):
         return "information at https://github.com/pytorch/pytorch/pull/129893"
@@ -25,8 +31,6 @@ class Benchmark(BenchmarkBase):
 
     def _prepare(self):
         torch._dynamo.reset()
-        gc.collect()
-        gc.disable()
 
     def _work(self):
         @torch.compile(fullgraph=True)
