@@ -1,7 +1,10 @@
 #include <c10/core/Allocator.h>
+#include <array>
 
 #include <c10/util/ThreadLocalDebugInfo.h>
 
+#include <cstring>
+
 namespace c10 {
 
 DataPtr Allocator::clone(const void* data, std::size_t n) {
@@ -36,10 +39,10 @@ at::DataPtr InefficientStdFunctionContext::makeDataPtr(
       device};
 }
 
-// NOLINTNEXTLINE(cppcoreguidelines-avoid-non-const-global-variables,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
-C10_API at::Allocator* allocator_array[at::COMPILE_TIME_MAX_DEVICE_TYPES];
-// NOLINTNEXTLINE(cppcoreguidelines-avoid-non-const-global-variables,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
-C10_API uint8_t allocator_priority[at::COMPILE_TIME_MAX_DEVICE_TYPES] = {0};
+static std::array<at::Allocator*, at::COMPILE_TIME_MAX_DEVICE_TYPES>
+    allocator_array{};
+static std::array<uint8_t, at::COMPILE_TIME_MAX_DEVICE_TYPES>
+    allocator_priority{};
 
 void SetAllocator(at::DeviceType t, at::Allocator* alloc, uint8_t priority) {
   if (priority >= allocator_priority[static_cast<int>(t)]) {
@@ -87,8 +90,6 @@ void reportOutOfMemoryToProfiler(
   }
 }
 
-MemoryReportingInfoBase::MemoryReportingInfoBase() = default;
-
 void MemoryReportingInfoBase::reportOutOfMemory(
     int64_t /*alloc_size*/,
     size_t /*total_allocated*/,
