@@ -31,6 +31,18 @@ c10::intrusive_ptr<GeneratorImpl> GeneratorImpl::clone() const {
   return c10::intrusive_ptr<GeneratorImpl>::reclaim(res);
 }
 
+void GeneratorImpl::graphsafe_set_state(
+    const c10::intrusive_ptr<c10::GeneratorImpl>& state) {
+  TORCH_CHECK_NOT_IMPLEMENTED(
+      false, "graphsafe_set_state is not supported in this Generator");
+}
+
+c10::intrusive_ptr<c10::GeneratorImpl> GeneratorImpl::graphsafe_get_state()
+    const {
+  TORCH_CHECK_NOT_IMPLEMENTED(
+      false, "graphsafe_get_state is not supported in this Generator");
+}
+
 /**
  * Gets the device of a generator.
  */
@@ -76,8 +88,7 @@ static uint64_t readURandomLong() {
  *   a 32 bit number to 64 bit.
  */
 uint64_t getNonDeterministicRandom(bool is_cuda) {
-  // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-  uint64_t s;
+  uint64_t s = 0;
   if (!is_cuda) {
 #ifdef _WIN32
     s = (uint64_t)std::chrono::high_resolution_clock::now()
