@@ -40,8 +40,7 @@ class C10_CUDA_API CUDAError : public c10::Error {
   do {                                                         \
     const cudaError_t __err = EXPR;                            \
     if (C10_UNLIKELY(__err != cudaSuccess)) {                  \
-      auto error_unused C10_UNUSED = cudaGetLastError();       \
-      (void)error_unused;                                      \
+      [[maybe_unused]] auto error_unused = cudaGetLastError(); \
       TORCH_WARN("CUDA warning: ", cudaGetErrorString(__err)); \
     }                                                          \
   } while (0)
@@ -50,20 +49,18 @@ class C10_CUDA_API CUDAError : public c10::Error {
 #define C10_CUDA_ERROR_HANDLED(EXPR) EXPR
 
 // Intentionally ignore a CUDA error
-#define C10_CUDA_IGNORE_ERROR(EXPR)                             \
-  do {                                                          \
-    const cudaError_t __err = EXPR;                             \
-    if (C10_UNLIKELY(__err != cudaSuccess)) {                   \
-      cudaError_t error_unused C10_UNUSED = cudaGetLastError(); \
-      (void)error_unused;                                       \
-    }                                                           \
+#define C10_CUDA_IGNORE_ERROR(EXPR)                                   \
+  do {                                                                \
+    const cudaError_t __err = EXPR;                                   \
+    if (C10_UNLIKELY(__err != cudaSuccess)) {                         \
+      [[maybe_unused]] cudaError_t error_unused = cudaGetLastError(); \
+    }                                                                 \
   } while (0)
 
 // Clear the last CUDA error
-#define C10_CUDA_CLEAR_ERROR()                                \
-  do {                                                        \
-    cudaError_t error_unused C10_UNUSED = cudaGetLastError(); \
-    (void)error_unused;                                       \
+#define C10_CUDA_CLEAR_ERROR()                                      \
+  do {                                                              \
+    [[maybe_unused]] cudaError_t error_unused = cudaGetLastError(); \
   } while (0)
 
 // This should be used directly after every kernel launch to ensure
