@@ -33,12 +33,15 @@
 #define __ubsan_ignore_pointer_overflow__ \
   __attribute__((no_sanitize("pointer-overflow")))
 #define __ubsan_ignore_function__ __attribute__((no_sanitize("function")))
+#define __ubsan_ignore_float_cast_overflow__ \
+  __attribute__((no_sanitize("float-cast-overflow")))
 #else
 #define __ubsan_ignore_float_divide_by_zero__
 #define __ubsan_ignore_undefined__
 #define __ubsan_ignore_signed_int_overflow__
 #define __ubsan_ignore_pointer_overflow__
 #define __ubsan_ignore_function__
+#define __ubsan_ignore_float_cast_overflow__
 #endif
 
 // Detect address sanitizer as some stuff doesn't work with it
@@ -64,6 +67,25 @@
 #define C10_ASAN_ENABLED 0
 #endif
 
+// Detect undefined-behavior sanitizer (UBSAN)
+#undef C10_UBSAN_ENABLED
+
+// for clang or gcc >= 14
+// NB: gcc 14 adds support for Clang's __has_feature
+//   https://gcc.gnu.org/gcc-14/changes.html
+//   gcc < 14 doesn't have a macro for UBSAN
+//   (e.g. __SANITIZE_UNDEFINED__ does not exist in gcc)
+//   https://github.com/google/sanitizers/issues/765
+#if defined(__has_feature)
+#if ((__has_feature(undefined_behavior_sanitizer)))
+#define C10_UBSAN_ENABLED 1
+#endif
+#endif
+
+#if !defined(C10_UBSAN_ENABLED)
+#define C10_UBSAN_ENABLED 0
+#endif
+
 // Disable the copy and assignment operator for a class. Note that this will
 // disable the usage of the class in std containers.
 #define C10_DISABLE_COPY_AND_ASSIGN(classname) \
@@ -96,46 +118,13 @@
 #define C10_HAS_CPP_ATTRIBUTE(x) (0)
 #endif
 
-/// C10_NODISCARD - Warn if a type or return value is discarded.
-
-// Technically, we should check if __cplusplus > 201402L here, because
-// [[nodiscard]] is only defined in C++17.  However, some compilers
-// we care about don't advertise being C++17 (e.g., clang), but
-// support the attribute anyway.  In fact, this is not just a good idea,
-// it's the law: clang::warn_unused_result doesn't work on nvcc + clang
-// and the best workaround for this case is to use [[nodiscard]]
-// instead; see https://github.com/pytorch/pytorch/issues/13118
-//
-// Note to future editors: if you have noticed that a compiler is
-// misbehaving (e.g., it advertises support, but the support doesn't
-// actually work, or it is emitting warnings).  Some compilers which
-// are strict about the matter include MSVC, which will complain:
-//
-//  error C2429: attribute 'nodiscard' requires compiler flag '/std:c++latest'
-//
-// Exhibits:
-//  - MSVC 19.14: https://godbolt.org/z/Dzd7gn (requires /std:c++latest)
-//  - Clang 8.0.0: https://godbolt.org/z/3PYL4Z (always advertises support)
-//  - gcc 8.3: https://godbolt.org/z/4tLMQS (always advertises support)
-#if C10_HAS_CPP_ATTRIBUTE(nodiscard)
+#ifndef FBCODE_CAFFE2
+/// DEPRECATED: Warn if a type or return value is discarded.
 #define C10_NODISCARD [[nodiscard]]
-// Workaround for llvm.org/PR23435, since clang 3.6 and below emit a spurious
-// error when __has_cpp_attribute is given a scoped attribute in C mode.
-#elif __cplusplus && C10_HAS_CPP_ATTRIBUTE(clang::warn_unused_result)
-// TODO: It's possible this is still triggering
-// https://github.com/pytorch/pytorch/issues/13118 on Windows; if it is, better
-// fix it.
-#define C10_NODISCARD [[clang::warn_unused_result]]
-#else
-#define C10_NODISCARD
-#endif
 
-// suppress an unused variable.
-#if defined(_MSC_VER) && !defined(__clang__)
-#define C10_UNUSED __pragma(warning(suppress : 4100 4101))
-#else
-#define C10_UNUSED __attribute__((__unused__))
-#endif //_MSC_VER
+/// DEPRECATED: Suppress an unused variable.
+#define C10_UNUSED [[maybe_unused]]
+#endif
 
 #if !defined(__has_attribute)
 #define __has_attribute(x) 0
@@ -225,6 +214,18 @@ using namespace c10::xpu;
 #define C10_ALWAYS_INLINE inline
 #endif
 
+// Unlike C10_ALWAYS_INLINE, C10_ALWAYS_INLINE_ATTRIBUTE can be used
+// on a lambda.
+#if defined(_MSC_VER)
+// MSVC 14.39 is reasonably recent and doesn't like
+// [[msvc::forceinline]] on a lambda, so don't try to use it.
+#define C10_ALWAYS_INLINE_ATTRIBUTE
+#elif __has_attribute(always_inline) || defined(__GNUC__)
+#define C10_ALWAYS_INLINE_ATTRIBUTE __attribute__((__always_inline__))
+#else
+#define C10_ALWAYS_INLINE_ATTRIBUTE
+#endif
+
 #if defined(_MSC_VER)
 #define C10_ATTR_VISIBILITY_HIDDEN
 #elif defined(__GNUC__)
@@ -326,6 +327,7 @@ constexpr uint32_t CUDA_THREADS_PER_BLOCK_FALLBACK = 256;
 #if defined(__ANDROID__) || defined(__APPLE__) || defined(__FreeBSD__)
 // Those platforms do not support assert()
 #define CUDA_KERNEL_ASSERT(cond)
+#define CUDA_KERNEL_ASSERT_MSG(cond, msg)
 #define SYCL_KERNEL_ASSERT(cond)
 #elif defined(_MSC_VER)
 #if defined(NDEBUG)
@@ -353,6 +355,16 @@ __host__ __device__
                static_cast<unsigned>(__LINE__)), \
            0);                                   \
   }
+// TODO: This doesn't assert the message because I (chilli) couldn't figure out
+// a nice way to convert a char* to a wchar_t*
+#define CUDA_KERNEL_ASSERT_MSG(cond, msg)        \
+  if (C10_UNLIKELY(!(cond))) {                   \
+    (void)(_wassert(                             \
+               _CRT_WIDE(#cond),                 \
+               _CRT_WIDE(__FILE__),              \
+               static_cast<unsigned>(__LINE__)), \
+           0);                                   \
+  }
 #define SYCL_KERNEL_ASSERT(cond)                 \
   if (C10_UNLIKELY(!(cond))) {                   \
     (void)(_wassert(                             \
@@ -394,6 +406,7 @@ __host__ __device__
 // ROCm disable kernel assert by default
 #if !defined(C10_USE_ROCM_KERNEL_ASSERT) and defined(USE_ROCM)
 #define CUDA_KERNEL_ASSERT(cond)
+#define CUDA_KERNEL_ASSERT_MSG(cond, msg)
 #define SYCL_KERNEL_ASSERT(cond)
 #else
 #define CUDA_KERNEL_ASSERT(cond)                                         \
@@ -401,6 +414,11 @@ __host__ __device__
     __assert_fail(                                                       \
         #cond, __FILE__, static_cast<unsigned int>(__LINE__), __func__); \
   }
+#define CUDA_KERNEL_ASSERT_MSG(cond, msg)                              \
+  if (C10_UNLIKELY(!(cond))) {                                         \
+    __assert_fail(                                                     \
+        msg, __FILE__, static_cast<unsigned int>(__LINE__), __func__); \
+  }
 #define SYCL_KERNEL_ASSERT(cond)                                         \
   if (C10_UNLIKELY(!(cond))) {                                           \
     __assert_fail(                                                       \
@@ -429,66 +447,14 @@ __host__ __device__
 #define C10_ALWAYS_INLINE_UNLESS_MOBILE C10_ALWAYS_INLINE
 #endif
 
-#if defined(__CUDA_ARCH__)
-#if defined(_MSC_VER) && defined(__CUDACC__)
-#define CONSTEXPR_EXCEPT_WIN_CUDA const
-#define C10_HOST_CONSTEXPR_EXCEPT_WIN_CUDA __host__
-
-// Note [static constexpr char* members for windows NVCC]
-// The Windows NVCC compiler doesn't handle static constexpr class members,
-// although it's fixed in a later version.
-// (see
-// https://developercommunity.visualstudio.com/t/intellisense-error-c11-static-constexpr-member-ini/245425)
-//
-// If we want to ensure that our field is static under all builds, then we need
-// to work around it specifically for windows NVCC by making it (a) const, (b)
-// defined outside of the class definition We need to define it outside of the
-// class definition because of the C++ standard; char* is not an integral type
-// (see
-// https://stackoverflow.com/questions/24278473/intellisense-a-member-of-type-const-char-const-cannot-have-an-in-class-in)
-//
-// So instead of this:
-// struct Foo {
-//     static constexpr const char* name = "foo";
-// }
-// In Windows NVCC, we end up with this:
-// struct Foo {
-//     static const char* name;
-// }
-// const char* Foo::name = "foo";
-//
-// This gives us a small perf hit for any code that wants to access these field
-// members, but right now it isn't used in any perf-critical code paths.
-#define STATIC_CONSTEXPR_STR_INL_EXCEPT_WIN_CUDA(field, val) \
-  static const char* field;
-#define STATIC_CONST_STR_OUT_OF_LINE_FOR_WIN_CUDA(cls, field, val) \
-  const char* cls::field = val;
-#else
-#define CONSTEXPR_EXCEPT_WIN_CUDA constexpr
-#define C10_HOST_CONSTEXPR_EXCEPT_WIN_CUDA __host__
-
-#define STATIC_CONSTEXPR_STR_INL_EXCEPT_WIN_CUDA(field, val) \
-  static constexpr const char* field = val;
-#define STATIC_CONST_STR_OUT_OF_LINE_FOR_WIN_CUDA(cls, field, val)
-#endif
-#else
-#if defined(_MSC_VER) && defined(__CUDACC__)
-#define CONSTEXPR_EXCEPT_WIN_CUDA const
-#define C10_HOST_CONSTEXPR_EXCEPT_WIN_CUDA
-
-#define STATIC_CONSTEXPR_STR_INL_EXCEPT_WIN_CUDA(field, val) \
-  static const char* field;
-#define STATIC_CONST_STR_OUT_OF_LINE_FOR_WIN_CUDA(cls, field, val) \
-  const char* cls::field = val;
-#else
+#if !defined(FBCODE_CAFFE2) && !defined(C10_NODEPRECATED)
 #define CONSTEXPR_EXCEPT_WIN_CUDA constexpr
 #define C10_HOST_CONSTEXPR_EXCEPT_WIN_CUDA constexpr
 
 #define STATIC_CONSTEXPR_STR_INL_EXCEPT_WIN_CUDA(field, val) \
-  static constexpr const char* field = val;
+  static constexpr const char field[] = val;
 #define STATIC_CONST_STR_OUT_OF_LINE_FOR_WIN_CUDA(cls, field, val)
-#endif
-#endif
+#endif // !defined(FBCODE_CAFFE2) && !defined(C10_NODEPRECATED)
 
 #ifndef HAS_DEMANGLE
 #if defined(__ANDROID__) || defined(_WIN32) || defined(__EMSCRIPTEN__)
