@@ -223,12 +223,14 @@ TEST(DispatchKeySet, DoubletonPerBackend) {
       // Skip these because they aren't real keys.
       if (tid1 == DispatchKey::StartOfDenseBackends ||
           tid1 == DispatchKey::StartOfSparseBackends ||
+          tid1 == DispatchKey::StartOfSparseCsrBackends ||
           tid1 == DispatchKey::StartOfQuantizedBackends ||
           tid1 == DispatchKey::StartOfNestedTensorBackends ||
           tid1 == DispatchKey::StartOfAutogradFunctionalityBackends)
         continue;
       if (tid2 == DispatchKey::StartOfDenseBackends ||
           tid2 == DispatchKey::StartOfSparseBackends ||
+          tid2 == DispatchKey::StartOfSparseCsrBackends ||
           tid2 == DispatchKey::StartOfQuantizedBackends ||
           tid2 == DispatchKey::StartOfNestedTensorBackends ||
           tid2 == DispatchKey::StartOfAutogradFunctionalityBackends)
@@ -326,6 +328,12 @@ TEST(DispatchKeySet, getHighestPriorityBackendTypeId) {
   ASSERT_EQ(
       DispatchKey::SparseCUDA, c10::highestPriorityBackendTypeId(sparse_cuda));
 
+  DispatchKeySet sparse_compressed_cuda(
+      {DispatchKey::Functionalize, DispatchKey::SparseCsrCUDA});
+  ASSERT_EQ(
+      DispatchKey::SparseCsrCUDA,
+      c10::highestPriorityBackendTypeId(sparse_compressed_cuda));
+
   // quantizedCUDA has higher priority than CUDA
   DispatchKeySet quantized_cuda(
       {DispatchKey::CUDA, DispatchKey::QuantizedCUDA});
@@ -400,7 +408,7 @@ TEST(DispatchKeySet, TestBackendComponentToString) {
 }
 
 TEST(DispatchKeySet, TestEndOfRuntimeBackendKeysAccurate) {
-  DispatchKey k;
+  DispatchKey k = DispatchKey::Undefined;
 #define SETTER(fullname, prefix) k = DispatchKey::EndOf##fullname##Backends;
   C10_FORALL_FUNCTIONALITY_KEYS(SETTER)
 #undef SETTER
@@ -417,6 +425,7 @@ TEST(DispatchKeySet, TestFunctionalityDispatchKeyToString) {
         k == DispatchKey::StartOfDenseBackends ||
         k == DispatchKey::StartOfQuantizedBackends ||
         k == DispatchKey::StartOfSparseBackends ||
+        k == DispatchKey::StartOfSparseCsrBackends ||
         k == DispatchKey::StartOfNestedTensorBackends ||
         k == DispatchKey::StartOfAutogradFunctionalityBackends)
       continue;
