@@ -11,3 +11,6 @@
 #include <math.h>
 
 static_assert(M_PI == c10::pi<double>, "c10::pi<double> must be equal to M_PI");
+static_assert(
+    M_SQRT1_2 == c10::frac_sqrt_2<double>,
+    "c10::frac_sqrt_2<double> must be equal to M_SQRT1_2");
