@@ -7,18 +7,21 @@ from mypy.plugin import Plugin
 
 def get_correct_mypy_version():
     # there's probably a more elegant way to do this
-    match, = re.finditer(
-        r'mypy==(\d+(?:\.\d+)*)',
-        (Path(__file__).parent.parent / '.ci' / 'docker' / 'requirements-ci.txt').read_text(),
+    (match,) = re.finditer(
+        r"mypy==(\d+(?:\.\d+)*)",
+        (
+            Path(__file__).parent.parent / ".ci" / "docker" / "requirements-ci.txt"
+        ).read_text(),
     )
-    version, = match.groups()
+    (version,) = match.groups()
     return version
 
 
 def plugin(version: str):
     correct_version = get_correct_mypy_version()
     if version != correct_version:
-        print(f'''\
+        print(
+            f"""\
 You are using mypy version {version}, which is not supported
 in the PyTorch repo. Please switch to mypy version {correct_version}.
 
@@ -29,5 +32,7 @@ For example, if you installed mypy via pip, run this:
 Or if you installed mypy via conda, run this:
 
     conda install -c conda-forge mypy={correct_version}
-''', file=sys.stderr)
+""",
+            file=sys.stderr,
+        )
     return Plugin
