@@ -4,8 +4,8 @@ from os import path
 
 import torch
 
+
 # Import all utils so that getattr below can find them
-from torch.utils import bottleneck, checkpoint, model_zoo
 
 all_submod_list = [
     "",
@@ -69,10 +69,10 @@ def run(args, submod):
         if not path.exists(new_filename):
             raise RuntimeError("New version data not collected")
 
-        with open(prev_filename, "r") as f:
+        with open(prev_filename) as f:
             prev_content = set(json.load(f))
 
-        with open(new_filename, "r") as f:
+        with open(new_filename) as f:
             new_content = set(json.load(f))
 
         if not args.show_all:
