@@ -21,15 +21,12 @@ using at::cuda::CUDAStream;
 class NCCLTestBase {
  public:
   NCCLTestBase(
-      const std::string& path,
+      std::string path,
       const std::chrono::milliseconds pgTimeout =
           c10d::kProcessGroupNCCLDefaultTimeout)
-      : path_(path), pgTimeout_(pgTimeout) {}
+      : path_(std::move(path)), pgTimeout_(pgTimeout) {}
 
-  NCCLTestBase(NCCLTestBase&& other) {
-    path_ = std::move(other.path_);
-    pg_ = std::move(other.pg_);
-  }
+  NCCLTestBase(NCCLTestBase&& other) noexcept = default;
 
   std::shared_ptr<::c10d::ProcessGroupNCCL> getProcessGroup() {
     return pg_;
@@ -41,7 +38,7 @@ class NCCLTestBase {
 
   void initialize(
       int rank,
-      int size,
+      size_t size,
       std::optional<::std::shared_ptr<::c10d::ProcessGroupNCCL>> split_from =
           std::nullopt) {
     store_ = c10::make_intrusive<::c10d::FileStore>(path_, size);
@@ -55,8 +52,8 @@ class NCCLTestBase {
       opts->split_color = ++color_;
     }
 #endif
-    pg_ = std::unique_ptr<::c10d::ProcessGroupNCCL>(
-        new ::c10d::ProcessGroupNCCL(store_, rank, size, std::move(opts)));
+    pg_ = std::make_unique<::c10d::ProcessGroupNCCL>(
+        store_, rank, size, std::move(opts));
   }
 
  protected:
@@ -76,22 +73,19 @@ class NCCLTest : public NCCLTestBase {
       std::chrono::milliseconds pgTimeout =
           c10d::kProcessGroupNCCLDefaultTimeout,
       int inputDim = 3)
-      : NCCLTestBase(path, pgTimeout),
-        numDevices_(1), // one device per rank (thread)
-        rank_(rank),
-        worldSize_(worldSize) {
+      : NCCLTestBase(path, pgTimeout), rank_(rank), worldSize_(worldSize) {
     // Each device has a single tensor to perf the NCCL op
-    ::at::globalContext().lazyInitCUDA();
+    ::at::globalContext().lazyInitDevice(c10::DeviceType::CUDA);
     tensors_.resize(numDevices_);
     inputs_.resize(numDevices_);
     outputs_.resize(numDevices_);
     at::cuda::OptionalCUDAGuard deviceGuard;
     assert(numDevices_ == 1);
     for (const auto i : c10::irange(numDevices_)) {
-      deviceGuard.set_index(rank_);
+      deviceGuard.set_index(static_cast<c10::DeviceIndex>(rank_));
       tensors_[i] = at::empty({inputDim, inputDim}, at::kCUDA);
-      inputs_[i].resize(worldSize_ * numDevices_);
-      outputs_[i].resize(worldSize_ * numDevices_);
+      inputs_[i].resize(static_cast<size_t>(worldSize_) * numDevices_);
+      outputs_[i].resize(static_cast<size_t>(worldSize_) * numDevices_);
       for (auto j = 0; j < worldSize_ * numDevices_; ++j) {
         inputs_[i][j] = at::empty({inputDim, inputDim}, at::kCUDA);
         outputs_[i][j] = at::empty({inputDim, inputDim}, at::kCUDA);
@@ -106,7 +100,7 @@ class NCCLTest : public NCCLTestBase {
     // getters to retrieve the current stream).
     //
     // 1 device only, hence 1 stream only
-    deviceGuard.set_index(rank_);
+    deviceGuard.set_index(static_cast<c10::DeviceIndex>(rank_));
     streams_.push_back(at::cuda::getStreamFromPool());
   }
 
@@ -148,7 +142,8 @@ class NCCLTest : public NCCLTestBase {
       std::vector<std::vector<at::Tensor>>& tensor_lists) {
     std::vector<std::vector<at::Tensor>> outputs(numDevices_);
     for (auto& output : outputs) {
-      output = std::vector<at::Tensor>(worldSize_ * numDevices_);
+      output = std::vector<at::Tensor>(
+          static_cast<size_t>(worldSize_ * numDevices_));
     }
 
     // For the duration of this function, make THC use our streams
@@ -169,8 +164,8 @@ class NCCLTest : public NCCLTestBase {
   void launchDeviceSleep() {
     at::cuda::OptionalCUDAGuard deviceGuard;
     for (const auto i : c10::irange(numDevices_)) {
-      deviceGuard.set_index(rank_);
-      cudaSleep(streams_[i], 2000 * 1000 * 1000);
+      deviceGuard.set_index(static_cast<c10::DeviceIndex>(rank_));
+      cudaSleep(streams_[i], 2000ull * 1000 * 1000);
     }
   }
 
@@ -178,7 +173,7 @@ class NCCLTest : public NCCLTestBase {
   void valueInitialization() {
     at::cuda::OptionalCUDAGuard deviceGuard;
     for (const auto i : c10::irange(numDevices_)) {
-      deviceGuard.set_index(rank_);
+      deviceGuard.set_index(static_cast<c10::DeviceIndex>(rank_));
       tensors_[i].fill_(pg_->getRank() * numDevices_ + i);
     }
   }
@@ -199,14 +194,15 @@ class NCCLTest : public NCCLTestBase {
   void valueInitializationForSparse() {
     at::cuda::OptionalCUDAGuard deviceGuard;
     for (const auto i : c10::irange(numDevices_)) {
-      deviceGuard.set_index(rank_);
+      deviceGuard.set_index(static_cast<c10::DeviceIndex>(rank_));
       tensors_[i].fill_(pg_->getRank() * numDevices_ + i + 1);
       // Convert the dense tensor to a sparse tensor in COO row format
       tensors_[i] = to_sparse_row_indices_format(tensors_[i]);
     }
   }
 
-  const int numDevices_;
+  // NOLINTNEXTLINE(cppcoreguidelines-avoid-const-or-ref-data-members)
+  const int numDevices_{1}; // one device per rank (thread)
   int rank_;
   int worldSize_;
   std::vector<at::Tensor> tensors_;
@@ -374,7 +370,7 @@ class ReduceScatterBaseNCCLTest : public NCCLTest {
   ReduceScatterBaseNCCLTest(const std::string& path, int rank, int worldSize)
       : NCCLTest(path, rank, worldSize) {
     at::cuda::OptionalCUDAGuard deviceGuard;
-    deviceGuard.set_index(rank_);
+    deviceGuard.set_index(static_cast<c10::DeviceIndex>(rank_));
     output_tensor_ = at::empty({1}, at::kCUDA);
     input_tensor_ = at::empty({worldSize}, at::kCUDA);
     for (const auto i : c10::irange(worldSize)) {
@@ -755,7 +751,7 @@ class ProcessGroupNCCLTest : public ::testing::Test {
     std::vector<std::thread> threads;
     threads.reserve(size_);
     for (const auto rank : c10::irange(size_)) {
-      threads.emplace_back(std::thread(testFunc, file.path, rank, size_));
+      threads.emplace_back(testFunc, file.path, rank, size_);
     }
     for (const auto rank : c10::irange(size_)) {
       threads[rank].join();
@@ -765,6 +761,38 @@ class ProcessGroupNCCLTest : public ::testing::Test {
   int size_{1};
 };
 
+TEST_F(ProcessGroupNCCLTest, CUDAEventCache) {
+  if (skipTest()) {
+    return;
+  }
+
+  // Test that the CUDAEventCache can be used to create CUDA events and reuse.
+  auto event1 = c10d::ProcessGroupNCCL::CUDAEventCache::get(1).create(true);
+  auto event2 = c10d::ProcessGroupNCCL::CUDAEventCache::get(1).create(false);
+
+  auto event1_ptr = event1.get();
+  auto event2_ptr = event2.get();
+  // Mimic the behavior of the destroy of events.
+  event1 = nullptr;
+  event2 = nullptr;
+
+  // Test that the CUDAEventCache is indeed reused.
+  auto event3 = c10d::ProcessGroupNCCL::CUDAEventCache::get(2).create(true);
+  auto event4 = c10d::ProcessGroupNCCL::CUDAEventCache::get(2).create(false);
+  // The cache has been used up, new events should be created.
+  auto event5 = c10d::ProcessGroupNCCL::CUDAEventCache::get(1).create(true);
+  auto event6 = c10d::ProcessGroupNCCL::CUDAEventCache::get(1).create(false);
+  // The cache has been used up, new events should be created.
+  auto event7 = c10d::ProcessGroupNCCL::CUDAEventCache::get(1).create(true);
+  auto event8 = c10d::ProcessGroupNCCL::CUDAEventCache::get(1).create(false);
+  EXPECT_NE(event1_ptr, event3.get());
+  EXPECT_NE(event2_ptr, event4.get());
+  EXPECT_EQ(event1_ptr, event5.get());
+  EXPECT_EQ(event2_ptr, event6.get());
+  EXPECT_NE(event1_ptr, event7.get());
+  EXPECT_NE(event2_ptr, event8.get());
+}
+
 TEST_F(ProcessGroupNCCLTest, testAllreduce) {
   if (skipTest()) {
     return;
@@ -827,7 +855,7 @@ TEST_F(ProcessGroupNCCLTest, testBackendName) {
   }
   TemporaryFile file;
   auto test = NCCLTestBase(file.path);
-  test.initialize(/*rank=*/0, /*world_size=*/1);
+  test.initialize(/*rank=*/0, /*size=*/1);
   EXPECT_EQ(
       test.getProcessGroup()->getBackendName(),
       std::string(c10d::NCCL_BACKEND_NAME));
