@@ -648,6 +648,24 @@ class DistMathOpsTest(DTensorTestBase):
         distance = torch.dist(local_Q @ torch.diag(local_L) @ local_Q.mT, local_A)
         self.assertEqual(distance.item(), 0.0)
 
+    @with_comms
+    def test_upsampling(self):
+        input = torch.arange(1, 5, dtype=torch.float32).view(1, 1, 2, 2)
+        mesh = self.build_device_mesh()
+        input_dtensor = distribute_tensor(
+            input, device_mesh=mesh, placements=[Shard(0)]
+        )
+
+        upsample_m = [
+            torch.nn.UpsamplingBilinear2d(scale_factor=2),
+            torch.nn.UpsamplingNearest2d(scale_factor=2),
+            torch.nn.Upsample(scale_factor=2, mode="bicubic"),
+        ]
+        for m in upsample_m:
+            result = m(input)
+            dtensor_result = m(input_dtensor)
+            self.assertEqual(result, dtensor_result.full_tensor())
+
 
 if __name__ == "__main__":
     run_tests()
