@@ -14,7 +14,7 @@ import threading
 import time
 from abc import ABC, abstractmethod
 from base64 import b64encode
-from datetime import datetime, timedelta
+from datetime import datetime, timedelta, timezone
 from typing import Callable, cast, Optional, Tuple
 from unittest import TestCase
 from unittest.mock import call, MagicMock, Mock, patch, PropertyMock
@@ -52,6 +52,10 @@ from torch.distributed.elastic.rendezvous.dynamic_rendezvous import (
 )
 
 
+TEST_PORT = 54321
+TEST_ADDR = "host"
+
+
 class CustomAssertMixin:
     assertDictEqual: Callable
 
@@ -134,7 +138,7 @@ class RendezvousStateTest(TestCase):
         state = _RendezvousState()
         state.round = 1
         state.complete = True
-        state.deadline = datetime.utcnow()
+        state.deadline = datetime.now(timezone.utc)
         state.closed = True
 
         # fmt: off
@@ -160,8 +164,8 @@ class RendezvousStateTest(TestCase):
 
                     state.wait_list.add(node_waiting)
 
-                    state.last_heartbeats[node_running] = datetime.utcnow()
-                    state.last_heartbeats[node_waiting] = datetime.utcnow()
+                    state.last_heartbeats[node_running] = datetime.now(timezone.utc)
+                    state.last_heartbeats[node_waiting] = datetime.now(timezone.utc)
 
                 bits = pickle.dumps(state)
 
@@ -1139,7 +1143,9 @@ class TestRendezvousKeepAliveOp(AbstractTestRendezvousOp, TestCase):
 
 
 class DummyStore(Store):
-    pass
+    @property
+    def port(self) -> int:
+        return TEST_PORT
 
 
 class DynamicRendezvousHandlerTest(TestCase):
@@ -1204,11 +1210,11 @@ class DynamicRendezvousHandlerTest(TestCase):
     def test_share_store_creates_tcp_store(self):
         handler = self._create_handler()
 
-        shared_store_info = RendezvousStoreInfo("host", 54321)
+        shared_store_info = RendezvousStoreInfo(TEST_ADDR, TEST_PORT)
         with patch.object(RendezvousStoreInfo, "build", return_value=shared_store_info):
             rdzv_info = handler.next_rendezvous()
-            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, "host")
-            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, 54321)
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, TEST_ADDR)
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, TEST_PORT)
         self.assertEqual(handler._shared_tcp_store_server, self._tcp_store_mock)
 
         rdzv_info = handler.next_rendezvous()
@@ -1217,19 +1223,30 @@ class DynamicRendezvousHandlerTest(TestCase):
     def test_share_store_when_tcp_store(self):
         handler = self._create_handler()
 
-        with patch.object(dist, "PrefixStore", new=Mock):
+        class CustomPrefixStore(Mock):
+            def get(self, key):
+                return (
+                    TEST_ADDR.encode("utf-8")
+                    if key == "MASTER_ADDR"
+                    else bytes(str(TEST_PORT), "utf-8")
+                )
+
+            def set(self, key, value):
+                pass
+
+        with patch.object(dist, "PrefixStore", new=CustomPrefixStore):
             handler._store = Mock(spec=dist.TCPStore)
-            type(handler._store).host = PropertyMock(return_value="host")
-            type(handler._store).port = PropertyMock(return_value=54321)
+            type(handler._store).host = PropertyMock(return_value=TEST_ADDR)
+            type(handler._store).port = PropertyMock(return_value=TEST_PORT - 1)
             rdzv_info = handler.next_rendezvous()
-            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, "host")
-            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, 54321)
-            self.assertEqual(handler._shared_tcp_store_server, handler._store)
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, TEST_ADDR)
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, TEST_PORT)
+            self.assertNotEqual(handler._shared_tcp_store_server, handler._store)
 
             rdzv_info = handler.next_rendezvous()
-            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, "host")
-            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, 54321)
-            self.assertEqual(handler._shared_tcp_store_server, handler._store)
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_addr, TEST_ADDR)
+            self.assertEqual(rdzv_info.bootstrap_store_info.master_port, TEST_PORT)
+            self.assertNotEqual(handler._shared_tcp_store_server, handler._store)
 
     @patch("torch.distributed.elastic.rendezvous.dynamic_rendezvous._delay")
     def test_next_rendezvous_skews_the_first_join_attempt(self, mock_delay) -> None:
@@ -1405,7 +1422,9 @@ class DynamicRendezvousHandlerTest(TestCase):
         self.assertEqual(self._state.last_heartbeats[self._node], now)
 
     def _assert_keep_alive_swallows_rendezvous_errors(self) -> None:
-        last_heartbeat_time = datetime.utcnow() - (self._keep_alive_interval * 2)
+        last_heartbeat_time = datetime.now(timezone.utc) - (
+            self._keep_alive_interval * 2
+        )
 
         self._state.last_heartbeats[self._node] = last_heartbeat_time
 
@@ -1795,14 +1814,27 @@ class IntegrationTest(TestCase):
 
     @patch.object(dist, "PrefixStore")
     def test_share_tcp_store_from_backend(self, prefix_store_class_mock):
-        prefix_store = Mock(spec=dist.PrefixStore)
-        prefix_store_class_mock.return_value = prefix_store
-
-        tcp_store = Mock(spec=dist.TCPStore)
         expected_addr = "expected_address"
-        expected_port = 54321
-        type(tcp_store).host = PropertyMock(return_value=expected_addr)
-        type(tcp_store).port = PropertyMock(return_value=expected_port)
+        expected_port = 54231
+
+        class CustomPrefixStore(Mock):
+            def get(self, key):
+                return (
+                    expected_addr.encode("utf-8")
+                    if key == "MASTER_ADDR"
+                    else bytes(str(expected_port), "utf-8")
+                )
+
+            def set(self, key, value):
+                pass
+
+        prefix_store = CustomPrefixStore(spec=dist.PrefixStore)
+        prefix_store_class_mock.return_value = prefix_store
+        tcp_store = Mock(spec=dist.TCPStore)
+        original_addr = "original_addr"
+        original_port = TEST_PORT
+        type(tcp_store).host = PropertyMock(return_value=original_addr)
+        type(tcp_store).port = PropertyMock(return_value=original_port)
         # this will be injected
         self._store = tcp_store
 
