@@ -22,15 +22,18 @@ from torch.distributed.fsdp.wrap import ModuleWrapPolicy
 from torch.nn.parallel.distributed import DistributedDataParallel as DDP
 from torch.testing._internal.common_distributed import skip_if_lt_x_gpu
 from torch.testing._internal.common_fsdp import (
-    CUDAInitMode,
+    DEVICEInitMode,
     FSDPInitMode,
     FSDPTest,
+    get_devtype,
     NestedWrappedModule,
     TransformerWithSharedParams,
 )
 from torch.testing._internal.common_utils import run_tests, TEST_WITH_DEV_DBG_ASAN
 
 
+device_type = torch.device(get_devtype())
+
 if not dist.is_available():
     print("Distributed not available, skipping tests", file=sys.stderr)
     sys.exit(0)
@@ -48,10 +51,6 @@ class TestUnshardParamsBase(FSDPTest):
     This contains any methods common to both the sharded and non-sharded cases.
     """
 
-    @property
-    def device(self) -> torch.device:
-        return torch.device("cuda", self.rank)
-
     def _test_unshard_params_writeback(
         self,
         writeback: bool,
@@ -59,8 +58,8 @@ class TestUnshardParamsBase(FSDPTest):
         **fsdp_kwargs: Dict[str, Any],
     ):
         model = nn.Sequential(
-            nn.Linear(5, 5, bias=False, device=self.device),
-            nn.Linear(5, 3, bias=False, device=self.device),
+            nn.Linear(5, 5, bias=False, device=device_type.type),
+            nn.Linear(5, 3, bias=False, device=device_type.type),
         )
         model[0] = FSDP(model[0], **fsdp_kwargs)
         model = FSDP(model, **fsdp_kwargs)
@@ -125,8 +124,8 @@ class TestUnshardParamsBase(FSDPTest):
         local_model = NestedWrappedModule.init(
             self.process_group,
             FSDPInitMode.NO_FSDP,
-            CUDAInitMode.CUDA_BEFORE,
-            fsdp_kwargs={},
+            DEVICEInitMode.DEVICE_BEFORE,
+            fsdp_kwargs={"device_id": device_type.type},
             deterministic=True,
         )
         # Apply FSDP such that the root module does not have FSDP applied,
@@ -134,7 +133,7 @@ class TestUnshardParamsBase(FSDPTest):
         fsdp_model = NestedWrappedModule.init(
             self.process_group,
             FSDPInitMode.RECURSIVE,
-            CUDAInitMode.CUDA_BEFORE,
+            DEVICEInitMode.DEVICE_BEFORE,
             fsdp_kwargs={
                 "cpu_offload": cpu_offload,
                 "mixed_precision": mixed_precision,
@@ -238,7 +237,7 @@ class TestUnshardParams(TestUnshardParamsBase):
         testing that writing to padding does not persist.
         NOTE: This method depends on FSDP internals.
         """
-        model = FSDP(nn.Linear(1, 1, bias=False, device=self.device))
+        model = FSDP(nn.Linear(1, 1, bias=False, device=device_type.type))
         flat_param = model._handle.flat_param
         self.assertEqual(1, flat_param.numel())
         # Write a known value to the *sharded* `FlatParameter`
@@ -291,8 +290,10 @@ class TestUnshardParams(TestUnshardParamsBase):
         }
         model = FSDP(
             nn.Sequential(
-                FSDP(nn.Linear(5, 5, bias=False, device=self.device), **fsdp_kwargs),
-                nn.Linear(5, 3, bias=False, device=self.device),
+                FSDP(
+                    nn.Linear(5, 5, bias=False, device=device_type.type), **fsdp_kwargs
+                ),
+                nn.Linear(5, 3, bias=False, device=device_type.type),
             ),
             **fsdp_kwargs,
         )
@@ -309,7 +310,7 @@ class TestUnshardParams(TestUnshardParamsBase):
         # Validate the expected behavior: the root does not reshard after
         # forward; the non-root reshards after forward; and both reshard after
         # backward
-        output = model(torch.zeros(5, device=self.device))
+        output = model(torch.zeros(5, device=device_type.type))
         self.assertEqual(
             expected_outer_flat_param_unsharded_numel,
             _get_unsharded_storage_size(outer_flat_param),
@@ -321,7 +322,7 @@ class TestUnshardParams(TestUnshardParamsBase):
 
         # Check that with parameter unsharding in between forward and backward
         # as well as after backward, the reshard behavior matches
-        output = model(torch.zeros(5, device=self.device))
+        output = model(torch.zeros(5, device=device_type.type))
         with FSDP.summon_full_params(
             model,
             rank0_only=rank0_only,
@@ -378,8 +379,10 @@ class TestUnshardParams(TestUnshardParamsBase):
         }
         model = FSDP(
             nn.Sequential(
-                FSDP(nn.Linear(5, 5, bias=False, device=self.device), **fsdp_kwargs),
-                nn.Linear(5, 3, bias=False, device=self.device),
+                FSDP(
+                    nn.Linear(5, 5, bias=False, device=device_type.type), **fsdp_kwargs
+                ),
+                nn.Linear(5, 3, bias=False, device=device_type.type),
             ),
             **fsdp_kwargs,
         )
@@ -434,7 +437,7 @@ class TestUnshardParams(TestUnshardParamsBase):
         model = NestedWrappedModule.init(
             self.process_group,
             FSDPInitMode.NO_FSDP,
-            CUDAInitMode.CUDA_BEFORE,
+            DEVICEInitMode.DEVICE_BEFORE,
             deterministic=True,
         )
         model.buffer = nn.Buffer(torch.ones(1))
@@ -445,7 +448,7 @@ class TestUnshardParams(TestUnshardParamsBase):
             NestedWrappedModule.init(
                 self.process_group,
                 FSDPInitMode.NO_FSDP,
-                CUDAInitMode.CUDA_BEFORE,
+                DEVICEInitMode.DEVICE_BEFORE,
                 deterministic=True,
             ),
             self.process_group,
@@ -554,18 +557,19 @@ class TestUnshardParams(TestUnshardParamsBase):
         model = TransformerWithSharedParams.init(
             self.process_group,
             FSDPInitMode.NO_FSDP,
-            CUDAInitMode.CUDA_BEFORE,
+            DEVICEInitMode.DEVICE_BEFORE,
             deterministic=True,
         )
-        ddp_model = DDP(model, device_ids=[self.rank])
+        ddp_model = DDP(model, device_ids=[device_type])
         fsdp_model = TransformerWithSharedParams.init(
             self.process_group,
             FSDPInitMode.RECURSIVE,
-            CUDAInitMode.CUDA_BEFORE,
+            DEVICEInitMode.DEVICE_BEFORE,
             deterministic=True,
             fsdp_kwargs={
                 "use_orig_params": use_orig_params,
                 "sharding_strategy": sharding_strategy,
+                "device_id": device_type.type,
             },
         )
         with FSDP.summon_full_params(fsdp_model):
@@ -573,7 +577,7 @@ class TestUnshardParams(TestUnshardParamsBase):
                 assert torch.all(torch.isclose(p1, p2))
 
         # Check calling after backward
-        inp = fsdp_model.get_input(torch.device("cuda"))
+        inp = fsdp_model.get_input(torch.device(device_type))
         ddp_out = ddp_model(*inp)
         fsdp_out = fsdp_model(*inp)
         ddp_out.sum().backward()
@@ -583,7 +587,7 @@ class TestUnshardParams(TestUnshardParamsBase):
             _check_grads(ddp_model, fsdp_model, old_fsdp_grads)
 
         # Check calling between forward and backward
-        inp = fsdp_model.get_input(torch.device("cuda"))
+        inp = fsdp_model.get_input(torch.device(device_type))
         ddp_out = ddp_model(*inp)
         fsdp_out = fsdp_model(*inp)
         old_fsdp_grads = _get_fsdp_grads(fsdp_model, is_supported)
@@ -611,11 +615,12 @@ class TestUnshardParams(TestUnshardParamsBase):
         fsdp_model = TransformerWithSharedParams.init(
             self.process_group,
             FSDPInitMode.RECURSIVE,
-            CUDAInitMode.CUDA_BEFORE,
+            DEVICEInitMode.DEVICE_BEFORE,
             deterministic=True,
             fsdp_kwargs={
                 "use_orig_params": True,
                 "sharding_strategy": sharding_strategy,
+                "device_id": device_type.type,
             },
         )
         for fsdp_module in FSDP.fsdp_modules(fsdp_model):
@@ -630,7 +635,7 @@ class TestUnshardParams(TestUnshardParamsBase):
         model = nn.Sequential(
             nn.Sequential(nn.Linear(16, 16), nn.Linear(16, 16)),
             nn.Sequential(nn.Linear(16, 16), nn.Linear(16, 16)),
-        ).cuda()
+        ).to(device_type.type)
         model = FSDP(model, auto_wrap_policy=ModuleWrapPolicy((nn.Sequential,)))
         with FSDP.summon_full_params(model[0]):
             # Check that the summoned module does not have its flat parameter
@@ -684,7 +689,7 @@ class TestUnshardParamsErrors(TestUnshardParamsBase):
                 with fsdp_module.summon_full_params(fsdp_module):
                     pass
 
-        model = FSDP(MyModule()).cuda(self.rank)
+        model = FSDP(MyModule()).to(device_type.type)
         with self.assertRaisesRegex(
             AssertionError, "Cannot manually unshard parameters during forward/backward"
         ):
@@ -692,8 +697,8 @@ class TestUnshardParamsErrors(TestUnshardParamsBase):
 
     @skip_if_lt_x_gpu(2)
     def test_unshard_params_from_backward_raises(self):
-        model = FSDP(nn.Linear(2, 1, device=self.device))
-        output = model(torch.ones(2, device=self.device))
+        model = FSDP(nn.Linear(2, 1, device=device_type.type))
+        output = model(torch.ones(2, device=device_type.type))
 
         def invalid_backward_hook(*args, **kwargs):
             with FSDP.summon_full_params(model):
@@ -711,7 +716,7 @@ class TestUnshardParamsErrors(TestUnshardParamsBase):
         nested_wrapped_module = NestedWrappedModule.init(
             self.process_group,
             FSDPInitMode.RECURSIVE,
-            CUDAInitMode.CUDA_BEFORE,
+            DEVICEInitMode.DEVICE_BEFORE,
         )
         with self.assertRaisesRegex(NotImplementedError, "is not supported"):
             with FSDP.summon_full_params(
@@ -724,7 +729,7 @@ class TestUnshardParamsErrors(TestUnshardParamsBase):
         nested_wrapped_module = NestedWrappedModule.init(
             self.process_group,
             FSDPInitMode.RECURSIVE,
-            CUDAInitMode.CUDA_BEFORE,
+            DEVICEInitMode.DEVICE_BEFORE,
             {"sharding_strategy": ShardingStrategy.NO_SHARD},
         )
         with self.assertRaisesRegex(NotImplementedError, "is not supported"):
