@@ -1,14 +1,15 @@
 #!/usr/bin/env python3
 # Owner(s): ["oncall: distributed"]
 
-import pathlib
 import sys
+from pathlib import Path
 from typing import Tuple
 
 import torch
 import torch.distributed as dist
 from torch import nn, Tensor
 
+
 if not dist.is_available():
     print("Distributed not available, skipping tests", file=sys.stderr)
     sys.exit(0)
@@ -45,7 +46,7 @@ class TestInstantiator(TestCase):
         self.assertEqual(return_type_str, "Tuple[Tensor, int, str]")
 
     def test_instantiate_scripted_remote_module_template(self):
-        dir_path = pathlib.Path(instantiator.INSTANTIATED_TEMPLATE_DIR_PATH)
+        dir_path = Path(instantiator.INSTANTIATED_TEMPLATE_DIR_PATH)
 
         # Cleanup.
         file_paths = dir_path.glob(f"{instantiator._FILE_PREFIX}*.py")
@@ -69,7 +70,7 @@ class TestInstantiator(TestCase):
         self.assertEqual(num_files_after, 1)
 
     def test_instantiate_non_scripted_remote_module_template(self):
-        dir_path = pathlib.Path(instantiator.INSTANTIATED_TEMPLATE_DIR_PATH)
+        dir_path = Path(instantiator.INSTANTIATED_TEMPLATE_DIR_PATH)
 
         # Cleanup.
         file_paths = dir_path.glob(f"{instantiator._FILE_PREFIX}*.py")
