@@ -338,6 +338,21 @@ class ProcessGroupUCCTest(MultiProcessTestCase):
         for i, tensor in enumerate(tensors):
             self.assertEqual(torch.full(size, float(i * self.world_size)), tensor)
 
+    @requires_ucc()
+    def _test_reduce_scatter_base_basics(self, fn):
+        pg = self._create_process_group_ucc()
+        n = self.world_size
+        input = fn(torch.ones(n, n, 10) * (self.rank + 1.0))
+        output = fn(torch.zeros(10))
+        expected_output = fn(torch.ones(10) * (n + 1) * n / 2)
+        fut = pg._reduce_scatter_base(output, input).get_future()
+        fut.wait()
+        result = fut.value()
+        self.assertEqual(result[0], expected_output)
+
+    def test_reduce_scatter_base_basics(self):
+        self._test_reduce_scatter_base_basics(lambda t: t.clone())
+
 
 class DistributedDataParallelTest(
     test_c10d_common.CommonDistributedDataParallelTest, MultiProcessTestCase
