@@ -6,16 +6,36 @@ from numbers import Number
 import pytest
 
 import torch
+from torch.autograd import grad
 from torch.autograd.functional import jacobian
-from torch.distributions import Dirichlet, Independent, Normal, TransformedDistribution, constraints
-from torch.distributions.transforms import (AbsTransform, AffineTransform, ComposeTransform,
-                                            CorrCholeskyTransform, CumulativeDistributionTransform,
-                                            ExpTransform, IndependentTransform,
-                                            LowerCholeskyTransform, PowerTransform,
-                                            ReshapeTransform, SigmoidTransform, TanhTransform,
-                                            SoftmaxTransform, SoftplusTransform, StickBreakingTransform,
-                                            identity_transform, Transform, _InverseTransform,
-                                            PositiveDefiniteTransform)
+from torch.distributions import (
+    constraints,
+    Dirichlet,
+    Independent,
+    Normal,
+    TransformedDistribution,
+)
+from torch.distributions.transforms import (
+    _InverseTransform,
+    AbsTransform,
+    AffineTransform,
+    ComposeTransform,
+    CorrCholeskyTransform,
+    CumulativeDistributionTransform,
+    ExpTransform,
+    identity_transform,
+    IndependentTransform,
+    LowerCholeskyTransform,
+    PositiveDefiniteTransform,
+    PowerTransform,
+    ReshapeTransform,
+    SigmoidTransform,
+    SoftmaxTransform,
+    SoftplusTransform,
+    StickBreakingTransform,
+    TanhTransform,
+    Transform,
+)
 from torch.distributions.utils import tril_matrix_to_vec, vec_to_tril_matrix
 from torch.testing._internal.common_utils import run_tests
 
@@ -24,55 +44,53 @@ def get_transforms(cache_size):
     transforms = [
         AbsTransform(cache_size=cache_size),
         ExpTransform(cache_size=cache_size),
-        PowerTransform(exponent=2,
-                       cache_size=cache_size),
-        PowerTransform(exponent=torch.tensor(5.).normal_(),
-                       cache_size=cache_size),
-        PowerTransform(exponent=torch.tensor(5.).normal_(),
-                       cache_size=cache_size),
+        PowerTransform(exponent=2, cache_size=cache_size),
+        PowerTransform(exponent=-2, cache_size=cache_size),
+        PowerTransform(exponent=torch.tensor(5.0).normal_(), cache_size=cache_size),
+        PowerTransform(exponent=torch.tensor(5.0).normal_(), cache_size=cache_size),
         SigmoidTransform(cache_size=cache_size),
         TanhTransform(cache_size=cache_size),
         AffineTransform(0, 1, cache_size=cache_size),
         AffineTransform(1, -2, cache_size=cache_size),
-        AffineTransform(torch.randn(5),
-                        torch.randn(5),
-                        cache_size=cache_size),
-        AffineTransform(torch.randn(4, 5),
-                        torch.randn(4, 5),
-                        cache_size=cache_size),
+        AffineTransform(torch.randn(5), torch.randn(5), cache_size=cache_size),
+        AffineTransform(torch.randn(4, 5), torch.randn(4, 5), cache_size=cache_size),
         SoftmaxTransform(cache_size=cache_size),
         SoftplusTransform(cache_size=cache_size),
         StickBreakingTransform(cache_size=cache_size),
         LowerCholeskyTransform(cache_size=cache_size),
         CorrCholeskyTransform(cache_size=cache_size),
         PositiveDefiniteTransform(cache_size=cache_size),
-        ComposeTransform([
-            AffineTransform(torch.randn(4, 5),
-                            torch.randn(4, 5),
-                            cache_size=cache_size),
-        ]),
-        ComposeTransform([
-            AffineTransform(torch.randn(4, 5),
-                            torch.randn(4, 5),
-                            cache_size=cache_size),
-            ExpTransform(cache_size=cache_size),
-        ]),
-        ComposeTransform([
-            AffineTransform(0, 1, cache_size=cache_size),
-            AffineTransform(torch.randn(4, 5),
-                            torch.randn(4, 5),
-                            cache_size=cache_size),
-            AffineTransform(1, -2, cache_size=cache_size),
-            AffineTransform(torch.randn(4, 5),
-                            torch.randn(4, 5),
-                            cache_size=cache_size),
-        ]),
+        ComposeTransform(
+            [
+                AffineTransform(
+                    torch.randn(4, 5), torch.randn(4, 5), cache_size=cache_size
+                ),
+            ]
+        ),
+        ComposeTransform(
+            [
+                AffineTransform(
+                    torch.randn(4, 5), torch.randn(4, 5), cache_size=cache_size
+                ),
+                ExpTransform(cache_size=cache_size),
+            ]
+        ),
+        ComposeTransform(
+            [
+                AffineTransform(0, 1, cache_size=cache_size),
+                AffineTransform(
+                    torch.randn(4, 5), torch.randn(4, 5), cache_size=cache_size
+                ),
+                AffineTransform(1, -2, cache_size=cache_size),
+                AffineTransform(
+                    torch.randn(4, 5), torch.randn(4, 5), cache_size=cache_size
+                ),
+            ]
+        ),
         ReshapeTransform((4, 5), (2, 5, 2)),
         IndependentTransform(
-            AffineTransform(torch.randn(5),
-                            torch.randn(5),
-                            cache_size=cache_size),
-            1),
+            AffineTransform(torch.randn(5), torch.randn(5), cache_size=cache_size), 1
+        ),
         CumulativeDistributionTransform(Normal(0, 1)),
     ]
     transforms += [t.inv for t in transforms]
@@ -85,9 +103,17 @@ def reshape_transform(transform, shape):
         if isinstance(transform.loc, Number):
             return transform
         try:
-            return AffineTransform(transform.loc.expand(shape), transform.scale.expand(shape), cache_size=transform._cache_size)
+            return AffineTransform(
+                transform.loc.expand(shape),
+                transform.scale.expand(shape),
+                cache_size=transform._cache_size,
+            )
         except RuntimeError:
-            return AffineTransform(transform.loc.reshape(shape), transform.scale.reshape(shape), cache_size=transform._cache_size)
+            return AffineTransform(
+                transform.loc.reshape(shape),
+                transform.scale.reshape(shape),
+                cache_size=transform._cache_size,
+            )
     if isinstance(transform, ComposeTransform):
         reshaped_parts = []
         for p in transform.parts:
@@ -103,8 +129,12 @@ def reshape_transform(transform, shape):
 # Generate pytest ids
 def transform_id(x):
     assert isinstance(x, Transform)
-    name = f'Inv({type(x._inv).__name__})' if isinstance(x, _InverseTransform) else f'{type(x).__name__}'
-    return f'{name}(cache_size={x._cache_size})'
+    name = (
+        f"Inv({type(x._inv).__name__})"
+        if isinstance(x, _InverseTransform)
+        else f"{type(x).__name__}"
+    )
+    return f"{name}(cache_size={x._cache_size})"
 
 
 def generate_data(transform):
@@ -116,12 +146,17 @@ def generate_data(transform):
     if isinstance(transform.inv, ReshapeTransform):
         return torch.randn(transform.inv.out_shape)
     domain = transform.domain
-    while (isinstance(domain, constraints.independent) and
-           domain is not constraints.real_vector):
+    while (
+        isinstance(domain, constraints.independent)
+        and domain is not constraints.real_vector
+    ):
         domain = domain.base_constraint
     codomain = transform.codomain
     x = torch.empty(4, 5)
-    positive_definite_constraints = [constraints.lower_cholesky, constraints.positive_definite]
+    positive_definite_constraints = [
+        constraints.lower_cholesky,
+        constraints.positive_definite,
+    ]
     if domain in positive_definite_constraints:
         x = torch.randn(6, 6)
         x = x.tril(-1) + x.diag().exp().diag_embed()
@@ -156,21 +191,23 @@ def generate_data(transform):
         x /= x.norm(dim=-1, keepdim=True)
         x.diagonal(dim1=-1).copy_(x.diagonal(dim1=-1).abs())
         return x
-    raise ValueError(f'Unsupported domain: {domain}')
+    raise ValueError(f"Unsupported domain: {domain}")
 
 
 TRANSFORMS_CACHE_ACTIVE = get_transforms(cache_size=1)
 TRANSFORMS_CACHE_INACTIVE = get_transforms(cache_size=0)
-ALL_TRANSFORMS = TRANSFORMS_CACHE_ACTIVE + TRANSFORMS_CACHE_INACTIVE + [identity_transform]
+ALL_TRANSFORMS = (
+    TRANSFORMS_CACHE_ACTIVE + TRANSFORMS_CACHE_INACTIVE + [identity_transform]
+)
 
 
-@pytest.mark.parametrize('transform', ALL_TRANSFORMS, ids=transform_id)
+@pytest.mark.parametrize("transform", ALL_TRANSFORMS, ids=transform_id)
 def test_inv_inv(transform, ids=transform_id):
     assert transform.inv.inv is transform
 
 
-@pytest.mark.parametrize('x', TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
-@pytest.mark.parametrize('y', TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
+@pytest.mark.parametrize("x", TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
+@pytest.mark.parametrize("y", TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
 def test_equality(x, y):
     if x is y:
         assert x == y
@@ -179,7 +216,7 @@ def test_equality(x, y):
     assert identity_transform == identity_transform.inv
 
 
-@pytest.mark.parametrize('transform', ALL_TRANSFORMS, ids=transform_id)
+@pytest.mark.parametrize("transform", ALL_TRANSFORMS, ids=transform_id)
 def test_with_cache(transform):
     if transform._cache_size == 0:
         transform = transform.with_cache(1)
@@ -188,20 +225,20 @@ def test_with_cache(transform):
     try:
         y = transform(x)
     except NotImplementedError:
-        pytest.skip('Not implemented.')
+        pytest.skip("Not implemented.")
     y2 = transform(x)
     assert y2 is y
 
 
-@pytest.mark.parametrize('transform', ALL_TRANSFORMS, ids=transform_id)
-@pytest.mark.parametrize('test_cached', [True, False])
+@pytest.mark.parametrize("transform", ALL_TRANSFORMS, ids=transform_id)
+@pytest.mark.parametrize("test_cached", [True, False])
 def test_forward_inverse(transform, test_cached):
     x = generate_data(transform).requires_grad_()
     assert transform.domain.check(x).all()  # verify that the input data are valid
     try:
         y = transform(x)
     except NotImplementedError:
-        pytest.skip('Not implemented.')
+        pytest.skip("Not implemented.")
     assert y.shape == transform.forward_shape(x.shape)
     if test_cached:
         x2 = transform.inv(y)  # should be implemented at least by caching
@@ -209,26 +246,30 @@ def test_forward_inverse(transform, test_cached):
         try:
             x2 = transform.inv(y.clone())  # bypass cache
         except NotImplementedError:
-            pytest.skip('Not implemented.')
+            pytest.skip("Not implemented.")
     assert x2.shape == transform.inverse_shape(y.shape)
     y2 = transform(x2)
     if transform.bijective:
         # verify function inverse
-        assert torch.allclose(x2, x, atol=1e-4, equal_nan=True), '\n'.join([
-            f'{transform} t.inv(t(-)) error',
-            f'x = {x}',
-            f'y = t(x) = {y}',
-            f'x2 = t.inv(y) = {x2}',
-        ])
+        assert torch.allclose(x2, x, atol=1e-4, equal_nan=True), "\n".join(
+            [
+                f"{transform} t.inv(t(-)) error",
+                f"x = {x}",
+                f"y = t(x) = {y}",
+                f"x2 = t.inv(y) = {x2}",
+            ]
+        )
     else:
         # verify weaker function pseudo-inverse
-        assert torch.allclose(y2, y, atol=1e-4, equal_nan=True), '\n'.join([
-            f'{transform} t(t.inv(t(-))) error',
-            f'x = {x}',
-            f'y = t(x) = {y}',
-            f'x2 = t.inv(y) = {x2}',
-            f'y2 = t(x2) = {y2}',
-        ])
+        assert torch.allclose(y2, y, atol=1e-4, equal_nan=True), "\n".join(
+            [
+                f"{transform} t(t.inv(t(-))) error",
+                f"x = {x}",
+                f"y = t(x) = {y}",
+                f"x2 = t.inv(y) = {x2}",
+                f"y2 = t(x2) = {y2}",
+            ]
+        )
 
 
 def test_compose_transform_shapes():
@@ -252,33 +293,36 @@ base_dist1 = Dirichlet(torch.ones(4, 4))
 base_dist2 = Normal(torch.zeros(3, 4, 4), torch.ones(3, 4, 4))
 
 
-@pytest.mark.parametrize('batch_shape, event_shape, dist', [
-    ((4, 4), (), base_dist0),
-    ((4,), (4,), base_dist1),
-    ((4, 4), (), TransformedDistribution(base_dist0, [transform0])),
-    ((4,), (4,), TransformedDistribution(base_dist0, [transform1])),
-    ((4,), (4,), TransformedDistribution(base_dist0, [transform0, transform1])),
-    ((), (4, 4), TransformedDistribution(base_dist0, [transform0, transform2])),
-    ((4,), (4,), TransformedDistribution(base_dist0, [transform1, transform0])),
-    ((), (4, 4), TransformedDistribution(base_dist0, [transform1, transform2])),
-    ((), (4, 4), TransformedDistribution(base_dist0, [transform2, transform0])),
-    ((), (4, 4), TransformedDistribution(base_dist0, [transform2, transform1])),
-    ((4,), (4,), TransformedDistribution(base_dist1, [transform0])),
-    ((4,), (4,), TransformedDistribution(base_dist1, [transform1])),
-    ((), (4, 4), TransformedDistribution(base_dist1, [transform2])),
-    ((4,), (4,), TransformedDistribution(base_dist1, [transform0, transform1])),
-    ((), (4, 4), TransformedDistribution(base_dist1, [transform0, transform2])),
-    ((4,), (4,), TransformedDistribution(base_dist1, [transform1, transform0])),
-    ((), (4, 4), TransformedDistribution(base_dist1, [transform1, transform2])),
-    ((), (4, 4), TransformedDistribution(base_dist1, [transform2, transform0])),
-    ((), (4, 4), TransformedDistribution(base_dist1, [transform2, transform1])),
-    ((3, 4, 4), (), base_dist2),
-    ((3,), (4, 4), TransformedDistribution(base_dist2, [transform2])),
-    ((3,), (4, 4), TransformedDistribution(base_dist2, [transform0, transform2])),
-    ((3,), (4, 4), TransformedDistribution(base_dist2, [transform1, transform2])),
-    ((3,), (4, 4), TransformedDistribution(base_dist2, [transform2, transform0])),
-    ((3,), (4, 4), TransformedDistribution(base_dist2, [transform2, transform1])),
-])
+@pytest.mark.parametrize(
+    ("batch_shape", "event_shape", "dist"),
+    [
+        ((4, 4), (), base_dist0),
+        ((4,), (4,), base_dist1),
+        ((4, 4), (), TransformedDistribution(base_dist0, [transform0])),
+        ((4,), (4,), TransformedDistribution(base_dist0, [transform1])),
+        ((4,), (4,), TransformedDistribution(base_dist0, [transform0, transform1])),
+        ((), (4, 4), TransformedDistribution(base_dist0, [transform0, transform2])),
+        ((4,), (4,), TransformedDistribution(base_dist0, [transform1, transform0])),
+        ((), (4, 4), TransformedDistribution(base_dist0, [transform1, transform2])),
+        ((), (4, 4), TransformedDistribution(base_dist0, [transform2, transform0])),
+        ((), (4, 4), TransformedDistribution(base_dist0, [transform2, transform1])),
+        ((4,), (4,), TransformedDistribution(base_dist1, [transform0])),
+        ((4,), (4,), TransformedDistribution(base_dist1, [transform1])),
+        ((), (4, 4), TransformedDistribution(base_dist1, [transform2])),
+        ((4,), (4,), TransformedDistribution(base_dist1, [transform0, transform1])),
+        ((), (4, 4), TransformedDistribution(base_dist1, [transform0, transform2])),
+        ((4,), (4,), TransformedDistribution(base_dist1, [transform1, transform0])),
+        ((), (4, 4), TransformedDistribution(base_dist1, [transform1, transform2])),
+        ((), (4, 4), TransformedDistribution(base_dist1, [transform2, transform0])),
+        ((), (4, 4), TransformedDistribution(base_dist1, [transform2, transform1])),
+        ((3, 4, 4), (), base_dist2),
+        ((3,), (4, 4), TransformedDistribution(base_dist2, [transform2])),
+        ((3,), (4, 4), TransformedDistribution(base_dist2, [transform0, transform2])),
+        ((3,), (4, 4), TransformedDistribution(base_dist2, [transform1, transform2])),
+        ((3,), (4, 4), TransformedDistribution(base_dist2, [transform2, transform0])),
+        ((3,), (4, 4), TransformedDistribution(base_dist2, [transform2, transform1])),
+    ],
+)
 def test_transformed_distribution_shapes(batch_shape, event_shape, dist):
     assert dist.batch_shape == batch_shape
     assert dist.event_shape == event_shape
@@ -286,10 +330,10 @@ def test_transformed_distribution_shapes(batch_shape, event_shape, dist):
     try:
         dist.log_prob(x)  # this should not crash
     except NotImplementedError:
-        pytest.skip('Not implemented.')
+        pytest.skip("Not implemented.")
 
 
-@pytest.mark.parametrize('transform', TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
+@pytest.mark.parametrize("transform", TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
 def test_jit_fwd(transform):
     x = generate_data(transform).requires_grad_()
 
@@ -299,14 +343,14 @@ def test_jit_fwd(transform):
     try:
         traced_f = torch.jit.trace(f, (x,))
     except NotImplementedError:
-        pytest.skip('Not implemented.')
+        pytest.skip("Not implemented.")
 
     # check on different inputs
     x = generate_data(transform).requires_grad_()
     assert torch.allclose(f(x), traced_f(x), atol=1e-5, equal_nan=True)
 
 
-@pytest.mark.parametrize('transform', TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
+@pytest.mark.parametrize("transform", TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
 def test_jit_inv(transform):
     y = generate_data(transform.inv).requires_grad_()
 
@@ -316,14 +360,14 @@ def test_jit_inv(transform):
     try:
         traced_f = torch.jit.trace(f, (y,))
     except NotImplementedError:
-        pytest.skip('Not implemented.')
+        pytest.skip("Not implemented.")
 
     # check on different inputs
     y = generate_data(transform.inv).requires_grad_()
     assert torch.allclose(f(y), traced_f(y), atol=1e-5, equal_nan=True)
 
 
-@pytest.mark.parametrize('transform', TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
+@pytest.mark.parametrize("transform", TRANSFORMS_CACHE_INACTIVE, ids=transform_id)
 def test_jit_jacobian(transform):
     x = generate_data(transform).requires_grad_()
 
@@ -334,23 +378,23 @@ def test_jit_jacobian(transform):
     try:
         traced_f = torch.jit.trace(f, (x,))
     except NotImplementedError:
-        pytest.skip('Not implemented.')
+        pytest.skip("Not implemented.")
 
     # check on different inputs
     x = generate_data(transform).requires_grad_()
     assert torch.allclose(f(x), traced_f(x), atol=1e-5, equal_nan=True)
 
 
-@pytest.mark.parametrize('transform', ALL_TRANSFORMS, ids=transform_id)
+@pytest.mark.parametrize("transform", ALL_TRANSFORMS, ids=transform_id)
 def test_jacobian(transform):
     x = generate_data(transform)
     try:
         y = transform(x)
         actual = transform.log_abs_det_jacobian(x, y)
     except NotImplementedError:
-        pytest.skip('Not implemented.')
+        pytest.skip("Not implemented.")
     # Test shape
-    target_shape = x.shape[:x.dim() - transform.domain.event_dim]
+    target_shape = x.shape[: x.dim() - transform.domain.event_dim]
     assert actual.shape == target_shape
 
     # Expand if required
@@ -363,7 +407,9 @@ def test_jacobian(transform):
     transform = reshape_transform(transform, x_.shape)
 
     # 1. Transforms with unit jacobian
-    if isinstance(transform, ReshapeTransform) or isinstance(transform.inv, ReshapeTransform):
+    if isinstance(transform, ReshapeTransform) or isinstance(
+        transform.inv, ReshapeTransform
+    ):
         expected = x.new_zeros(x.shape[x.dim() - transform.domain.event_dim])
         expected = x.new_zeros(x.shape[x.dim() - transform.domain.event_dim])
     # 2. Transforms with 0 off-diagonal elements
@@ -377,8 +423,10 @@ def test_jacobian(transform):
         if isinstance(transform, CorrCholeskyTransform):
             jac = jacobian(lambda x: tril_matrix_to_vec(transform(x), diag=-1), x_)
         elif isinstance(transform.inv, CorrCholeskyTransform):
-            jac = jacobian(lambda x: transform(vec_to_tril_matrix(x, diag=-1)),
-                           tril_matrix_to_vec(x_, diag=-1))
+            jac = jacobian(
+                lambda x: transform(vec_to_tril_matrix(x, diag=-1)),
+                tril_matrix_to_vec(x_, diag=-1),
+            )
         elif isinstance(transform, StickBreakingTransform):
             jac = jacobian(lambda x: transform(x)[..., :-1], x_)
         else:
@@ -390,20 +438,28 @@ def test_jacobian(transform):
         # can be computed.
         gather_idx_shape = list(jac.shape)
         gather_idx_shape[-2] = 1
-        gather_idxs = torch.arange(n).reshape((n,) + (1,) * (len(jac.shape) - 1)).expand(gather_idx_shape)
+        gather_idxs = (
+            torch.arange(n)
+            .reshape((n,) + (1,) * (len(jac.shape) - 1))
+            .expand(gather_idx_shape)
+        )
         jac = jac.gather(-2, gather_idxs).squeeze(-2)
         out_ndims = jac.shape[-2]
-        jac = jac[..., :out_ndims]  # Remove extra zero-valued dims (for inverse stick-breaking).
+        jac = jac[
+            ..., :out_ndims
+        ]  # Remove extra zero-valued dims (for inverse stick-breaking).
         expected = torch.slogdet(jac).logabsdet
 
     assert torch.allclose(actual, expected, atol=1e-5)
 
 
-@pytest.mark.parametrize("event_dims",
-                         [(0,), (1,), (2, 3), (0, 1, 2), (1, 2, 0), (2, 0, 1)],
-                         ids=str)
+@pytest.mark.parametrize(
+    "event_dims", [(0,), (1,), (2, 3), (0, 1, 2), (1, 2, 0), (2, 0, 1)], ids=str
+)
 def test_compose_affine(event_dims):
-    transforms = [AffineTransform(torch.zeros((1,) * e), 1, event_dim=e) for e in event_dims]
+    transforms = [
+        AffineTransform(torch.zeros((1,) * e), 1, event_dim=e) for e in event_dims
+    ]
     transform = ComposeTransform(transforms)
     assert transform.codomain.event_dim == max(event_dims)
     assert transform.domain.event_dim == max(event_dims)
@@ -418,15 +474,17 @@ def test_compose_affine(event_dims):
     if transform.domain.event_dim > 1:
         base_dist = base_dist.expand((1,) * (transform.domain.event_dim - 1))
     dist = TransformedDistribution(base_dist, transforms)
-    assert dist.support.event_dim == max(1, max(event_dims))
+    assert dist.support.event_dim == max(1, *event_dims)
 
 
 @pytest.mark.parametrize("batch_shape", [(), (6,), (5, 4)], ids=str)
 def test_compose_reshape(batch_shape):
-    transforms = [ReshapeTransform((), ()),
-                  ReshapeTransform((2,), (1, 2)),
-                  ReshapeTransform((3, 1, 2), (6,)),
-                  ReshapeTransform((6,), (2, 3))]
+    transforms = [
+        ReshapeTransform((), ()),
+        ReshapeTransform((2,), (1, 2)),
+        ReshapeTransform((3, 1, 2), (6,)),
+        ReshapeTransform((6,), (2, 3)),
+    ]
     transform = ComposeTransform(transforms)
     assert transform.codomain.event_dim == 2
     assert transform.domain.event_dim == 2
@@ -444,16 +502,19 @@ def test_compose_reshape(batch_shape):
 @pytest.mark.parametrize("base_batch_dim", [0, 1, 2])
 @pytest.mark.parametrize("base_event_dim", [0, 1, 2])
 @pytest.mark.parametrize("num_transforms", [0, 1, 2, 3])
-def test_transformed_distribution(base_batch_dim, base_event_dim, transform_dim,
-                                  num_transforms, sample_shape):
+def test_transformed_distribution(
+    base_batch_dim, base_event_dim, transform_dim, num_transforms, sample_shape
+):
     shape = torch.Size([2, 3, 4, 5])
     base_dist = Normal(0, 1)
-    base_dist = base_dist.expand(shape[4 - base_batch_dim - base_event_dim:])
+    base_dist = base_dist.expand(shape[4 - base_batch_dim - base_event_dim :])
     if base_event_dim:
         base_dist = Independent(base_dist, base_event_dim)
-    transforms = [AffineTransform(torch.zeros(shape[4 - transform_dim:]), 1),
-                  ReshapeTransform((4, 5), (20,)),
-                  ReshapeTransform((3, 20), (6, 10))]
+    transforms = [
+        AffineTransform(torch.zeros(shape[4 - transform_dim :]), 1),
+        ReshapeTransform((4, 5), (20,)),
+        ReshapeTransform((3, 20), (6, 10)),
+    ]
     transforms = transforms[:num_transforms]
     transform = ComposeTransform(transforms)
 
@@ -491,9 +552,25 @@ def test_save_load_transform():
     stream = io.BytesIO()
     torch.save(dist, stream)
     stream.seek(0)
-    other = torch.load(stream)
+    with torch.serialization.safe_globals(
+        [TransformedDistribution, AffineTransform, Normal]
+    ):
+        other = torch.load(stream)
     assert torch.allclose(log_prob, other.log_prob(x))
 
 
+@pytest.mark.parametrize("transform", ALL_TRANSFORMS, ids=transform_id)
+def test_transform_sign(transform: Transform):
+    try:
+        sign = transform.sign
+    except NotImplementedError:
+        pytest.skip("Not implemented.")
+
+    x = generate_data(transform).requires_grad_()
+    y = transform(x).sum()
+    (derivatives,) = grad(y, [x])
+    assert torch.less(torch.as_tensor(0.0), derivatives * sign).all()
+
+
 if __name__ == "__main__":
     run_tests()
