@@ -6,12 +6,16 @@ import torch
 from torch.distributions.utils import tril_matrix_to_vec, vec_to_tril_matrix
 from torch.testing._internal.common_utils import run_tests
 
-@pytest.mark.parametrize('shape', [
-    (2, 2),
-    (3, 3),
-    (2, 4, 4),
-    (2, 2, 4, 4),
-])
+
+@pytest.mark.parametrize(
+    "shape",
+    [
+        (2, 2),
+        (3, 3),
+        (2, 4, 4),
+        (2, 2, 4, 4),
+    ],
+)
 def test_tril_matrix_to_vec(shape):
     mat = torch.randn(shape)
     n = mat.shape[-1]
