@@ -3,13 +3,18 @@ import contextlib
 import functools
 import logging
 import os
+import re
 import unittest.mock
 
 import torch
 import torch._dynamo.test_case
 import torch._dynamo.testing
 import torch.distributed as dist
-from torch._dynamo.testing import empty_line_normalizer, skipIfNotPy311
+from torch._dynamo.testing import (
+    empty_line_normalizer,
+    extract_graph_and_tracker,
+    skipIfNotPy311,
+)
 from torch._dynamo.trace_rules import _as_posix_path
 from torch.nn.parallel import DistributedDataParallel as DDP
 from torch.testing._internal.common_utils import (
@@ -31,6 +36,13 @@ requires_distributed = functools.partial(
 )
 
 
+def munge_shape_guards(s: str) -> str:
+    def munge(s):
+        return re.sub(r"[^ ]+:\d+ in [^ ]+", "#:# in #", s)
+
+    return "\n".join([munge(l) for l in s.splitlines() if "LAMBDA_GUARD" in l])
+
+
 def example_fn(a):
     output = a.mul(torch.ones(1000, 1000))
     output = output.add(torch.ones(1000, 1000))
@@ -59,7 +71,7 @@ ARGS = (torch.ones(1000, 1000, requires_grad=True),)
 def multi_record_test(num_records, **kwargs):
     @make_logging_test(**kwargs)
     def fn(self, records):
-        fn_opt = torch._dynamo.optimize("inductor")(example_fn)
+        fn_opt = torch.compile(example_fn, backend="inductor")
         fn_opt(*ARGS)
         self.assertEqual(len(records), num_records)
 
@@ -69,7 +81,7 @@ def multi_record_test(num_records, **kwargs):
 def within_range_record_test(num_records_lower, num_records_higher, **kwargs):
     @make_logging_test(**kwargs)
     def fn(self, records):
-        fn_opt = torch._dynamo.optimize("inductor")(example_fn)
+        fn_opt = torch.compile(example_fn, backend="inductor")
         fn_opt(*ARGS)
         self.assertGreaterEqual(len(records), num_records_lower)
         self.assertLessEqual(len(records), num_records_higher)
@@ -83,13 +95,13 @@ def single_record_test(**kwargs):
 
 class LoggingTests(LoggingTestCase):
     test_bytecode = multi_record_test(2, bytecode=True)
-    test_output_code = multi_record_test(2, output_code=True)
+    test_output_code = multi_record_test(3, output_code=True)
     test_aot_graphs = multi_record_test(3, aot_graphs=True)
 
     @requires_cuda
     @make_logging_test(schedule=True)
     def test_schedule(self, records):
-        fn_opt = torch._dynamo.optimize("inductor")(inductor_schedule_fn)
+        fn_opt = torch.compile(inductor_schedule_fn, backend="inductor")
         fn_opt(torch.ones(1000, 1000, device="cuda"))
         self.assertGreater(len(records), 0)
         self.assertLess(len(records), 5)
@@ -97,7 +109,7 @@ class LoggingTests(LoggingTestCase):
     @requires_cuda
     @make_logging_test(fusion=True)
     def test_fusion(self, records):
-        fn_opt = torch._dynamo.optimize("inductor")(inductor_schedule_fn)
+        fn_opt = torch.compile(inductor_schedule_fn, backend="inductor")
         fn_opt(torch.ones(1000, 1000, device="cuda"))
         self.assertGreater(len(records), 0)
         self.assertLess(len(records), 8)
@@ -115,7 +127,7 @@ class LoggingTests(LoggingTestCase):
         def fn(x, y):
             return torch.add(x, y)
 
-        fn_opt = torch._dynamo.optimize("inductor")(fn)
+        fn_opt = torch.compile(fn, backend="inductor")
         fn_opt(torch.ones(1000, 1000), torch.ones(1000, 1000))
         fn_opt(torch.ones(1000, 1000), 1)
         self.assertGreater(len(records), 0)
@@ -126,7 +138,7 @@ class LoggingTests(LoggingTestCase):
     @skipIfTorchDynamo("too slow")
     @make_logging_test(dynamo=logging.DEBUG)
     def test_dynamo_debug_default_off_artifacts(self, records):
-        fn_opt = torch._dynamo.optimize("inductor")(example_fn)
+        fn_opt = torch.compile(example_fn, backend="inductor")
         fn_opt(torch.ones(1000, 1000))
         self.assertEqual(len([r for r in records if ".__bytecode" in r.name]), 0)
         self.assertEqual(len([r for r in records if ".__output_code" in r.name]), 0)
@@ -134,7 +146,7 @@ class LoggingTests(LoggingTestCase):
     @make_logging_test()
     def test_dynamo_error(self, records):
         try:
-            fn_opt = torch._dynamo.optimize("inductor")(dynamo_error_fn)
+            fn_opt = torch.compile(dynamo_error_fn, backend="inductor")
             fn_opt(*ARGS)
         except Exception:
             pass
@@ -154,8 +166,8 @@ from user code:
         )
 
     test_aot = within_range_record_test(2, 6, aot=logging.INFO)
-    test_inductor_debug = within_range_record_test(3, 17, inductor=logging.DEBUG)
-    test_inductor_info = within_range_record_test(2, 4, inductor=logging.INFO)
+    test_inductor_debug = within_range_record_test(3, 22, inductor=logging.DEBUG)
+    test_inductor_info = within_range_record_test(2, 9, inductor=logging.INFO)
 
     @make_logging_test()
     def test_inductor_error(self, records):
@@ -176,7 +188,7 @@ from user code:
         )
 
         try:
-            fn_opt = torch._dynamo.optimize("inductor")(inductor_error_fn)
+            fn_opt = torch.compile(inductor_error_fn, backend="inductor")
             fn_opt(*ARGS)
         except Exception:
             pass
@@ -189,15 +201,6 @@ due to:
 Traceback (most recent call last):
   File "test_logging.py", line N, in throw
     raise AssertionError
-torch._inductor.exc.LoweringException: AssertionError:
-  target: aten.round.default
-  args[0]: TensorBox(StorageBox(
-    InputBuffer(name='primals_1', layout=FixedLayout('cpu', torch.float32, size=[1000, 1000], stride=[1000, 1]))
-  ))
-
-The above exception was the direct cause of the following exception:
-
-Traceback (most recent call last):
 torch._dynamo.exc.BackendCompilerFailed: backend='inductor' raised:
 LoweringException: AssertionError:
   target: aten.round.default
@@ -227,9 +230,8 @@ LoweringException: AssertionError:
         os.environ["MASTER_PORT"] = str(find_free_port())
         dist.init_process_group("gloo", rank=0, world_size=1)
 
-        ddp_model = torch._dynamo.optimize("inductor")(
-            DDP(ToyModel().to("cuda:0"), device_ids=[0], bucket_cap_mb=4)
-        )
+        model = DDP(ToyModel().to("cuda:0"), device_ids=[0], bucket_cap_mb=4)
+        ddp_model = torch.compile(model, backend="inductor")
 
         ddp_model(torch.randn(1024, 1024, device="cuda:0"))
 
@@ -284,7 +286,7 @@ LoweringException: AssertionError:
 
     @make_logging_test(graph_breaks=True)
     def test_graph_breaks(self, records):
-        @torch._dynamo.optimize("inductor")
+        @torch.compile(backend="inductor")
         def fn(x):
             torch._dynamo.graph_break()
             return x + 1
@@ -295,7 +297,7 @@ LoweringException: AssertionError:
 
     @make_settings_test("torch._dynamo.utils")
     def test_dump_compile_times(self, records):
-        fn_opt = torch._dynamo.optimize("inductor")(example_fn)
+        fn_opt = torch.compile(example_fn, backend="inductor")
         fn_opt(torch.ones(1000, 1000))
         # This function runs during exit via atexit.register.
         # We're not actually going to run atexit._run_exit_funcs() here,
@@ -373,7 +375,7 @@ LoweringException: AssertionError:
                 return x * 2
             return x * 3
 
-        fn_opt = torch._dynamo.optimize("eager")(fn)
+        fn_opt = torch.compile(fn, backend="eager")
         fn_opt(torch.ones(3, 3))
 
         found_x2 = False
@@ -401,7 +403,7 @@ LoweringException: AssertionError:
         def fn3(x):
             return x * 4
 
-        fn_opt = torch._dynamo.optimize("eager")(fn1)
+        fn_opt = torch.compile(fn1, backend="eager")
         fn_opt(torch.ones(3, 3))
 
         found_x2 = False
@@ -438,7 +440,7 @@ LoweringException: AssertionError:
         def outer(pred, x):
             return inner(pred, x)
 
-        fn_opt = torch._dynamo.optimize("eager")(outer)
+        fn_opt = torch.compile(outer, backend="eager")
         fn_opt(torch.tensor(True), torch.ones(3, 3))
 
         found_x2 = False
@@ -466,7 +468,7 @@ LoweringException: AssertionError:
 
             return fn2()
 
-        fn_opt = torch._dynamo.optimize("eager")(fn1)
+        fn_opt = torch.compile(fn1, backend="eager")
         fn_opt()
 
         found_funcname = False
@@ -506,7 +508,7 @@ print("arf")
         def fn(x, y):
             return (x * 2) @ (y * 3)
 
-        fn_opt = torch._dynamo.optimize("eager")(fn)
+        fn_opt = torch.compile(fn, backend="eager")
         fn_opt(torch.randn(10, 20), torch.randn(20, 30))
 
         self.assertEqual(len(records), 3)
@@ -533,6 +535,24 @@ print("arf")
                    ~~~~~~~~^~~~~~~~~""",
         )
 
+    @skipIfNotPy311
+    @make_logging_test(trace_call=True)
+    def test_trace_call_prefix(self, records):
+        def fn(x, y):
+            return (x * 2) @ (y * 3)
+
+        fn_opt = torch.compile(fn, backend="eager")
+        fn_opt(torch.randn(10, 20), torch.randn(20, 30))
+
+        msg0 = munge_exc(records[0].getMessage())
+        self.assertExpectedInline(
+            msg0,
+            """\
+TRACE FX call mul from test_logging.py:N in fn (LoggingTests.test_trace_call_prefix.fn)
+            return (x * 2) @ (y * 3)
+                    ~~^~~""",
+        )
+
     @skipIfNotPy311
     @make_logging_test(trace_call=True)
     def test_trace_call_inline_call(self, records):
@@ -542,7 +562,7 @@ print("arf")
         def f(x):
             return g(g(x))
 
-        fn_opt = torch._dynamo.optimize("eager")(f)
+        fn_opt = torch.compile(f, backend="eager")
         fn_opt(torch.randn(3, 3))
 
         self.assertEqual(len(records), 4)
@@ -561,12 +581,14 @@ print("arf")
             return x * 2
                    ~~^~~""",
         )
-        self.assertExpectedInline(
-            messages[2],
-            """\
-            return g(g(x))
-                   ~^^^^^^""",
-        )
+        # skip this check since 3.13 removed carets for this case
+        # see https://github.com/python/cpython/issues/99180
+        # self.assertExpectedInline(
+        #     messages[2],
+        #     """\
+        #     return g(g(x))
+        #            ~^^^^^^""",
+        # )
         self.assertExpectedInline(
             messages[3],
             """\
@@ -582,7 +604,7 @@ print("arf")
             torch._dynamo.graph_break()
             return x * 3
 
-        fn_opt = torch._dynamo.optimize("eager")(fn)
+        fn_opt = torch.compile(fn, backend="eager")
         fn_opt(torch.randn(3, 3))
 
         self.assertEqual(len(records), 3)
@@ -616,7 +638,7 @@ print("arf")
         zs = [3.0]
         x = torch.tensor([1.0])
 
-        fn_opt = torch._dynamo.optimize("eager")(fn)
+        fn_opt = torch.compile(fn, backend="eager")
         fn_opt(x, ys, zs)
         fn_opt(x, ys[:1], zs)
 
@@ -631,6 +653,64 @@ print("arf")
             record_str,
         )
 
+    @make_logging_test(guards=True)
+    def test_guards_sloc(self, records):
+        @torch.compile(dynamic=True, backend="eager")
+        def f(x, y, z):
+            x = x * 3
+            if x.size(0) % 3 == 0:
+                return x + torch.cat([y, z])
+            else:
+                return x * 2
+
+        f(torch.randn(6), torch.randn(3), torch.randn(3))
+
+        record = self.getRecord(records, "TREE_GUARD_MANAGER")
+        self.assertExpectedInline(
+            munge_shape_guards(record.getMessage()),
+            """\
++- LAMBDA_GUARD: L['x'].size()[0] == 2*L['z'].size()[0]  # return x + torch.cat([y, z])  # #:# in # #:# in #
++- LAMBDA_GUARD: L['y'].size()[0] == L['z'].size()[0]  # duck sizing added this equality because these variables had the same size 3 (to avoid this specialization, set torch.fx.experimental._config.use_duck_shape = False)
++- LAMBDA_GUARD: ((2*L['z'].size()[0]) % 3) == 0  # if x.size(0) % 3 == 0:  # #:# in # #:# in #
++- LAMBDA_GUARD: 2 <= L['z'].size()[0]  # return x + torch.cat([y, z])  # #:# in # (user code shown is first use of this value--the guard itself is not due user code but due to 0/1 specialization in the framework; to avoid specialization try torch._dynamo.mark_unbacked(tensor, dim))""",  # noqa: B950
+        )
+
+    @make_logging_test(guards=True)
+    def test_guards_polyfill_sloc(self, records):
+        @torch.compile(dynamic=True, backend="eager")
+        def f(x, y):
+            return any([x.size(0) == y.size(0) * 2])
+
+        f(torch.randn(6), torch.randn(3))
+
+        record = self.getRecord(records, "TREE_GUARD_MANAGER")
+        self.assertExpectedInline(
+            munge_shape_guards(record.getMessage()),
+            """\
++- LAMBDA_GUARD: L['x'].size()[0] == 2*L['y'].size()[0]  # return any([x.size(0) == y.size(0) * 2])  # #:# in # #:# in #
++- LAMBDA_GUARD: 2 <= L['y'].size()[0]  # return any([x.size(0) == y.size(0) * 2])  # #:# in # (user code shown is first use of this value--the guard itself is not due user code but due to 0/1 specialization in the framework; to avoid specialization try torch._dynamo.mark_unbacked(tensor, dim))""",  # noqa: B950
+        )
+
+    @make_logging_test(guards=True)
+    def test_guards_sloc_vr(self, records):
+        @torch.compile(dynamic=True, backend="eager")
+        def f(x, y):
+            torch._check(x.size(0) > 5)
+            torch._check(x.size(0) < 30)
+            torch._check(x.size(0) == y.size(0) * 2)
+            return torch.tensor(True)
+
+        f(torch.randn(6), torch.randn(3))
+
+        record = self.getRecord(records, "TREE_GUARD_MANAGER")
+        self.assertExpectedInline(
+            munge_shape_guards(record.getMessage()),
+            """\
++- LAMBDA_GUARD: L['x'].size()[0] == 2*L['y'].size()[0]  # torch._check(x.size(0) == y.size(0) * 2)  # #:# in # #:# in #
++- LAMBDA_GUARD: 3 <= L['y'].size()[0]  # torch._check(x.size(0) > 5)  # #:# in # #:# in #
++- LAMBDA_GUARD: L['y'].size()[0] <= 14  # torch._check(x.size(0) < 30)  # #:# in # #:# in #""",  # noqa: B950
+        )
+
     @make_logging_test(cudagraph_static_inputs=True)
     def test_cudagraph_static_inputs(self, records):
         @torch.compile(mode="reduce-overhead")
@@ -643,6 +723,41 @@ print("arf")
         self.assertGreater(len(records), 0)
         self.assertLess(len(records), 4)
 
+    @make_logging_test(perf_hints=True)
+    @requires_cuda
+    def test_optimizer_non_static_param(self, records):
+        params = [torch.randn(10, 10, device="cuda") for _ in range(2)]
+        for param in params:
+            param.grad = torch.zeros_like(param)
+        opt = torch.optim.Adam(params)
+        compiled_opt_step = torch.compile(opt.step, mode="reduce-overhead")
+        compiled_opt_step()
+        self.assertGreater(len(records), 0)
+        self.assertLess(len(records), 3)
+
+    @make_logging_test(graph_region_expansion=True)
+    def test_graph_region_expansion(self, records):
+        with torch._dynamo.config.patch("track_nodes_for_deduplication", True):
+
+            def inner_fn(x, y):
+                x0 = x + 1
+                y0 = y + 2
+                z = x0.sum() + y0.sum()
+                return z
+
+            def fn(x, y):
+                o0 = inner_fn(x, y)
+                o1 = torch.sin(o0)
+                o2 = inner_fn(x, o1)
+                o3 = inner_fn(x, y)
+                return o2 * o3 * o3
+
+            graph, tracker = extract_graph_and_tracker(
+                fn, torch.randn(10, 10), torch.randn(10, 10)
+            )
+            tracker.get_identical_regions(graph)
+            self.assertGreater(len(records), 0)
+
     @skipIfTorchDynamo("too slow")
     @make_logging_test(**torch._logging.DEFAULT_LOGGING)
     def test_default_logging(self, records):
@@ -654,7 +769,7 @@ print("arf")
             print("hello")
             return a + 1
 
-        fn_opt = torch._dynamo.optimize("eager")(fn)
+        fn_opt = torch.compile(fn, backend="eager")
         fn_opt(torch.ones(10, 10))
         fn_opt(-torch.ones(10, 5))
 
@@ -705,6 +820,40 @@ fn(torch.randn(5))
                     empty_line_normalizer(stderr.decode("utf-8")),
                 )
 
+    @make_settings_test("torch._dynamo.eval_frame")
+    def test_log_traced_frames(self, records):
+        # Test program
+        @torch.compile()
+        def foo():
+            x = torch.ones([10])
+
+            def bar():
+                y = x + x
+                torch._dynamo.graph_break()
+                z = y * x
+                return z
+
+            return bar(), bar
+
+        foo()
+
+        # `_log_traced_frames` is registered as an atexit callback, so we invoke
+        # it explicitly for testing.
+        torch._dynamo.eval_frame._log_traced_frames()
+
+        # Get the relevant log.
+        record = self.getRecord(records, "TorchDynamo attempted to trace")
+
+        # Check
+        self.assertExpectedInline(
+            munge_exc(record.getMessage()),
+            """\
+TorchDynamo attempted to trace the following frames: [
+  * foo test_logging.py:N
+  * bar test_logging.py:N
+]""",
+        )
+
 
 # single record tests
 exclusions = {
@@ -742,6 +891,7 @@ exclusions = {
     "cudagraph_static_inputs",
     "benchmarking",
     "loop_ordering",
+    "graph_region_expansion",
 }
 for name in torch._logging._internal.log_registry.artifact_names:
     if name not in exclusions:
