@@ -1,20 +1,19 @@
 # Owner(s): ["module: dynamo"]
-import functools
 import unittest
 
 import torch._dynamo
 from torch._dynamo.test_minifier_common import MinifierTestBase
+from torch.testing._internal.common_utils import skipIfNNModuleInlined
 
-requires_cuda = functools.partial(
-    unittest.skipIf, not torch.cuda.is_available(), "requires cuda"
-)
+
+requires_cuda = unittest.skipUnless(torch.cuda.is_available(), "requires cuda")
 
 
 class MinifierTests(MinifierTestBase):
     # Test that compile, runtime, and accuracy errors after dynamo can be repro'd (both CPU and CUDA)
     def _test_after_dynamo(self, device, backend, expected_error):
         run_code = f"""\
-@torch._dynamo.optimize({backend!r})
+@torch.compile(backend={backend!r})
 def inner(x):
     for _ in range(10):
         x = torch.sin(x)
@@ -42,19 +41,19 @@ inner(torch.randn(20, 20).to("{device}"))
             "cpu", "relu_accuracy_error_TESTING_ONLY", "AccuracyError"
         )
 
-    @requires_cuda()
+    @requires_cuda
     def test_after_dynamo_cuda_compile_error(self):
         self._test_after_dynamo(
             "cuda", "relu_compile_error_TESTING_ONLY", "ReluCompileError"
         )
 
-    @requires_cuda()
+    @requires_cuda
     def test_after_dynamo_cuda_runtime_error(self):
         self._test_after_dynamo(
             "cuda", "relu_runtime_error_TESTING_ONLY", "ReluRuntimeError"
         )
 
-    @requires_cuda()
+    @requires_cuda
     def test_after_dynamo_cuda_accuracy_error(self):
         self._test_after_dynamo(
             "cuda", "relu_accuracy_error_TESTING_ONLY", "AccuracyError"
@@ -62,7 +61,7 @@ inner(torch.randn(20, 20).to("{device}"))
 
     def test_after_dynamo_non_leaf_compile_error(self):
         run_code = """\
-@torch._dynamo.optimize("non_leaf_compile_error_TESTING_ONLY")
+@torch.compile(backend="non_leaf_compile_error_TESTING_ONLY")
 def inner(x):
     return x + 1
 
@@ -74,7 +73,7 @@ inner(torch.randn(20, 20, requires_grad=True) + 1)
 
     # Ensure that the testing backends pass when relu is not present.
     def _test_after_dynamo_backend_passes(self, device, backend):
-        @torch._dynamo.optimize(backend)
+        @torch.compile(backend=backend)
         def inner(x):
             for _ in range(10):
                 x = torch.sin(x)
@@ -95,45 +94,46 @@ inner(torch.randn(20, 20, requires_grad=True) + 1)
             "cpu", "relu_accuracy_error_TESTING_ONLY"
         )
 
-    @requires_cuda()
+    @requires_cuda
     def test_after_dynamo_cuda_compile_backend_passes(self):
         self._test_after_dynamo_backend_passes(
             "cuda", "relu_compile_error_TESTING_ONLY"
         )
 
-    @requires_cuda()
+    @requires_cuda
     def test_after_dynamo_cuda_runtime_backend_passes(self):
         self._test_after_dynamo_backend_passes(
             "cuda", "relu_runtime_error_TESTING_ONLY"
         )
 
-    @requires_cuda()
+    @requires_cuda
     def test_after_dynamo_cuda_accuracy_backend_passes(self):
         self._test_after_dynamo_backend_passes(
             "cuda", "relu_accuracy_error_TESTING_ONLY"
         )
 
     # Test that a module with mixed cpu/cuda parts with an error after dynamo can be repro'd
-    @requires_cuda()
+    @skipIfNNModuleInlined()
+    @requires_cuda
     def test_cpu_cuda_module_after_dynamo(self):
         backend_name = "relu_compile_error_TESTING_ONLY"
         run_code = f"""\
 class CpuCudaModule(torch.nn.Module):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.m_x = torch.nn.Linear(20, 20).cuda()
         self.m_y = torch.nn.Linear(20, 20)
         self.p_x = torch.nn.Parameter(torch.randn(20, 20).cuda())
         self.p_y = torch.nn.Parameter(torch.randn(20, 20))
-        self.register_buffer("b_x", torch.ones(20, 20).cuda())
-        self.register_buffer("b_y", torch.ones(20, 20))
+        self.b_x = torch.nn.Buffer(torch.ones(20, 20).cuda())
+        self.b_y = torch.nn.Buffer(torch.ones(20, 20))
 
     def forward(self, x, y):
         return self.m_x(x) + self.p_x + self.b_x, self.m_y(y) + self.p_y + self.b_y
 
 mod = CpuCudaModule()
 
-@torch._dynamo.optimize({backend_name!r})
+@torch.compile(backend={backend_name!r})
 def inner(x1, y1):
     x2 = torch.randn(20, 20).cuda()
     y2 = torch.randn(20, 20)
@@ -149,7 +149,7 @@ inner(torch.randn(20, 20).cuda(), torch.randn(20, 20))
             res.minifier_module(),
             """\
 class Repro(torch.nn.Module):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.G__mod___m_x = Linear(in_features=20, out_features=20, bias=True).cuda()
         self.G__mod___m_y = Linear(in_features=20, out_features=20, bias=True)
@@ -162,22 +162,22 @@ class Repro(torch.nn.Module):
         l_x1_ = L_x1_
         l_y1_ = L_y1_
         randn = torch.randn(20, 20)
-        cuda = randn.cuda();  randn = None
-        randn_1 = torch.randn(20, 20)
-        add = l_x1_ + cuda;  l_x1_ = cuda = None
-        add_1 = l_y1_ + randn_1;  l_y1_ = randn_1 = None
+        x2 = randn.cuda();  randn = None
+        y2 = torch.randn(20, 20)
+        add = l_x1_ + x2;  l_x1_ = x2 = None
+        add_1 = l_y1_ + y2;  l_y1_ = y2 = None
         g__mod___m_x = self.G__mod___m_x(add);  add = None
         g__mod___p_x = self.G__mod___p_x
         add_2 = g__mod___m_x + g__mod___p_x;  g__mod___m_x = g__mod___p_x = None
         g__mod___b_x = self.G__mod___b_x
-        add_3 = add_2 + g__mod___b_x;  add_2 = g__mod___b_x = None
+        x3 = add_2 + g__mod___b_x;  add_2 = g__mod___b_x = None
         g__mod___m_y = self.G__mod___m_y(add_1);  add_1 = None
         g__mod___p_y = self.G__mod___p_y
         add_4 = g__mod___m_y + g__mod___p_y;  g__mod___m_y = g__mod___p_y = None
         g__mod___b_y = self.G__mod___b_y
-        add_5 = add_4 + g__mod___b_y;  add_4 = g__mod___b_y = None
-        cpu = add_3.cpu();  add_3 = None
-        add_6 = cpu + add_5;  cpu = add_5 = None
+        y3 = add_4 + g__mod___b_y;  add_4 = g__mod___b_y = None
+        cpu = x3.cpu();  x3 = None
+        add_6 = cpu + y3;  cpu = y3 = None
         relu = torch.relu(add_6);  add_6 = None
         return (relu,)""",
         )
@@ -186,7 +186,7 @@ class Repro(torch.nn.Module):
     def test_if_graph_minified(self):
         backend_name = "relu_compile_error_TESTING_ONLY"
         run_code = f"""\
-@torch._dynamo.optimize({backend_name!r})
+@torch.compile(backend={backend_name!r})
 def inner(x):
     for _ in range(20):
         x = torch.sin(x)
@@ -204,12 +204,12 @@ inner(torch.randn(20, 20))
             res.repro_module(),
             """\
 class Repro(torch.nn.Module):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
 
-    def forward(self, sin_19):
-        relu = torch.relu(sin_19);  sin_19 = None
-        return (relu,)""",
+    def forward(self, x_19):
+        x_20 = torch.relu(x_19);  x_19 = None
+        return (x_20,)""",
         )
 
 
