@@ -3,10 +3,12 @@
 try:
     from . import test_export, testing
 except ImportError:
-    import test_export
-    import testing
+    import test_export  # @manual=fbcode//caffe2/test:test_export-library
+    import testing  # @manual=fbcode//caffe2/test:test_export-library
+
 from torch.export import export
 
+
 test_classes = {}
 
 
