@@ -7,6 +7,7 @@ with test_sym_bool)
 # Owner(s): ["oncall: export"]
 import copy
 import io
+import math
 import tempfile
 import unittest
 import zipfile
@@ -28,7 +29,7 @@ from torch._export.serde.serialize import (
 )
 from torch._higher_order_ops.torchbind import enable_torchbind_tracing
 from torch._subclasses.fake_tensor import FakeTensor, FakeTensorMode
-from torch.export import Dim, export, load, save
+from torch.export import Dim, export_for_training, load, save
 from torch.fx.experimental.symbolic_shapes import is_concrete_int, ValueRanges
 from torch.testing._internal.common_utils import (
     instantiate_parametrized_tests,
@@ -95,7 +96,7 @@ class TestSerialize(TestCase):
                 return torch.ops.aten.add.Tensor._schema
 
         inp = (torch.ones(10),)
-        ep = export(TestModule(), inp)
+        ep = export_for_training(TestModule(), inp)
 
         # Register the custom op handler.
         foo_custom_op = FooExtensionOp()
@@ -160,7 +161,7 @@ class TestSerialize(TestCase):
 
         model = MyModule().eval()
         random_inputs = (torch.rand([2, 3]), torch.rand([2, 3]))
-        exp_program = torch.export.export(model, random_inputs, {"use_p": True})
+        exp_program = export_for_training(model, random_inputs, {"use_p": True})
 
         output_buffer = io.BytesIO()
         # Tests that example inputs are preserved when saving and loading module.
@@ -174,6 +175,28 @@ class TestSerialize(TestCase):
         loaded_out = loaded_model.module()(*loaded_args, **loaded_kwargs)
         self.assertEqual(orig_out, loaded_out)
 
+    def test_metadata_run_decomp_serder(self):
+        class M(torch.nn.Module):
+            def forward(self, x):
+                return x.sin()
+
+        exp_program = export_for_training(M(), (torch.randn(4, 4),))
+
+        output_buffer = io.BytesIO()
+        # Tests that example forward arg names are preserved when saving and loading module.
+        torch.export.save(exp_program, output_buffer)
+        loaded_model = torch.export.load(output_buffer)
+
+        ep = loaded_model.run_decompositions({})
+        # We should preserve the original module name
+        self.assertExpectedInline(
+            str(ep.graph_module.code).strip(),
+            """\
+def forward(self, x):
+    sin = torch.ops.aten.sin.default(x);  x = None
+    return (sin,)""",
+        )
+
     def test_metadata_parsing_with_layer_split(self):
         # Tests that modules with more complicated layer patterns can be serialized
         # and deserialized correctly.
@@ -197,7 +220,7 @@ class TestSerialize(TestCase):
         inp = (torch.ones(10),)
         # Module will only be able to roundtrip if metadata
         # can be correctly parsed.
-        ep = export(MyModule(), inp)
+        ep = export_for_training(MyModule(), inp)
         buffer = io.BytesIO()
         save(ep, buffer)
         loaded_ep = load(buffer)
@@ -220,7 +243,7 @@ class TestSerialize(TestCase):
 
         # Check that module can be roundtripped, thereby confirming proper deserialization.
         inp = (torch.ones(10),)
-        ep = export(MyModule(), inp)
+        ep = export_for_training(MyModule(), inp)
         buffer = io.BytesIO()
         save(ep, buffer)
         loaded_ep = load(buffer)
@@ -243,7 +266,7 @@ class TestSerialize(TestCase):
                     eps=1e-5,
                 )
 
-        exported_module = export(
+        exported_module = export_for_training(
             MyModule(),
             (
                 torch.ones([512, 512], requires_grad=True),
@@ -286,7 +309,7 @@ class TestSerialize(TestCase):
             "b": {1: dim1_bc},
             "c": {0: dim0_ac, 1: dim1_bc},
         }
-        exported_module = export(
+        exported_module = export_for_training(
             DynamicShapeSimpleModel(), inputs, dynamic_shapes=dynamic_shapes
         ).run_decompositions()
         serialized = ExportedProgramSerializer().serialize(exported_module)
@@ -299,6 +322,59 @@ class TestSerialize(TestCase):
             self.assertEqual(node.inputs[0].name, "self")
             self.assertEqual(node.inputs[1].name, "dim")
 
+    def test_serialize_sym_float(self) -> None:
+        class DynamicFloatSimpleModel(torch.nn.Module):
+            def __init__(self, multiplier: torch.SymFloat):
+                super().__init__()
+                self.multiplier = multiplier
+
+            def forward(self, a, b, c) -> torch.Tensor:
+                d = (torch.matmul(a, b) + c) / 2
+                e = d * self.multiplier
+                e_s0 = e.shape[0]
+                e_s1 = e.shape[1]
+                e_s3 = e_s0 * e_s1
+                f = e.view(e_s3)
+                return torch.cat([f, f])
+
+        multiplier_sym = torch.SymFloat("multiplier_sym")
+        model = DynamicFloatSimpleModel(multiplier_sym)
+        inputs = (
+            torch.randn(2, 4),
+            torch.randn(4, 7),
+            torch.randn(2, 7),
+        )
+        dim0_ac = Dim("dim0_ac")
+        dim1_bc = Dim("dim1_b")
+
+    def test_serialize_infinite_sym_int(self) -> None:
+        class DynamicShapeSimpleModel(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+
+            def forward(self, a, b, c) -> torch.Tensor:
+                d = (torch.matmul(a, b) + c) / 2
+                d_s0 = d.shape[0]
+                d_s1 = d.shape[1]
+                d_s3 = d_s0 * d_s1
+                e = d.view(d_s3)
+                return torch.cat([e, e])
+
+        inputs = (torch.randn(2, 4), torch.randn(4, 7), torch.randn(2, 7))
+        dim0_ac = torch.export.Dim("dim0_ac")
+        dim1_bc = torch.export.Dim("dim1_b")
+        dynamic_shapes = {
+            "a": {0: dim0_ac},
+            "b": {1: dim1_bc},
+            "c": {0: dim0_ac, 1: dim1_bc},
+        }
+        exported_module = export_for_training(
+            DynamicShapeSimpleModel(), inputs, dynamic_shapes=dynamic_shapes
+        ).run_decompositions()
+        serialized = ExportedProgramSerializer().serialize(exported_module)
+        for v in serialized.exported_program.range_constraints.values():
+            self.assertEqual(v.max_val, None)
+
     def test_serialize_list_returns(self) -> None:
         class MyModule(torch.nn.Module):
             def __init__(self) -> None:
@@ -308,7 +384,7 @@ class TestSerialize(TestCase):
                 return torch.split(x, 2)
 
         input = torch.arange(10.0).reshape(5, 2)
-        exported_module = export(MyModule(), (input,)).run_decompositions()
+        exported_module = export_for_training(MyModule(), (input,)).run_decompositions()
 
         serialized = ExportedProgramSerializer().serialize(exported_module)
         node = serialized.exported_program.graph_module.graph.nodes[-1]
@@ -349,7 +425,7 @@ class TestSerialize(TestCase):
             def forward(self, x):
                 return torch.ops.aten.var_mean.correction(x, [1])[0]
 
-        exported_module = export(
+        exported_module = export_for_training(
             MyModule(),
             (torch.ones([512, 512], requires_grad=True),),
         ).run_decompositions()
@@ -371,7 +447,7 @@ class TestSerialize(TestCase):
             def forward(self, x):
                 return x + x
 
-        ep = torch.export.export(
+        ep = export_for_training(
             M(), (torch.randn(4),), dynamic_shapes=({0: Dim("temp")},)
         )
 
@@ -403,7 +479,7 @@ class TestSerialize(TestCase):
         f = Foo()
 
         x, _ = torch.sort(torch.randn(3, 4))
-        exported_module = export(f, (x,)).run_decompositions()
+        exported_module = export_for_training(f, (x,)).run_decompositions()
         serialized = ExportedProgramSerializer().serialize(exported_module)
 
         node = serialized.exported_program.graph_module.graph.nodes[-1]
@@ -421,7 +497,7 @@ class TestSerialize(TestCase):
                 b = x + y
                 return b + a
 
-        ep = torch.export.export(Module(), (torch.randn(3, 2), torch.randn(3, 2)))
+        ep = export_for_training(Module(), (torch.randn(3, 2), torch.randn(3, 2)))
         s = ExportedProgramSerializer().serialize(ep)
         c = canonicalize(s.exported_program)
         g = c.graph_module.graph
@@ -435,7 +511,7 @@ class TestSerialize(TestCase):
             def forward(self, x):
                 return torch.ops.aten.sum.dim_IntList(x, [])
 
-        ep = torch.export.export(M(), (torch.randn(3, 2),))
+        ep = torch.export.export_for_training(M(), (torch.randn(3, 2),))
         serialized = ExportedProgramSerializer().serialize(ep)
         for node in serialized.exported_program.graph_module.graph.nodes:
             if "aten.sum.dim_IntList" in node.target:
@@ -552,21 +628,24 @@ class TestDeserialize(TestCase):
 
         def _check_graph(pre_dispatch):
             if pre_dispatch:
+                ep = torch.export.export_for_training(
+                    fn,
+                    _deepcopy_inputs(inputs),
+                    {},
+                    dynamic_shapes=dynamic_shapes,
+                    strict=strict,
+                )
+            else:
+                # We should have this branch because
+                # PT2 Inference goes through this private
+                # export API.
                 ep = torch.export._trace._export(
-                    fn,
-                    _deepcopy_inputs(inputs),
-                    {},
-                    dynamic_shapes=dynamic_shapes,
-                    pre_dispatch=True,
-                    strict=strict,
-                )
-            else:
-                ep = torch.export.export(
                     fn,
                     _deepcopy_inputs(inputs),
                     {},
                     dynamic_shapes=dynamic_shapes,
                     strict=strict,
+                    pre_dispatch=False,
                 )
             ep.graph.eliminate_dead_code()
 
@@ -624,6 +703,20 @@ class TestDeserialize(TestCase):
 
             self.check_graph(M(), (torch.randn(3), torch.randn(3), torch.randn(3)))
 
+    def test_sym_bool_dynamic_shapes(self) -> None:
+        class MyModule(torch.nn.Module):
+            def __init__(self) -> None:
+                super().__init__()
+
+            def forward(self, x, y):
+                z = x[:, -y.shape[0] :, :]
+                return z
+
+        inputs = (torch.ones(4, 5, 10), torch.ones(3))
+        dynamic_shapes = {"x": {}, "y": {0: Dim("seqlen", max=4)}}
+        # Compile with dynamic_shapes set to get operator.neg involved
+        self.check_graph(MyModule(), inputs, dynamic_shapes=dynamic_shapes)
+
     def test_auto_functionalize(self):
         with torch.library._scoped_library("mylib", "FRAGMENT") as lib:
             torch.library.define(
@@ -738,6 +831,7 @@ class TestDeserialize(TestCase):
         dynamic_shapes = {"a": {0: dim0_ac}, "b": None, "c": {0: dim0_ac}}
         self.check_graph(DynamicShapeSimpleModel(), inputs, dynamic_shapes)
 
+    @unittest.expectedFailure  # T206587081
     def test_sym_bool(self):
         class Module(torch.nn.Module):
             def forward(self, x, y):
@@ -808,6 +902,26 @@ class TestDeserialize(TestCase):
 
         self.check_graph(M(), inputs)
 
+    def test_arg_from(self):
+        class M(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.register_buffer("compress_weight", torch.ones((10, 10)))
+                self.register_buffer("compress_bias", torch.ones(10))
+
+            def forward(self) -> None:
+                if self.compress_weight is None or self.compress_bias is None:
+                    return
+                torch.nn.init.kaiming_uniform_(self.compress_weight, a=math.sqrt(5))
+                fan_in, _ = torch.nn.init._calculate_fan_in_and_fan_out(
+                    self.compress_weight
+                )
+                bound = 1 / math.sqrt(fan_in) if fan_in > 0 else 0
+                torch.nn.init.uniform_(self.compress_bias, -bound, bound)
+
+        with torch.no_grad():
+            self.check_graph(M(), ())
+
     def test_map(self):
         from functorch.experimental import control_flow
 
@@ -882,7 +996,7 @@ class TestDeserialize(TestCase):
                 a = a * 2
                 return a, b
 
-        ep = torch.export.export(M(), (torch.ones(3),))
+        ep = torch.export.export_for_training(M(), (torch.ones(3),))
 
         # insert another getitem node
         for node in ep.graph.nodes:
@@ -1028,7 +1142,7 @@ def forward(self, x):
             def forward(self):
                 return self.p * self.p
 
-        ep = torch.export.export(M(), ())
+        ep = torch.export.export_for_training(M(), ())
         ep._example_inputs = None
         roundtrip_ep = deserialize(serialize(ep))
         self.assertTrue(torch.allclose(ep.module()(), roundtrip_ep.module()()))
@@ -1045,7 +1159,7 @@ class TestSchemaVersioning(TestCase):
                 return x + x
 
         f = Module()
-        ep = export(f, (torch.randn(1, 3),))
+        ep = export_for_training(f, (torch.randn(1, 3),))
 
         serialized_program = ExportedProgramSerializer().serialize(ep)
         serialized_program.exported_program.schema_version.major = -1
@@ -1081,7 +1195,7 @@ class TestSaveLoad(TestCase):
                 y = self.linear(y)
                 return y
 
-        ep = export(Module(), inp)
+        ep = export_for_training(Module(), inp)
 
         buffer = io.BytesIO()
         save(ep, buffer)
@@ -1098,7 +1212,7 @@ class TestSaveLoad(TestCase):
         f = Foo()
 
         inp = (torch.randn(2, 2),)
-        ep = export(f, inp)
+        ep = export_for_training(f, inp)
 
         with tempfile.NamedTemporaryFile() as f:
             save(ep, f)
@@ -1115,7 +1229,7 @@ class TestSaveLoad(TestCase):
         f = Foo()
 
         inp = (torch.tensor([6]), torch.tensor([7]))
-        ep = export(f, inp)
+        ep = export_for_training(f, inp)
 
         with TemporaryFileName() as fname:
             path = Path(fname)
@@ -1133,7 +1247,7 @@ class TestSaveLoad(TestCase):
 
         f = Foo()
 
-        ep = export(f, inp)
+        ep = export_for_training(f, inp)
 
         buffer = io.BytesIO()
         save(ep, buffer, extra_files={"extra.txt": "moo"})
@@ -1151,7 +1265,7 @@ class TestSaveLoad(TestCase):
 
         f = Foo()
 
-        ep = export(f, (torch.randn(1, 3),))
+        ep = export_for_training(f, (torch.randn(1, 3),))
 
         with tempfile.NamedTemporaryFile() as f:
             save(ep, f)
@@ -1177,7 +1291,7 @@ class TestSaveLoad(TestCase):
                 list_tensor = [torch.tensor(3), torch.tensor(4)]
                 return x + self.a + list_tensor[0] + list_tensor[1]
 
-        ep = export(Foo(), (torch.tensor(1),))
+        ep = export_for_training(Foo(), (torch.tensor(1),))
         buffer = io.BytesIO()
         save(ep, buffer)
         buffer.seek(0)
@@ -1203,7 +1317,7 @@ class TestSerializeCustomClass(TestCase):
         f = Foo()
 
         inputs = (torch.zeros(4, 4),)
-        ep = export(f, inputs)
+        ep = export_for_training(f, inputs)
 
         # Replace one of the values with an instance of our custom class
         for node in ep.graph.nodes:
@@ -1257,7 +1371,7 @@ class TestSerializeCustomClass(TestCase):
 
         inputs = (torch.zeros(2, 3),)
         with enable_torchbind_tracing():
-            ep = export(f, inputs, strict=False)
+            ep = export_for_training(f, inputs, strict=False)
 
         serialized_vals = serialize(ep)
         ep = deserialize(serialized_vals)
@@ -1271,7 +1385,7 @@ class TestSerializeCustomClass(TestCase):
         f = Foo()
 
         inputs = (torch.zeros(4, 4),)
-        ep = export(f, inputs)
+        ep = export_for_training(f, inputs)
 
         new_gm = copy.deepcopy(ep.graph_module)
         new_gm.meta["custom"] = {}
@@ -1306,7 +1420,7 @@ class TestSerializeCustomClass(TestCase):
         f = Foo()
 
         inputs = (torch.ones(2, 2),)
-        ep = export(f, inputs)
+        ep = export_for_training(f, inputs)
 
         new_gm = copy.deepcopy(ep.graph_module)
         new_gm.meta["custom"] = {}
@@ -1334,37 +1448,6 @@ class TestSerializeCustomClass(TestCase):
                 self.assertTrue(node.meta["custom"]["quantization_tag"] == "foo")
         self.assertTrue(counter > 1)
 
-    # TODO For some reason, this doesn't work on Windows ONLY.
-    # def test_custom_tag_metadata_reexport(self):
-    #     class Foo(torch.nn.Module):
-    #         def forward(self, x):
-    #             return x + x
-    #
-    #     f = Foo()
-    #
-    #     inputs = (torch.zeros(4, 4),)
-    #     ep = export(f, inputs)
-    #
-    #     new_gm = copy.deepcopy(ep.graph_module)
-    #     new_gm.meta["custom"] = {}
-    #     new_gm.meta["custom"]["f"] = "bar"
-    #
-    #     for node in new_gm.graph.nodes:
-    #         if node.op == "call_function" and node.target == torch.ops.aten.add.Tensor:
-    #             node.meta["custom"] = {}
-    #             node.meta["custom"]["quantization_tag"] = "foo"
-    #
-    #     new_ep = ep._update(new_gm, ep.graph_signature)
-    #     new_ep = torch.export.export(new_ep.module(), inputs)
-    #
-    #     self.assertEqual(new_ep.graph_module.meta["custom"]["f"], "bar")
-    #     counter = 0
-    #     for node in new_ep.graph.nodes:
-    #         if node.op == "call_function" and node.target == torch.ops.aten.add.Tensor:
-    #             counter += 1
-    #             self.assertTrue(node.meta["custom"]["quantization_tag"] == "foo")
-    #     self.assertEqual(counter, 1)
-
     def test_custom_tag_metadata_copy(self):
         class Foo(torch.nn.Module):
             def forward(self, x):
@@ -1373,7 +1456,7 @@ class TestSerializeCustomClass(TestCase):
         f = Foo()
 
         inputs = (torch.zeros(4, 4),)
-        ep = export(f, inputs)
+        ep = export_for_training(f, inputs)
 
         new_gm = copy.deepcopy(ep.graph_module)
         new_gm.meta["custom"] = {}
